/*==================================================================================================
*   Project              : RTD AUTOSAR 4.4
*   Platform             : CORTEXM
*   Peripheral           : FLEXIO
*   Dependencies         : 
*
*   Autosar Version      : 4.4.0
*   Autosar Revision     : ASR_REL_4_4_REV_0000
*   Autosar Conf.Variant :
*   SW Version           : 0.9.0
*   Build Version        : S32K3_RTD_0_9_0_P02_D2107_ASR_REL_4_4_REV_0000_20210716
*
*   (c) Copyright 2020 - 2021 NXP Semiconductors
*   All Rights Reserved.
*
*   NXP Confidential. This software is owned or controlled by NXP and may only be
*   used strictly in accordance with the applicable license terms. By expressly
*   accepting such terms or by downloading, installing, activating and/or otherwise
*   using the software, you are agreeing that you have read, and that you agree to
*   comply with and are bound by, such license terms. If you do not agree to be
*   bound by the applicable license terms, then you may not retain, install,
*   activate or otherwise use the software.
==================================================================================================*/

#ifndef FLEXIO_UART_IP_H
#define FLEXIO_UART_IP_H

/**
*   @file
*   @defgroup flexio_uart_ip Flexio UART IPL
*   @addtogroup  flexio_uart_ip Flexio UART IPL
*   @{
*/

#ifdef __cplusplus
extern "C"{
#endif

/*==================================================================================================
*                                          INCLUDE FILES
* 1) system and project includes
* 2) needed interfaces from external units
* 3) internal and external interfaces from this unit
==================================================================================================*/
#include "Flexio_Uart_Ip_Cfg.h"
#include "Mcal.h"
/*==================================================================================================
*                                 SOURCE FILE VERSION INFORMATION
==================================================================================================*/
#define FLEXIO_UART_IP_VENDOR_ID                    43
#define FLEXIO_UART_IP_AR_RELEASE_MAJOR_VERSION     4
#define FLEXIO_UART_IP_AR_RELEASE_MINOR_VERSION     4
#define FLEXIO_UART_IP_AR_RELEASE_REVISION_VERSION  0
#define FLEXIO_UART_IP_SW_MAJOR_VERSION             0
#define FLEXIO_UART_IP_SW_MINOR_VERSION             9
#define FLEXIO_UART_IP_SW_PATCH_VERSION             0
/*==================================================================================================
*                                       FILE VERSION CHECKS
==================================================================================================*/

/* Checks against Flexio_Uart_Ip_Cfg.h */
#if (FLEXIO_UART_IP_VENDOR_ID != FLEXIO_UART_IP_CFG_VENDOR_ID)
    #error "Flexio_Uart_Ip.h and Flexio_Uart_Ip_Cfg.h have different vendor ids"
#endif
#if ((FLEXIO_UART_IP_AR_RELEASE_MAJOR_VERSION    != FLEXIO_UART_IP_CFG_AR_RELEASE_MAJOR_VERSION) || \
     (FLEXIO_UART_IP_AR_RELEASE_MINOR_VERSION    != FLEXIO_UART_IP_CFG_AR_RELEASE_MINOR_VERSION) || \
     (FLEXIO_UART_IP_AR_RELEASE_REVISION_VERSION != FLEXIO_UART_IP_CFG_AR_RELEASE_REVISION_VERSION))
     #error "AUTOSAR Version Numbers of Flexio_Uart_Ip.h and Flexio_Uart_Ip_Cfg.h are different"
#endif
#if ((FLEXIO_UART_IP_SW_MAJOR_VERSION != FLEXIO_UART_IP_CFG_SW_MAJOR_VERSION) || \
     (FLEXIO_UART_IP_SW_MINOR_VERSION != FLEXIO_UART_IP_CFG_SW_MINOR_VERSION) || \
     (FLEXIO_UART_IP_SW_PATCH_VERSION != FLEXIO_UART_IP_CFG_SW_PATCH_VERSION))
    #error "Software Version Numbers of Flexio_Uart_Ip.h and Flexio_Uart_Ip_Cfg.h are different"
#endif
#if (FLEXIO_UART_IP_IS_USING == STD_ON)
/*==================================================================================================
*                                            CONSTANTS
==================================================================================================*/
#define UART_START_SEC_CONFIG_DATA_UNSPECIFIED
/* @violates @ref Uart_c_REF_1 This violation is not fixed since the inclusion of Uart_MemMap.h is as per AUTOSAR requirement*/
#include "Uart_MemMap.h"
/* Calling the external Configuration symbols defined by FlexIO_Uart_Ip_Cfg.h */
FLEXIO_UART_IP_CONFIG_EXT
#define UART_STOP_SEC_CONFIG_DATA_UNSPECIFIED
/* @violates @ref Uart_c_REF_1 This violation is not fixed since the inclusion of Uart_MemMap.h is as per AUTOSAR requirement*/
#include "Uart_MemMap.h"
/*==================================================================================================
*                                       DEFINES AND MACROS
==================================================================================================*/

/*==================================================================================================
*                                              ENUMS
==================================================================================================*/

/*==================================================================================================
*                                  STRUCTURES AND OTHER TYPEDEFS
==================================================================================================*/

/*==================================================================================================
*                                  GLOBAL VARIABLE DECLARATIONS
==================================================================================================*/

/*==================================================================================================
*                                       FUNCTION PROTOTYPES
==================================================================================================*/
#define UART_START_SEC_CODE
/* @violates @ref Uart_c_REF_1 This violation is not fixed since the inclusion of Uart_MemMap.h is as per AUTOSAR requirement*/
#include "Uart_MemMap.h"

/*!
 * @brief Initializes an Flexio UART operation channel.
 *
 * The caller provides memory for the driver state structures during initialization.
 * The user must select the Flexio UART clock source in the application to initialize the Flexio UART.
 *
 * @param u32Channel  Flexio UART channel number
 * @param pUserConfig user configuration structure of type #Lpuart_Uart_Ip_UserConfigType
 * @return void
 */
void Flexio_Uart_Ip_Init(uint32 u32Channel,
                         const Flexio_Uart_Ip_UserConfigType * pUserConfig);

/*!
 * @brief Shuts down the Flexio UART by disabling interrupts and transmitter/receiver.
 *
 * @param u32Channel  Flexio UART channel number
 * @return void
 */
void Flexio_Uart_Ip_Deinit(uint32 u32Channel);

/*!
 * @brief Returns the Flexio UART baud rate.
 *
 * This function returns the Flexio UART configured baud rate.
 *
 * @param u32Channel  Flexio UART channel number.
 * @param[out] pConfiguredBaudRate Flexio UART configured baud rate.
 */
void Flexio_Uart_Ip_GetBaudRate(uint32 u32Channel, uint32 * pConfiguredBaudRate);

/*!
 * @brief Configures the Flexio Uart baud rate.
 *
 * This function configures the Flexio Uart baud rate.
 * In some Flexio Uart channels the user must disable the transmitter/receiver
 * before calling this function.
 * Generally, this may be applied to all Flexio Uarts to ensure safe operation.
 *
 * @param u32Channel  Flexio Uart channel number.
 * @param u32DesiredBaudrate Flexio Uart desired baud rate.
 * @param u32ClockFrequency Clock Frequency of Flexio Uart channel.
 * @return FLEXIO_UART_IP_STATUS_BUSY if called during an on-going transfer, FLEXIO_UART_IP_STATUS_SUCCESS otherwise
 */
Flexio_Uart_Ip_StatusType Flexio_Uart_Ip_SetBaudRate(uint32 u32Channel,
                                                     Flexio_Uart_Ip_BaudrateType u32DesiredBaudrate,
                                                     uint32 u32ClockFrequency);

/*!
 * @brief Perform a Asynchronous UART transmission
 *
 * This function sends a block of data and returns immediately.
 * The rest of the transmission is handled by the interrupt service routine (if the driver
 * is initialized in interrupt mode)
 *
 * @param u32Channel    FLEXIO peripheral channel number
 * @param pTxBuff    pointer to the data to be transferred
 * @param u32TxSize    length in bytes of the data to be transferred
 * @return    Error or success status returned by API
 */
Flexio_Uart_Ip_StatusType Flexio_Uart_Ip_AsyncSend(uint32 u32Channel,
                                                  const uint8 * pTxBuff,
                                                  uint32 u32TxSize);

/*!
 * @brief Transfer multiple bytes of data using polling method.
 * @param u32Channel    FLEXIO peripheral channel number
 * @param pTxBuff    pointer to the data to be transferred
 * @param u32TxSize    length in bytes of the data to be transferred
 * @return    Error or success status returned by API
 */
Flexio_Uart_Ip_StatusType Flexio_Uart_Ip_SyncSend(uint32 u32Channel,
                                                  const uint8 * pTxBuff,
                                                  uint32 u32TxSize);

/*!
 * @brief Perform a Asynchronous UART reception
 *
 * This function receives a block of data and returns immediately.
 * The rest of the transmission is handled by the interrupt service routine (if the driver
 * is initialized in interrupt mode) or by the Flexio_Uart_Ip_GetStatus() function (if
 * the driver is initialized in polling mode).
 *
 * @param u32Channel    FLEXIO peripheral channel number
 * @param pRxBuff    pointer to the receive buffer
 * @param u32RxSize    length in bytes of the data to be received
 * @return    Error or success status returned by API
 */
Flexio_Uart_Ip_StatusType Flexio_Uart_Ip_AsyncReceive(uint32 u32Channel,
                                                      uint8 * pRxBuff,
                                                      uint32 u32RxSize);

/*!
 * @brief Receive multiple bytes of data using polling method.
 *
 * This function receives a block of data and returns immediately.
 * The rest of the transmission is handled by the Flexio_Uart_Ip_GetStatus() function to initialized in polling mode.
 *
 * @param u32Channel    FLEXIO peripheral channel number
 * @param pRxBuff    pointer to the receive buffer
 * @param u32RxSize    length in bytes of the data to be received
 * @return    Error or success status returned by API
 */
Flexio_Uart_Ip_StatusType Flexio_Uart_Ip_SyncReceive(uint32 u32Channel,
                                                     uint8 * pRxBuff,
                                                     uint32 u32RxSize);

/*FUNCTION**********************************************************************
 *
 * Function Name : Flexio_Uart_Ip_GetStatus
 * Description   : Get the status of the current non-blocking UART transaction
 * Implements : Flexio_Uart_Ip_GetStatus_Activity
 *
 *END**************************************************************************/
Flexio_Uart_Ip_StatusType Flexio_Uart_Ip_GetStatus(uint32 u32Channel, uint32 *pBytesRemaining);

/*FUNCTION**********************************************************************
 *
 * Function Name : Flexio_Uart_Ip_CompleteSendUsingDma
 * Description   : Finish up a transmit by completing the process of sending
 * data and disabling the DMA requests. This is a callback for DMA major loop
 * completion, so it must match the DMA callback signature.
 *
 *END**************************************************************************/
void Flexio_Uart_Ip_CompleteSendUsingDma(uint32 u32Channel);

/**
 * @internal
 * @brief   : Finish up a receive by completing the process of receiving data
 * and disabling the DMA requests. This is a part of callback for DMA major loop
 * completion, so it must match the DMA callback signature.
 * @param u32Channel Flexio channel number
 * @return void
 */
void Flexio_Uart_Ip_CompleteReceiveUsingDma(uint32 u32Channel);

/*FUNCTION**********************************************************************
 *
 * Function Name : Flexio_Uart_Ip_AbortTransferData
 * Description   : This function terminates an non-blocking FLEXIO transmission
 * early. During a non-blocking FLEXIO transmission, the user has the option to
 * terminate the transmission early if the transmission is still in progress.
 *
 * Implements    : Flexio_Uart_Ip_AbortTransferData_Activity
 *END**************************************************************************/
Flexio_Uart_Ip_StatusType Flexio_Uart_Ip_AbortTransferData(uint32 u32Channel);

/*!
 * @brief Sets the internal driver reference to the tx buffer.
 *
 * This function can be called from the tx callback to provide the driver
 * with a new buffer, for continuous transmission.
 *
 * @param u32Channel  Flexio Uart channel number
 * @param pTxData  source buffer containing 8-bit data chars to send
 * @param u32TxSize  the number of bytes to send
 * @return FLEXIO_UART_IP_STATUS_SUCCESS
 */
Flexio_Uart_Ip_StatusType Flexio_Uart_Ip_SetTxBuffer(uint32 u32Channel,
                                                     const uint8 * pTxData,
                                                     uint32 u32TxSize);

/*!
 * @brief Sets the internal driver reference to the rx buffer.
 *
 * This function can be called from the rx callback to provide the driver
 * with a new buffer, for continuous reception.
 *
 * @param u32Channel  FLEXIO channel number
 * @param pRxData  destination buffer containing 8-bit data chars to receive
 * @param u32RxSize  the number of bytes to receive
 * @return FLEXIO_UART_IP_STATUS_SUCCESS
 */
Flexio_Uart_Ip_StatusType Flexio_Uart_Ip_SetRxBuffer(uint32 u32Channel,
                                                     uint8 * pRxData,
                                                     uint32 u32RxSize);


#define UART_STOP_SEC_CODE
/* @violates @ref Uart_c_REF_1 This violation is not fixed since the inclusion of Uart_MemMap.h is as per AUTOSAR requirement*/
#include "Uart_MemMap.h"
#endif /* (FLEXIO_UART_IP_IS_USING == STD_ON) */
#if defined(__cplusplus)
}
#endif

/*! @}*/

#endif /* FLEXIO_UART_IP_H */
