/*==================================================================================================
*   Project              : RTD AUTOSAR 4.4
*   Platform             : CORTEXM
*   Peripheral           : FLEXIO
*   Dependencies         : 
*
*   Autosar Version      : 4.4.0
*   Autosar Revision     : ASR_REL_4_4_REV_0000
*   Autosar Conf.Variant :
*   SW Version           : 0.9.0
*   Build Version        : S32K3_RTD_0_9_0_P02_D2107_ASR_REL_4_4_REV_0000_20210716
*
*   (c) Copyright 2020 - 2021 NXP Semiconductors
*   All Rights Reserved.
*
*   NXP Confidential. This software is owned or controlled by NXP and may only be
*   used strictly in accordance with the applicable license terms. By expressly
*   accepting such terms or by downloading, installing, activating and/or otherwise
*   using the software, you are agreeing that you have read, and that you agree to
*   comply with and are bound by, such license terms. If you do not agree to be
*   bound by the applicable license terms, then you may not retain, install,
*   activate or otherwise use the software.
==================================================================================================*/

#ifndef FLEXIO_UART_IP_HWACCESS_H__
#define FLEXIO_UART_IP_HWACCESS_H__

/**
*   @file
*   @internal
*   @defgroup flexio_ip Flexio IPL
*   @addtogroup  flexio_ip Flexio  IPL
*   @{
*/

#ifdef __cplusplus
extern "C"{
#endif

/*==================================================================================================
*                                          INCLUDE FILES
* 1) system and project includes
* 2) needed interfaces from external units
* 3) internal and external interfaces from this unit
==================================================================================================*/
#include "StandardTypes.h"
#include "S32K344_FLEXIO.h"
#include "Flexio_Uart_Ip_Types.h"
#include "OsIf.h"
#ifdef FLEXIO_UART_IP_IS_USING
    #if (FLEXIO_UART_IP_IS_USING == STD_ON)
        #include "Flexio_Mcl_Ip_HwAccess.h"
    #endif /* (FLEXIO_UART_IP_IS_USING == STD_ON) */
#endif /* #ifdef FLEXIO_UART_IP_IS_USING */
/*==================================================================================================
*                                 SOURCE FILE VERSION INFORMATION
==================================================================================================*/
#define FLEXIO_UART_IP_HWACCESS_VENDOR_ID                    43
#define FLEXIO_UART_IP_HWACCESS_AR_RELEASE_MAJOR_VERSION     4
#define FLEXIO_UART_IP_HWACCESS_AR_RELEASE_MINOR_VERSION     4
#define FLEXIO_UART_IP_HWACCESS_AR_RELEASE_REVISION_VERSION  0
#define FLEXIO_UART_IP_HWACCESS_SW_MAJOR_VERSION             0
#define FLEXIO_UART_IP_HWACCESS_SW_MINOR_VERSION             9
#define FLEXIO_UART_IP_HWACCESS_SW_PATCH_VERSION             0

/* Checks against Flexio_Uart_Ip_Types.h */
#if (FLEXIO_UART_IP_HWACCESS_VENDOR_ID != FLEXIO_UART_IP_TYPES_VENDOR_ID)
    #error "Flexio_Uart_Ip_HwAccess.h and Flexio_Uart_Ip_Types.h have different vendor ids"
#endif
#if ((FLEXIO_UART_IP_HWACCESS_AR_RELEASE_MAJOR_VERSION    != FLEXIO_UART_IP_TYPES_AR_RELEASE_MAJOR_VERSION) || \
     (FLEXIO_UART_IP_HWACCESS_AR_RELEASE_MINOR_VERSION    != FLEXIO_UART_IP_TYPES_AR_RELEASE_MINOR_VERSION) || \
     (FLEXIO_UART_IP_HWACCESS_AR_RELEASE_REVISION_VERSION != FLEXIO_UART_IP_TYPES_AR_RELEASE_REVISION_VERSION))
     #error "AUTOSAR Version Numbers of Flexio_Uart_Ip_HwAccess.h and Flexio_Uart_Ip_Types.h are different"
#endif
#if ((FLEXIO_UART_IP_HWACCESS_SW_MAJOR_VERSION != FLEXIO_UART_IP_TYPES_SW_MAJOR_VERSION) || \
     (FLEXIO_UART_IP_HWACCESS_SW_MINOR_VERSION != FLEXIO_UART_IP_TYPES_SW_MINOR_VERSION) || \
     (FLEXIO_UART_IP_HWACCESS_SW_PATCH_VERSION != FLEXIO_UART_IP_TYPES_SW_PATCH_VERSION))
    #error "Software Version Numbers of Flexio_Uart_Ip_HwAccess.h and Flexio_Uart_Ip_Types.h are different"
#endif

#ifndef DISABLE_MCAL_INTERMODULE_ASR_CHECK
/* Check if current file and StandardTypes.h header file are of the same Autosar version */
    #if ((FLEXIO_UART_IP_HWACCESS_AR_RELEASE_MAJOR_VERSION != STD_AR_RELEASE_MAJOR_VERSION) || \
            (FLEXIO_UART_IP_HWACCESS_AR_RELEASE_MINOR_VERSION != STD_AR_RELEASE_MINOR_VERSION))
        #error "Flexio_Uart_Ip_HwAccess.h and StandardTypes.h are different"
    #endif
#ifdef FLEXIO_UART_IP_IS_USING
    #if (FLEXIO_UART_IP_IS_USING == STD_ON)
        /* Checks against Flexio_Mcl_Ip_HwAccess.h */
        #if ((FLEXIO_UART_IP_HWACCESS_AR_RELEASE_MAJOR_VERSION    != FLEXIO_IP_HW_ACCESS_AR_RELEASE_MAJOR_VERSION_H) || \
            (FLEXIO_UART_IP_HWACCESS_AR_RELEASE_MINOR_VERSION    != FLEXIO_IP_HW_ACCESS_AR_RELEASE_MINOR_VERSION_H))
            #error "AUTOSAR Version Numbers of Flexio_Uart_Ip_HwAccess.h and Flexio_Mcl_Ip_HwAccess.h are different"
        #endif
    #endif /* (FLEXIO_UART_IP_IS_USING == STD_ON) */
#endif /* #ifdef FLEXIO_UART_IP_IS_USING */
#endif

#ifdef FLEXIO_UART_IP_IS_USING
#if (FLEXIO_UART_IP_IS_USING == STD_ON)
 /*******************************************************************************
 * Definitions
 ******************************************************************************/
/* Wrap-around value for timeout */
#define TIMEOUT_WRAP     (0xFFFFFFU)

/*******************************************************************************
 * API
 ******************************************************************************/

#define UART_START_SEC_CODE
/* @violates @ref Uart_c_REF_1 This violation is not fixed since the inclusion of Uart_MemMap.h is as per AUTOSAR requirement*/
#include "Uart_MemMap.h"

/*
 * This function sets the following configurations for the specified shifter:
 * start bit, stop bit, input source
 */
static inline void Flexio_Uart_Ip_SetShifterConfig(FLEXIO_Type * pBase,
                                               uint32 shifter,
                                               Flexio_Mcl_Ip_ShifterStartType start,
                                               Flexio_Mcl_Ip_ShifterStopType stop,
                                               Flexio_Mcl_Ip_ShifterSourceType source)
{
    pBase->SHIFTCFG[shifter] = FLEXIO_SHIFTCFG_SSTART(start)
                                | FLEXIO_SHIFTCFG_SSTOP(stop)
                                | FLEXIO_SHIFTCFG_INSRC(source);
}

/*
 * This function configures the control settings for the specified shifter: mode settings,
 * pin settings and timer settings.
 */
static inline void Flexio_Uart_Ip_SetShifterControl(FLEXIO_Type * pBase,
                                                uint32 shifter,
                                                Flexio_Mcl_Ip_ShifterModeType mode,
                                                uint8 pin,
                                                Flexio_Mcl_Ip_PinPolarityType pinPolarity,
                                                Flexio_Mcl_Ip_PinConfigType pinConfig,
                                                uint32 timer,
                                                Flexio_Mcl_Ip_TimerPolarityType timerPolarity)
{
    pBase->SHIFTCTL[shifter] = FLEXIO_SHIFTCTL_SMOD(mode)
                                | FLEXIO_SHIFTCTL_PINPOL(pinPolarity)
                                | FLEXIO_SHIFTCTL_PINSEL(pin)
                                | FLEXIO_SHIFTCTL_PINCFG(pinConfig)
                                | FLEXIO_SHIFTCTL_TIMPOL(timerPolarity)
                                | FLEXIO_SHIFTCTL_TIMSEL(timer);
}

/*
 * This function configures the compare value for the specified timer.
 * The timer compare value is loaded into the timer counter when the timer is first enabled,
 * when the timer is reset and when the timer decrements down to zero.
 * In dual 8-bit counters baud/bit mode, the lower 8-bits configure the baud rate divider
 * and the upper 8-bits configure the number of bits in each word.
 * In dual 8-bit counters PWM mode, the lower 8-bits configure the high period of the output
 * and the upper 8-bits configure the low period.
 * In 16-bit counter mode, the compare value can be used to generate the baud rate divider
 * (if shift clock source is timer output) or the number of bits in each word (when the shift
 * clock source is a pin or trigger input).
 */
static inline void Flexio_Uart_Ip_SetTimerCompare(FLEXIO_Type * pBase,
                                             uint32 timer,
                                             uint16 value)
{
    pBase->TIMCMP[timer] = (uint32)value;
}

/*
 * This function sets the following configurations for the specified timer:
 * start bit, stop bit, enable condition, disable condition, reset condition,
 * decrement condition, initial output
 */
static inline void Flexio_Uart_Ip_SetTimerConfig(FLEXIO_Type * pBase,
                                                uint32 timer,
                                                Flexio_Mcl_Ip_TimerStartType start,
                                                Flexio_Mcl_Ip_TimerStopType stop,
                                                Flexio_Mcl_Ip_TimerEnableType enable,
                                                Flexio_Mcl_Ip_TimerDisableType disable,
                                                Flexio_Mcl_Ip_TimerResetType reset,
                                                Flexio_Uart_Ip_TimerDecrementType decrement,
                                                Flexio_Mcl_Ip_TimerOutputType output)
{

    pBase->TIMCFG[timer] = FLEXIO_TIMCFG_TSTART(start)
                            | FLEXIO_TIMCFG_TSTOP(stop)
                            | FLEXIO_TIMCFG_TIMENA(enable)
                            | FLEXIO_TIMCFG_TIMDIS(disable)
                            | FLEXIO_TIMCFG_TIMRST(reset)
                            | FLEXIO_TIMCFG_TIMDEC(decrement)
                            | FLEXIO_TIMCFG_TIMOUT(output);
}

/*
 * This function configures the control settings for the specified timer: mode settings,
 * pin settings and trigger settings.
 */
static inline void Flexio_Uart_Ip_SetTimerControl(FLEXIO_Type * pBase,
                                                uint32 timer,
                                                uint8 trigger,
                                                Flexio_Mcl_Ip_TriggerPolarityType triggerPolarity,
                                                Flexio_Mcl_Ip_TriggerSourceType triggerSource,
                                                uint8 pin,
                                                Flexio_Mcl_Ip_PinPolarityType pinPolarity,
                                                Flexio_Mcl_Ip_PinConfigType pinConfig,
                                                Flexio_Mcl_Ip_TimerModeType mode)
{
    pBase->TIMCTL[timer] = FLEXIO_TIMCTL_TIMOD(mode)
                            | FLEXIO_TIMCTL_PINSEL(pin)
                            | FLEXIO_TIMCTL_PINPOL(pinPolarity)
                            | FLEXIO_TIMCTL_PINCFG(pinConfig)
                            | FLEXIO_TIMCTL_TRGSRC(triggerSource)
                            | FLEXIO_TIMCTL_TRGPOL(triggerPolarity)
                            | FLEXIO_TIMCTL_TRGSEL(trigger);
}

/*
 * This function configures the mode for the specified timer.
 * In 8-bit counter mode, the lower 8-bits of the counter and compare register are used to
 * configure the baud rate of the timer shift clock and the upper 8-bits are used to configure
 * the shifter bit count.
 * In 8-bit PWM mode, the lower 8-bits of the counter and compare
 * register are used to configure the high period of the timer shift clock and the upper
 * 8-bits are used to configure the low period of the timer shift clock. The shifter bit count
 * is configured using another timer or external signal.
 * In 16-bit counter mode, the full 16-bits of the counter and compare register are used to
 * configure either the baud rate of the shift clock or the shifter bit count.
 */
static inline void Flexio_Uart_Ip_SetTimerMode(FLEXIO_Type * pBase,
                                                uint32 timer,
                                                Flexio_Mcl_Ip_TimerModeType mode)
{
    pBase->TIMCTL[timer] = (pBase->TIMCTL[timer] & ~(FLEXIO_TIMCTL_TIMOD_MASK)) | FLEXIO_TIMCTL_TIMOD(mode);
}

/*
 * This function configures the mode for the specified shifter.
 */
static inline void Flexio_Uart_Ip_SetShifterMode(FLEXIO_Type * pBase,
                                             uint32 shifter,
                                             Flexio_Mcl_Ip_ShifterModeType mode)
{
    pBase->SHIFTCTL[shifter] = (pBase->SHIFTCTL[shifter] & ~(FLEXIO_SHIFTCTL_SMOD_MASK)) | FLEXIO_SHIFTCTL_SMOD(mode);
}

/*
 * This function configures the sending or receiving of
 * a start bit in Transmit, Receive or Match Store modes.
 */
static inline void Flexio_Uart_Ip_SetShifterStartBit(FLEXIO_Type * pBase,
                                                 uint32 shifter,
                                                 Flexio_Mcl_Ip_ShifterStartType start)
{
    pBase->SHIFTCFG[shifter] = (pBase->SHIFTCFG[shifter] & ~(FLEXIO_SHIFTCFG_SSTART_MASK)) | FLEXIO_SHIFTCFG_SSTART(start);
}

/*
 * This function writes data in the specified shifter buffer. The data can be
 * written in any of the four ways allowed by the hardware - see description of type
 * Flexio_Mcl_Ip_ShifterBufferModeType for details.
 */
static inline void Flexio_Uart_Ip_WriteShifterBuffer(FLEXIO_Type * pBase,
                                                 uint32 shifter,
                                                 uint32 value,
                                                 Flexio_Mcl_Ip_ShifterBufferModeType mode)
{
    switch (mode)
    {
        case FLEXIO_SHIFTER_RW_MODE_NORMAL:
            pBase->SHIFTBUF[shifter] = value;
            break;
        case FLEXIO_SHIFTER_RW_MODE_BIT_SWAP:
            pBase->SHIFTBUFBIS[shifter] = value;
            break;
        default:
            /* bad write mode */
            break;
    }
}

/*
 * This function reads data from the specified shifter buffer. The data can be
 * read in any of the four ways allowed by the hardware - see description of type
 * Flexio_Mcl_Ip_ShifterBufferModeType for details.
 */
static inline uint32 Flexio_Uart_Ip_ReadShifterBuffer(const FLEXIO_Type * pBase,
                                                    uint32 shifter,
                                                    Flexio_Mcl_Ip_ShifterBufferModeType mode)
{
    uint32 data;

    switch (mode)
    {
        case FLEXIO_SHIFTER_RW_MODE_NORMAL:
            data = pBase->SHIFTBUF[shifter];
            break;
        case FLEXIO_SHIFTER_RW_MODE_BIT_SWAP:
            data = pBase->SHIFTBUFBIS[shifter];
            break;
        default:
            /* bad read mode */
            data = 0U;
            break;
    }
    return data;
}

/*
 * Returns the state of the interrupt for corresponding timer.
 */
static inline boolean Flexio_Uart_Ip_GetTimerInterrupt(const FLEXIO_Type * pBase, uint32 timer)
{
    uint32 regValue = (uint32)pBase->TIMIEN;
    regValue = (regValue >> timer) & 1U;
    return (boolean)regValue;
}

/*
 * Returns the state of the error interrupt for corresponding shifter.
 */
static inline boolean Flexio_Uart_Ip_GetShifterErrorInterrupt(const FLEXIO_Type * pBase, uint32 shifter)
{
    uint32 regValue = (uint32)pBase->SHIFTEIEN;
    regValue = (regValue >> shifter) & 1U;
    return (boolean)regValue;
}

/*
 * Returns the state of the interrupt for corresponding shifter.
 */
static inline boolean Flexio_Uart_Ip_GetShifterInterrupt(const FLEXIO_Type * pBase, uint32 shifter)
{
    uint32 regValue = (uint32)pBase->SHIFTSIEN;
    regValue = (regValue >> shifter) & 1U;
    return (boolean)regValue;
}


/**
 * @brief   : Checks for timeout condition
 * @internal
 * @return  TRUE     Timeout occurs
 *          FALSE    Timeout does not occur
 */
static inline boolean Flexio_Uart_Ip_CheckTimeout(uint32 startTime, uint32 timeoutUs)
{
    uint32 timeoutTicks;
    uint32 currentTime;
    boolean retVal;
    timeoutTicks = OsIf_MicrosToTicks(timeoutUs,FLEXIO_UART_IP_TIMEOUT_TYPE);
    currentTime = OsIf_GetCounter(FLEXIO_UART_IP_TIMEOUT_TYPE);

    if (currentTime <= startTime)
    {
        retVal = ((startTime - currentTime) > timeoutTicks) ? TRUE : FALSE;
    }
    else
    {
        /* wrap around */
        retVal = ((TIMEOUT_WRAP + startTime - currentTime) > timeoutTicks) ? TRUE : FALSE;
    }
    return retVal;
}

#define UART_STOP_SEC_CODE
/* @violates @ref Uart_c_REF_1 This violation is not fixed since the inclusion of Uart_MemMap.h is as per AUTOSAR requirement*/
#include "Uart_MemMap.h"

#endif /* (FLEXIO_UART_IP_IS_USING == STD_ON) */
#endif /* #ifdef FLEXIO_UART_IP_IS_USING */

#if defined(__cplusplus)
}
#endif

/*! @}*/

#endif /* FLEXIO_UART_IP_HWACCESS_H__ */
