/*==================================================================================================
*   Project              : RTD AUTOSAR 4.4
*   Platform             : CORTEXM
*   Peripheral           : 
*   Dependencies         : none
*
*   Autosar Version      : 4.4.0
*   Autosar Revision     : ASR_REL_4_4_REV_0000
*   Autosar Conf.Variant :
*   SW Version           : 0.9.0
*   Build Version        : S32K3_RTD_0_9_0_P02_D2107_ASR_REL_4_4_REV_0000_20210716
*
*   (c) Copyright 2020 - 2021 NXP Semiconductors
*   All Rights Reserved.
*
*   NXP Confidential. This software is owned or controlled by NXP and may only be
*   used strictly in accordance with the applicable license terms. By expressly
*   accepting such terms or by downloading, installing, activating and/or otherwise
*   using the software, you are agreeing that you have read, and that you agree to
*   comply with and are bound by, such license terms. If you do not agree to be
*   bound by the applicable license terms, then you may not retain, install,
*   activate or otherwise use the software.
==================================================================================================*/
/**
*   @file       MC_RGM.h
*   @version    0.9.0
*
*   @brief   POWER IP driver header file.
*   @details POWER IP driver header file.
*
*   @addtogroup POWER_DRIVER Power Ip Driver
*   @{
*/


#ifndef MC_RGM_H
#define MC_RGM_H


#ifdef __cplusplus
extern "C"{
#endif



/*==================================================================================================
                                         INCLUDE FILES
 1) system and project includes
 2) needed interfaces from external units
 3) internal and external interfaces from this unit
==================================================================================================*/
#include "Power_Ip_Types.h"
#include "StandardTypes.h"
#include "Mcal.h"

/*==================================================================================================
                               SOURCE FILE VERSION INFORMATION
==================================================================================================*/
#define MC_RGM_VENDOR_ID                       43
#define MC_RGM_AR_RELEASE_MAJOR_VERSION        4
#define MC_RGM_AR_RELEASE_MINOR_VERSION        4
#define MC_RGM_AR_RELEASE_REVISION_VERSION     0
#define MC_RGM_SW_MAJOR_VERSION                0
#define MC_RGM_SW_MINOR_VERSION                9
#define MC_RGM_SW_PATCH_VERSION                0

/*==================================================================================================
                                      FILE VERSION CHECKS
==================================================================================================*/
#ifndef DISABLE_MCAL_INTERMODULE_ASR_CHECK
/* Check if MC_RGM.h file and StandardTypes.h file are of the same Autosar version */
#if ((MC_RGM_AR_RELEASE_MAJOR_VERSION    != STD_AR_RELEASE_MAJOR_VERSION) || \
     (MC_RGM_AR_RELEASE_MINOR_VERSION    != STD_AR_RELEASE_MINOR_VERSION))
    #error "AutoSar Version Numbers of MC_RGM.h and StandardTypes.h are different"
#endif

/* Check if MC_RGM.h file and Mcal.h file are of the same Autosar version */
#if ((MC_RGM_AR_RELEASE_MAJOR_VERSION    != MCAL_AR_RELEASE_MAJOR_VERSION) || \
     (MC_RGM_AR_RELEASE_MINOR_VERSION    != MCAL_AR_RELEASE_MINOR_VERSION))
    #error "AutoSar Version Numbers of MC_RGM.h and Mcal.h are different"
#endif
#endif

/* Check if MC_RGM.h file and Power_Ip_Types.h file have same versions */
#if (MC_RGM_VENDOR_ID  != POWER_IP_TYPES_VENDOR_ID)
    #error "MC_RGM.h and Power_Ip_Types.h have different vendor IDs"
#endif

/* Check if MC_RGM.h file and Power_Ip_Types.h file are of the same Autosar version */
#if ((MC_RGM_AR_RELEASE_MAJOR_VERSION    != POWER_IP_TYPES_AR_RELEASE_MAJOR_VERSION) || \
     (MC_RGM_AR_RELEASE_MINOR_VERSION    != POWER_IP_TYPES_AR_RELEASE_MINOR_VERSION) || \
     (MC_RGM_AR_RELEASE_REVISION_VERSION != POWER_IP_TYPES_AR_RELEASE_REVISION_VERSION))
    #error "AutoSar Version Numbers of MC_RGM.h and Power_Ip_Types.h are different"
#endif

/* Check if MC_RGM.h file and Power_Ip_Types.h file are of the same Software version */
#if ((MC_RGM_SW_MAJOR_VERSION != POWER_IP_TYPES_SW_MAJOR_VERSION) || \
     (MC_RGM_SW_MINOR_VERSION != POWER_IP_TYPES_SW_MINOR_VERSION) || \
     (MC_RGM_SW_PATCH_VERSION != POWER_IP_TYPES_SW_PATCH_VERSION))
    #error "Software Version Numbers of MC_RGM.h and Power_Ip_Types.h are different"
#endif
/*==================================================================================================
*                                          CONSTANTS
==================================================================================================*/


/*==================================================================================================
                                       DEFINES AND MACROS
==================================================================================================*/

/*==================================================================================================
*                                             ENUMS
==================================================================================================*/


/*==================================================================================================
*                                STRUCTURES AND OTHER TYPEDEFS
==================================================================================================*/


/*==================================================================================================
*                                GLOBAL VARIABLE DECLARATIONS
==================================================================================================*/


/*==================================================================================================
*                                    FUNCTION PROTOTYPES
==================================================================================================*/
#define MCU_START_SEC_CODE

#include "Mcu_MemMap.h"


#if (defined(POWER_IP_ENABLE_USER_MODE_SUPPORT))
  #if (STD_ON == POWER_IP_ENABLE_USER_MODE_SUPPORT)
    #if (defined(MCAL_MC_RGM_REG_PROT_AVAILABLE))
      #if (STD_ON == MCAL_MC_RGM_REG_PROT_AVAILABLE)
void MC_RGM_SetUserAccessAllowed(void);
      #endif /* (STD_ON == MCAL_MC_RGM_REG_PROT_AVAILABLE) */
    #endif
    #if (defined(MCAL_RDC_REG_PROT_AVAILABLE))
      #if (STD_ON == MCAL_RDC_REG_PROT_AVAILABLE)
void RDC_SetUserAccessAllowed(void);
      #endif /* (STD_ON == MCAL_RDC_REG_PROT_AVAILABLE) */
    #endif
  #endif
#endif /* (STD_ON == POWER_IP_ENABLE_USER_MODE_SUPPORT) */


#ifdef POWER_IP_ENABLE_USER_MODE_SUPPORT
  #if (STD_ON == POWER_IP_ENABLE_USER_MODE_SUPPORT)

#define  Call_MC_RGM_ResetInit(MC_RGM_pConfigPtr) \
    OsIf_Trusted_Call1param(MC_RGM_ResetInit,(MC_RGM_pConfigPtr))
  #else

#define  Call_MC_RGM_ResetInit(MC_RGM_pConfigPtr) \
    MC_RGM_ResetInit(MC_RGM_pConfigPtr)
  #endif
#endif

#ifdef POWER_IP_ENABLE_USER_MODE_SUPPORT
  #if (STD_ON == POWER_IP_ENABLE_USER_MODE_SUPPORT)

#define  Call_MC_RGM_PerformReset(MC_RGM_pConfigPtr) \
    OsIf_Trusted_Call1param(MC_RGM_PerformReset,(MC_RGM_pConfigPtr))
  #else

#define  Call_MC_RGM_PerformReset(MC_RGM_pConfigPtr) \
    MC_RGM_PerformReset(MC_RGM_pConfigPtr)
  #endif
#endif /* POWER_IP_ENABLE_USER_MODE_SUPPORT */


#ifdef POWER_IP_ENABLE_USER_MODE_SUPPORT
  #if (STD_ON == POWER_IP_ENABLE_USER_MODE_SUPPORT)

#define  Call_MC_RGM_GetResetReason() \
    OsIf_Trusted_Call_Return(MC_RGM_GetResetReason)
  #else

#define  Call_MC_RGM_GetResetReason() \
    MC_RGM_GetResetReason()
  #endif
#endif /* POWER_IP_ENABLE_USER_MODE_SUPPORT */


#ifdef POWER_IP_ENABLE_USER_MODE_SUPPORT
  #if (STD_ON == POWER_IP_ENABLE_USER_MODE_SUPPORT)

#define  Call_MC_RGM_GetResetRawValue() \
    OsIf_Trusted_Call_Return(MC_RGM_GetResetRawValue)
  #else

#define  Call_MC_RGM_GetResetRawValue() \
    MC_RGM_GetResetRawValue()
  #endif
#endif

#if (defined(FEATURE_RESET_DURING_STANDBY_SUPPORTED))
  #if (FEATURE_RESET_DURING_STANDBY_SUPPORTED == STD_ON)
    #if (defined(MCU_ENTER_LOW_POWER_MODE))
      #if (MCU_ENTER_LOW_POWER_MODE == STD_ON)
        #ifdef POWER_IP_ENABLE_USER_MODE_SUPPORT
          #if (STD_ON == POWER_IP_ENABLE_USER_MODE_SUPPORT)

#define  Call_MC_RGM_ResetDuringStandby() \
    OsIf_Trusted_Call_Return(MC_RGM_ResetDuringStandby)
          #else

#define  Call_MC_RGM_ResetDuringStandby() \
    MC_RGM_ResetDuringStandby()
          #endif
        #endif /* POWER_IP_ENABLE_USER_MODE_SUPPORT */
      #endif /*(MCU_ENTER_LOW_POWER_MODE == STD_ON) */
    #endif
  #endif
#endif /* (FEATURE_RESET_DURING_STANDBY_SUPPORTED == STD_ON) */

#ifdef POWER_IP_ENABLE_USER_MODE_SUPPORT
  #if (STD_ON == POWER_IP_ENABLE_USER_MODE_SUPPORT)

#define  Call_Power_Ip_MC_RGM_ModeConfig(MC_RGM_pModeConfigPtr) \
    OsIf_Trusted_Call1param(Power_Ip_MC_RGM_ModeConfig,(MC_RGM_pModeConfigPtr))
  #else

#define  Call_Power_Ip_MC_RGM_ModeConfig(MC_RGM_pModeConfigPtr) \
    Power_Ip_MC_RGM_ModeConfig(MC_RGM_pModeConfigPtr)
  #endif
#endif /* POWER_IP_ENABLE_USER_MODE_SUPPORT */


#ifdef POWER_IP_ENABLE_USER_MODE_SUPPORT
  #if (STD_ON == POWER_IP_ENABLE_USER_MODE_SUPPORT)

#define  Call_Power_Ip_MC_RGM_CheckModeConfig(MC_RGM_pModeConfigPtr) \
    OsIf_Trusted_Call1param(Power_Ip_MC_RGM_CheckModeConfig,(MC_RGM_pModeConfigPtr))
  #else

#define  Call_Power_Ip_MC_RGM_CheckModeConfig(MC_RGM_pModeConfigPtr) \
    Power_Ip_MC_RGM_CheckModeConfig(MC_RGM_pModeConfigPtr)
  #endif
#endif /* POWER_IP_ENABLE_USER_MODE_SUPPORT */


void MC_RGM_ResetInit(const Power_Ip_MC_RGM_ConfigType * MC_RGM_pConfigPtr);

Power_Ip_ResetType MC_RGM_GetResetReason(void);
Power_Ip_RawResetType MC_RGM_GetResetRawValue(void);

#if (defined(MCU_RESET_DOMAINS_SUPPORTED))
  #if (MCU_RESET_DOMAINS_SUPPORTED == STD_ON)
void Power_Ip_MC_RGM_ModeConfig(const Power_Ip_MC_RGM_ModeConfigType * MC_RGM_pModeConfigPtr);
void Power_Ip_MC_RGM_CheckModeConfig(const Power_Ip_MC_RGM_ModeConfigType * MC_RGM_pModeConfigPtr);
  #endif /* (MCU_RESET_DOMAINS_SUPPORTED == STD_ON) */
#endif

#if (defined(FEATURE_RESET_DURING_STANDBY_SUPPORTED))
  #if (FEATURE_RESET_DURING_STANDBY_SUPPORTED == STD_ON)
    #if (defined(MCU_ENTER_LOW_POWER_MODE))
      #if (MCU_ENTER_LOW_POWER_MODE == STD_ON)
boolean MC_RGM_ResetDuringStandby(void);
      #endif /*(MCU_ENTER_LOW_POWER_MODE == STD_ON) */
    #endif
  #endif
#endif /* (FEATURE_RESET_DURING_STANDBY_SUPPORTED == STD_ON) */

#if (defined(MCU_PERFORM_RESET_API))
  #if (MCU_PERFORM_RESET_API == STD_ON)
void MC_RGM_PerformReset(const Power_Ip_MC_RGM_ConfigType * MC_RGM_pConfigPtr);
  #endif /* (MCU_PERFORM_RESET_API == STD_ON) */
#endif

#if (defined(MCU_RESET_ALTERNATE_ISR_USED))
  #if (MCU_RESET_ALTERNATE_ISR_USED == STD_ON)
void MC_RGM_ResetAltInt(void);
#endif /* (MCU_RESET_ALTERNATE_ISR_USED == STD_ON) */
#endif


#define MCU_STOP_SEC_CODE

#include "Mcu_MemMap.h"


#ifdef __cplusplus
}
#endif

#endif /* MC_RGM_H */

/** @} */
