/*==================================================================================================
*   Project              : RTD AUTOSAR 4.4
*   Platform             : CORTEXM
*   Peripheral           : 
*   Dependencies         : none
*
*   Autosar Version      : 4.4.0
*   Autosar Revision     : ASR_REL_4_4_REV_0000
*   Autosar Conf.Variant :
*   SW Version           : 0.9.0
*   Build Version        : S32K3_RTD_0_9_0_P02_D2107_ASR_REL_4_4_REV_0000_20210716
*
*   (c) Copyright 2020 - 2021 NXP Semiconductors
*   All Rights Reserved.
*
*   NXP Confidential. This software is owned or controlled by NXP and may only be
*   used strictly in accordance with the applicable license terms. By expressly
*   accepting such terms or by downloading, installing, activating and/or otherwise
*   using the software, you are agreeing that you have read, and that you agree to
*   comply with and are bound by, such license terms. If you do not agree to be
*   bound by the applicable license terms, then you may not retain, install,
*   activate or otherwise use the software.
==================================================================================================*/
/**
*   @file       PMC.h
*   @version    0.9.0
*
*   @brief   POWER IP driver header file.
*   @details POWER IP driver header file.
*
*   @addtogroup POWER_DRIVER Power Ip Driver
*   @{
*/


#ifndef PMC_H
#define PMC_H


#ifdef __cplusplus
extern "C"{
#endif



/*==================================================================================================
                                         INCLUDE FILES
 1) system and project includes
 2) needed interfaces from external units
 3) internal and external interfaces from this unit
==================================================================================================*/
#include "Power_Ip_Types.h"
#include "StandardTypes.h"
#include "Mcal.h"

/*==================================================================================================
                               SOURCE FILE VERSION INFORMATION
==================================================================================================*/
#define PMC_VENDOR_ID                       43
#define PMC_AR_RELEASE_MAJOR_VERSION        4
#define PMC_AR_RELEASE_MINOR_VERSION        4
#define PMC_AR_RELEASE_REVISION_VERSION     0
#define PMC_SW_MAJOR_VERSION                0
#define PMC_SW_MINOR_VERSION                9
#define PMC_SW_PATCH_VERSION                0

/*==================================================================================================
                                      FILE VERSION CHECKS
==================================================================================================*/
#ifndef DISABLE_MCAL_INTERMODULE_ASR_CHECK
/* Check if PMC.h file and StandardTypes.h file are of the same Autosar version */
#if ((PMC_AR_RELEASE_MAJOR_VERSION    != STD_AR_RELEASE_MAJOR_VERSION) || \
     (PMC_AR_RELEASE_MINOR_VERSION    != STD_AR_RELEASE_MINOR_VERSION))
    #error "AutoSar Version Numbers of PMC.h and StandardTypes.h are different"
#endif

/* Check if PMC.h file and Mcal.h file are of the same Autosar version */
#if ((PMC_AR_RELEASE_MAJOR_VERSION    != MCAL_AR_RELEASE_MAJOR_VERSION) || \
     (PMC_AR_RELEASE_MINOR_VERSION    != MCAL_AR_RELEASE_MINOR_VERSION))
    #error "AutoSar Version Numbers of PMC.h and Mcal.h are different"
#endif
#endif

/* Check if PMC.h file and Power_Ip_Types.h file have same versions */
#if (PMC_VENDOR_ID  != POWER_IP_TYPES_VENDOR_ID)
    #error "PMC.h and Power_Ip_Types.h have different vendor IDs"
#endif

/* Check if PMC.h file and Power_Ip_Types.h file are of the same Autosar version */
#if ((PMC_AR_RELEASE_MAJOR_VERSION    != POWER_IP_TYPES_AR_RELEASE_MAJOR_VERSION) || \
     (PMC_AR_RELEASE_MINOR_VERSION    != POWER_IP_TYPES_AR_RELEASE_MINOR_VERSION) || \
     (PMC_AR_RELEASE_REVISION_VERSION != POWER_IP_TYPES_AR_RELEASE_REVISION_VERSION))
    #error "AutoSar Version Numbers of PMC.h and Power_Ip_Types.h are different"
#endif

/* Check if PMC.h file and Power_Ip_Types.h file are of the same Software version */
#if ((PMC_SW_MAJOR_VERSION != POWER_IP_TYPES_SW_MAJOR_VERSION) || \
     (PMC_SW_MINOR_VERSION != POWER_IP_TYPES_SW_MINOR_VERSION) || \
     (PMC_SW_PATCH_VERSION != POWER_IP_TYPES_SW_PATCH_VERSION))
    #error "Software Version Numbers of PMC.h and Power_Ip_Types.h are different"
#endif

/*==================================================================================================
*                                          CONSTANTS
==================================================================================================*/


/*==================================================================================================
                                       DEFINES AND MACROS
==================================================================================================*/

/*==================================================================================================
*                                             ENUMS
==================================================================================================*/


/*==================================================================================================
*                                STRUCTURES AND OTHER TYPEDEFS
==================================================================================================*/



/*==================================================================================================
*                                GLOBAL VARIABLE DECLARATIONS
==================================================================================================*/


/*==================================================================================================
*                                    FUNCTION PROTOTYPES
==================================================================================================*/
#define MCU_START_SEC_CODE
#include "Mcu_MemMap.h"


#if (defined(POWER_IP_ENABLE_USER_MODE_SUPPORT))
  #if (STD_ON == POWER_IP_ENABLE_USER_MODE_SUPPORT)
    #if (defined(MCAL_PMC_REG_PROT_AVAILABLE))
      #if (STD_ON == MCAL_PMC_REG_PROT_AVAILABLE)
void PMC_SetUserAccessAllowed(void);
      #endif /* (STD_ON == MCAL_PMC_REG_PROT_AVAILABLE) */
    #endif
  #endif /* (STD_ON == POWER_IP_ENABLE_USER_MODE_SUPPORT) */
#endif


#ifdef POWER_IP_ENABLE_USER_MODE_SUPPORT
  #if (STD_ON == POWER_IP_ENABLE_USER_MODE_SUPPORT)
/**
* @violates @ref PMC_H_REF_2 A function should be used in preference to a function-like macro
* where they are interchangeable.
*/
#define  Call_PMC_PowerInit(PMC_pConfigPtr) \
    OsIf_Trusted_Call1param(PMC_PowerInit,(PMC_pConfigPtr))
  #else
/**
* @violates @ref PMC_H_REF_2 A function should be used in preference to a function-like macro
* where they are interchangeable.
*/
/**
* @violates @ref PMC_H_REF_7 This macros reserved for future use.
*/
#define  Call_PMC_PowerInit(PMC_pConfigPtr) \
    PMC_PowerInit(PMC_pConfigPtr)
  #endif  /* (STD_ON == POWER_IP_ENABLE_USER_MODE_SUPPORT) */
#endif /* #ifdef POWER_IP_ENABLE_USER_MODE_SUPPORT */


void PMC_PowerInit(const PMC_ConfigType * pConfigPtr);


#if (defined(MCU_ENTER_LOW_POWER_MODE))
  #if (MCU_ENTER_LOW_POWER_MODE == STD_ON)

#ifdef POWER_IP_ENABLE_USER_MODE_SUPPORT
  #if (STD_ON == POWER_IP_ENABLE_USER_MODE_SUPPORT)

#define  Call_PMC_PrepareLowPowerEntry() \
    OsIf_Trusted_Call(PMC_PrepareLowPowerEntry)
  #else

#define  Call_PMC_PrepareLowPowerEntry() \
    PMC_PrepareLowPowerEntry()
  #endif  /* (STD_ON == POWER_IP_ENABLE_USER_MODE_SUPPORT) */
#endif /* #ifdef POWER_IP_ENABLE_USER_MODE_SUPPORT */


void PMC_PrepareLowPowerEntry(void);

  #endif /* (MCU_ENTER_LOW_POWER_MODE == STD_ON) */
#endif

#if (defined(MCU_VOLTAGE_ERROR_ISR_USED))
  #if(MCU_VOLTAGE_ERROR_ISR_USED == STD_ON)
void PMC_VoltageErrorIsr(void);
  #endif
#endif /* (MCU_VOLTAGE_ERROR_ISR_USED == STD_ON) */

#define MCU_STOP_SEC_CODE
#include "Mcu_MemMap.h"


#ifdef __cplusplus
}
#endif

#endif /* PMC_H */

/** @} */
