/*==================================================================================================
*   Project              : RTD AUTOSAR 4.4
*   Platform             : CORTEXM
*   Peripheral           : 
*   Dependencies         : none
*
*   Autosar Version      : 4.4.0
*   Autosar Revision     : ASR_REL_4_4_REV_0000
*   Autosar Conf.Variant :
*   SW Version           : 0.9.0
*   Build Version        : S32K3_RTD_0_9_0_P02_D2107_ASR_REL_4_4_REV_0000_20210716
*
*   (c) Copyright 2020 - 2021 NXP Semiconductors
*   All Rights Reserved.
*
*   NXP Confidential. This software is owned or controlled by NXP and may only be
*   used strictly in accordance with the applicable license terms. By expressly
*   accepting such terms or by downloading, installing, activating and/or otherwise
*   using the software, you are agreeing that you have read, and that you agree to
*   comply with and are bound by, such license terms. If you do not agree to be
*   bound by the applicable license terms, then you may not retain, install,
*   activate or otherwise use the software.
==================================================================================================*/
/**
*   @file       Power_Ip_Private.h
*   @version    0.9.0
*
*   @brief   POWER IP driver header file.
*   @details POWER IP driver header file.
*
*   @addtogroup POWER_DRIVER Power Ip Driver
*   @{
*/


#ifndef POWER_IP_PRIVATE_H
#define POWER_IP_PRIVATE_H


#ifdef __cplusplus
extern "C"{
#endif


/*==================================================================================================
                                         INCLUDE FILES
 1) system and project includes
 2) needed interfaces from external units
 3) internal and external interfaces from this unit
==================================================================================================*/
#include "Mcal.h"
#include "Power_Ip.h"
#include "MC_ME.h"
#include "MC_RGM.h"
#include "PMC.h"
#include "Power_Ip_CortexM7.h"

#if defined(S32G2XX)
    #include "Power_Ip_Specific.h"
#elif defined(S32R45)
    #include "Power_Ip_Specific1.h"
#elif defined(S32K3XX)
    #include "Power_Ip_Specific.h"
    #include "Power_Ip_FLASH.h"
#else
    #error Platform not found!
#endif

#ifdef POWER_IP_DEV_ERROR_DETECT
#if (STD_ON == POWER_IP_DEV_ERROR_DETECT)
    #include "Devassert.h"
#endif /* (STD_ON == POWER_IP_DEV_ERROR_DETECT) */
#endif /* #ifdef POWER_IP_DEV_ERROR_DETECT */
/*==================================================================================================
                               SOURCE FILE VERSION INFORMATION
==================================================================================================*/
#define POWER_IP_PRIVATE_VENDOR_ID                    43
#define POWER_IP_PRIVATE_AR_RELEASE_MAJOR_VERSION     4
#define POWER_IP_PRIVATE_AR_RELEASE_MINOR_VERSION     4
#define POWER_IP_PRIVATE_AR_RELEASE_REVISION_VERSION  0
#define POWER_IP_PRIVATE_SW_MAJOR_VERSION             0
#define POWER_IP_PRIVATE_SW_MINOR_VERSION             9
#define POWER_IP_PRIVATE_SW_PATCH_VERSION             0


/*==================================================================================================
                                      FILE VERSION CHECKS
==================================================================================================*/

#ifndef DISABLE_MCAL_INTERMODULE_ASR_CHECK
/* Check if Power_Ip_Private.h file and Mcal.h file are of the same Autosar version */
#if ((POWER_IP_PRIVATE_AR_RELEASE_MAJOR_VERSION    != MCAL_AR_RELEASE_MAJOR_VERSION) || \
     (POWER_IP_PRIVATE_AR_RELEASE_MINOR_VERSION    != MCAL_AR_RELEASE_MINOR_VERSION))
    #error "AutoSar Version Numbers of Power_Ip_Private.h and Mcal.h are different"
#endif
#endif

/* Check if Power_Ip_Private.h file and MC_ME.h file have same versions */
#if (POWER_IP_PRIVATE_VENDOR_ID  != MC_ME_VENDOR_ID)
    #error "Power_Ip_Private.h and MC_ME.h have different vendor IDs"
#endif

/* Check if Power_Ip_Private.h file and MC_ME.h file are of the same Autosar version */
#if ((POWER_IP_PRIVATE_AR_RELEASE_MAJOR_VERSION    != MC_ME_AR_RELEASE_MAJOR_VERSION) || \
     (POWER_IP_PRIVATE_AR_RELEASE_MINOR_VERSION    != MC_ME_AR_RELEASE_MINOR_VERSION) || \
     (POWER_IP_PRIVATE_AR_RELEASE_REVISION_VERSION != MC_ME_AR_RELEASE_REVISION_VERSION))
    #error "AutoSar Version Numbers of Power_Ip_Private.h and MC_ME.h are different"
#endif

/* Check if Power_Ip_Private.h file and MC_ME.h file are of the same Software version */
#if ((POWER_IP_PRIVATE_SW_MAJOR_VERSION != MC_ME_SW_MAJOR_VERSION) || \
     (POWER_IP_PRIVATE_SW_MINOR_VERSION != MC_ME_SW_MINOR_VERSION) || \
     (POWER_IP_PRIVATE_SW_PATCH_VERSION != MC_ME_SW_PATCH_VERSION))
    #error "Software Version Numbers of Power_Ip_Private.h and MC_ME.h are different"
#endif

/* Check if Power_Ip_Private.h file and MC_RGM.h file have same versions */
#if (POWER_IP_PRIVATE_VENDOR_ID  != MC_RGM_VENDOR_ID)
    #error "Power_Ip_Private.h and MC_RGM.h have different vendor IDs"
#endif

/* Check if Power_Ip_Private.h file and MC_RGM.h file are of the same Autosar version */
#if ((POWER_IP_PRIVATE_AR_RELEASE_MAJOR_VERSION    != MC_RGM_AR_RELEASE_MAJOR_VERSION) || \
     (POWER_IP_PRIVATE_AR_RELEASE_MINOR_VERSION    != MC_RGM_AR_RELEASE_MINOR_VERSION) || \
     (POWER_IP_PRIVATE_AR_RELEASE_REVISION_VERSION != MC_RGM_AR_RELEASE_REVISION_VERSION))
    #error "AutoSar Version Numbers of Power_Ip_Private.h and MC_RGM.h are different"
#endif

/* Check if Power_Ip_Private.h file and MC_RGM.h file are of the same Software version */
#if ((POWER_IP_PRIVATE_SW_MAJOR_VERSION != MC_RGM_SW_MAJOR_VERSION) || \
     (POWER_IP_PRIVATE_SW_MINOR_VERSION != MC_RGM_SW_MINOR_VERSION) || \
     (POWER_IP_PRIVATE_SW_PATCH_VERSION != MC_RGM_SW_PATCH_VERSION))
    #error "Software Version Numbers of Power_Ip_Private.h and MC_RGM.h are different"
#endif

/* Check if Power_Ip_Private.h file and PMC.h file have same versions */
#if (POWER_IP_PRIVATE_VENDOR_ID  != PMC_VENDOR_ID)
    #error "Power_Ip_Private.h and PMC.h have different vendor IDs"
#endif

/* Check if Power_Ip_Private.h file and MC_RGM.h file are of the same Autosar version */
#if ((POWER_IP_PRIVATE_AR_RELEASE_MAJOR_VERSION    != PMC_AR_RELEASE_MAJOR_VERSION) || \
     (POWER_IP_PRIVATE_AR_RELEASE_MINOR_VERSION    != PMC_AR_RELEASE_MINOR_VERSION) || \
     (POWER_IP_PRIVATE_AR_RELEASE_REVISION_VERSION != PMC_AR_RELEASE_REVISION_VERSION))
    #error "AutoSar Version Numbers of Power_Ip_Private.h and PMC.h are different"
#endif

/* Check if Power_Ip_Private.h file and PMC.h file are of the same Software version */
#if ((POWER_IP_PRIVATE_SW_MAJOR_VERSION != PMC_SW_MAJOR_VERSION) || \
     (POWER_IP_PRIVATE_SW_MINOR_VERSION != PMC_SW_MINOR_VERSION) || \
     (POWER_IP_PRIVATE_SW_PATCH_VERSION != PMC_SW_PATCH_VERSION))
    #error "Software Version Numbers of Power_Ip_Private.h and PMC.h are different"
#endif

/* Check if Power_Ip_Private.h file and Power_Ip.h file have same versions */
#if (POWER_IP_PRIVATE_VENDOR_ID  != POWER_IP_VENDOR_ID)
    #error "Power_Ip_Private.h and Power_Ip.h have different vendor IDs"
#endif

/* Check if Power_Ip_Private.h file and Power_Ip.h file are of the same Autosar version */
#if ((POWER_IP_PRIVATE_AR_RELEASE_MAJOR_VERSION    != POWER_IP_AR_RELEASE_MAJOR_VERSION) || \
     (POWER_IP_PRIVATE_AR_RELEASE_MINOR_VERSION    != POWER_IP_AR_RELEASE_MINOR_VERSION) || \
     (POWER_IP_PRIVATE_AR_RELEASE_REVISION_VERSION != POWER_IP_AR_RELEASE_REVISION_VERSION))
    #error "AutoSar Version Numbers of Power_Ip_Private.h and Power_Ip.h are different"
#endif

/* Check if Power_Ip_Private.h file and Power_Ip.h file are of the same Software version */
#if ((POWER_IP_PRIVATE_SW_MAJOR_VERSION != POWER_IP_SW_MAJOR_VERSION) || \
     (POWER_IP_PRIVATE_SW_MINOR_VERSION != POWER_IP_SW_MINOR_VERSION) || \
     (POWER_IP_PRIVATE_SW_PATCH_VERSION != POWER_IP_SW_PATCH_VERSION))
    #error "Software Version Numbers of Power_Ip_Private.h and Power_Ip.h are different"
#endif

/* Check if Power_Ip_Private.h file and Power_Ip_CortexM7.h file have same versions */
#if (POWER_IP_PRIVATE_VENDOR_ID  != POWER_IP_CORTEXM7_VENDOR_ID)
    #error "Power_Ip_Private.h and Power_Ip_CortexM7.h have different vendor IDs"
#endif

/* Check if Power_Ip_Private.h file and Power_Ip_CortexM7.h file are of the same Autosar version */
#if ((POWER_IP_PRIVATE_AR_RELEASE_MAJOR_VERSION    != POWER_IP_CORTEXM7_AR_RELEASE_MAJOR_VERSION) || \
     (POWER_IP_PRIVATE_AR_RELEASE_MINOR_VERSION    != POWER_IP_CORTEXM7_AR_RELEASE_MINOR_VERSION) || \
     (POWER_IP_PRIVATE_AR_RELEASE_REVISION_VERSION != POWER_IP_CORTEXM7_AR_RELEASE_REVISION_VERSION))
    #error "AutoSar Version Numbers of Power_Ip_Private.h and Power_Ip_CortexM7.h are different"
#endif

/* Check if Power_Ip_Private.h file and Power_Ip_CortexM7.h file are of the same Software version */
#if ((POWER_IP_PRIVATE_SW_MAJOR_VERSION != POWER_IP_CORTEXM7_SW_MAJOR_VERSION) || \
     (POWER_IP_PRIVATE_SW_MINOR_VERSION != POWER_IP_CORTEXM7_SW_MINOR_VERSION) || \
     (POWER_IP_PRIVATE_SW_PATCH_VERSION != POWER_IP_CORTEXM7_SW_PATCH_VERSION))
    #error "Software Version Numbers of Power_Ip_Private.h and Power_Ip_CortexM7.h are different"
#endif

#if defined(S32G2XX)
/* Check if Power_Ip_Private.h file and Power_Ip_Specific.h file are of the same Autosar version */
#if ((POWER_IP_PRIVATE_AR_RELEASE_MAJOR_VERSION    != POWER_IP_SPECIFIC_AR_RELEASE_MAJOR_VERSION) || \
     (POWER_IP_PRIVATE_AR_RELEASE_MINOR_VERSION    != POWER_IP_SPECIFIC_AR_RELEASE_MINOR_VERSION) || \
     (POWER_IP_PRIVATE_AR_RELEASE_REVISION_VERSION != POWER_IP_SPECIFIC_AR_RELEASE_REVISION_VERSION))
    #error "AutoSar Version Numbers of Power_Ip_Private.h and Power_Ip_Specific.h are different"
#endif

/* Check if Power_Ip_Private.h file and Power_Ip_Specific.h file have same versions */
#if (POWER_IP_PRIVATE_VENDOR_ID  != POWER_IP_SPECIFIC_VENDOR_ID)
    #error "Power_Ip_Private.h and Power_Ip_Specific.h have different vendor IDs"
#endif

/* Check if Power_Ip_Private.h file and Power_Ip_Specific.h file are of the same Software version */
#if ((POWER_IP_PRIVATE_SW_MAJOR_VERSION != POWER_IP_SPECIFIC_SW_MAJOR_VERSION) || \
     (POWER_IP_PRIVATE_SW_MINOR_VERSION != POWER_IP_SPECIFIC_SW_MINOR_VERSION) || \
     (POWER_IP_PRIVATE_SW_PATCH_VERSION != POWER_IP_SPECIFIC_SW_PATCH_VERSION))
    #error "Software Version Numbers of Power_Ip_Private.h and Power_Ip_Specific.h are different"
#endif
#elif defined(S32R45)
/* Check if Power_Ip_Private.h file and Power_Ip_Specific1.h file are of the same Autosar version */
#if ((POWER_IP_PRIVATE_AR_RELEASE_MAJOR_VERSION    != POWER_IP_SPECIFIC1_AR_RELEASE_MAJOR_VERSION) || \
     (POWER_IP_PRIVATE_AR_RELEASE_MINOR_VERSION    != POWER_IP_SPECIFIC1_AR_RELEASE_MINOR_VERSION) || \
     (POWER_IP_PRIVATE_AR_RELEASE_REVISION_VERSION != POWER_IP_SPECIFIC1_AR_RELEASE_REVISION_VERSION))
    #error "AutoSar Version Numbers of Power_Ip_Private.h and Power_Ip_Specific1.h are different"
#endif

/* Check if Power_Ip_Private.h file and Power_Ip_Specific1.h file have same versions */
#if (POWER_IP_PRIVATE_VENDOR_ID  != POWER_IP_SPECIFIC1_VENDOR_ID)
    #error "Power_Ip_Private.h and Power_Ip_Specific1.h have different vendor IDs"
#endif

/* Check if Power_Ip_Private.h file and Power_Ip_Specific1.h file are of the same Software version */
#if ((POWER_IP_PRIVATE_SW_MAJOR_VERSION != POWER_IP_SPECIFIC1_SW_MAJOR_VERSION) || \
     (POWER_IP_PRIVATE_SW_MINOR_VERSION != POWER_IP_SPECIFIC1_SW_MINOR_VERSION) || \
     (POWER_IP_PRIVATE_SW_PATCH_VERSION != POWER_IP_SPECIFIC1_SW_PATCH_VERSION))
    #error "Software Version Numbers of Power_Ip_Private.h and Power_Ip_Specific1.h are different"
#endif
#elif defined(S32K3XX)
/* Check if Power_Ip_Private.h file and Power_Ip_Specific.h file are of the same Autosar version */
#if ((POWER_IP_PRIVATE_AR_RELEASE_MAJOR_VERSION    != POWER_IP_SPECIFIC_AR_RELEASE_MAJOR_VERSION) || \
     (POWER_IP_PRIVATE_AR_RELEASE_MINOR_VERSION    != POWER_IP_SPECIFIC_AR_RELEASE_MINOR_VERSION) || \
     (POWER_IP_PRIVATE_AR_RELEASE_REVISION_VERSION != POWER_IP_SPECIFIC_AR_RELEASE_REVISION_VERSION))
    #error "AutoSar Version Numbers of Power_Ip_Private.h and Power_Ip_Specific.h are different"
#endif

/* Check if Power_Ip_Private.h file and Power_Ip_Specific.h file have same versions */
#if (POWER_IP_PRIVATE_VENDOR_ID  != POWER_IP_SPECIFIC_VENDOR_ID)
    #error "Power_Ip_Private.h and Power_Ip_Specific.h have different vendor IDs"
#endif

/* Check if Power_Ip_Private.h file and Power_Ip_Specific.h file are of the same Software version */
#if ((POWER_IP_PRIVATE_SW_MAJOR_VERSION != POWER_IP_SPECIFIC_SW_MAJOR_VERSION) || \
     (POWER_IP_PRIVATE_SW_MINOR_VERSION != POWER_IP_SPECIFIC_SW_MINOR_VERSION) || \
     (POWER_IP_PRIVATE_SW_PATCH_VERSION != POWER_IP_SPECIFIC_SW_PATCH_VERSION))
    #error "Software Version Numbers of Power_Ip_Private.h and Power_Ip_Specific.h are different"
#endif
#else
    #error "Unknown platform!"
#endif

#ifndef DISABLE_MCAL_INTERMODULE_ASR_CHECK
#ifdef POWER_IP_DEV_ERROR_DETECT
#if (STD_ON == POWER_IP_DEV_ERROR_DETECT)
/* Check if Power_Ip_Private.h file and Devassert.h file are of the same Autosar version */
#if ((POWER_IP_PRIVATE_AR_RELEASE_MAJOR_VERSION    != DEVASSERT_AR_RELEASE_MAJOR_VERSION) || \
     (POWER_IP_PRIVATE_AR_RELEASE_MINOR_VERSION    != DEVASSERT_AR_RELEASE_MINOR_VERSION))
    #error "AutoSar Version Numbers of Power_Ip_Private.h and Devassert.h are different"
#endif
#endif /* (STD_ON == POWER_IP_DEV_ERROR_DETECT) */
#endif /* #ifdef POWER_IP_DEV_ERROR_DETECT */
#endif

/*==================================================================================================
*                                          CONSTANTS
==================================================================================================*/


/*==================================================================================================
                                       DEFINES AND MACROS
==================================================================================================*/
#ifdef POWER_IP_DEV_ERROR_DETECT
  #if (POWER_IP_DEV_ERROR_DETECT == STD_ON)
    #define POWER_DEV_ASSERT(x)      DevAssert(x)
  #else
    #define POWER_DEV_ASSERT(x)
  #endif
#else
    #define POWER_DEV_ASSERT(x)
#endif
/*==================================================================================================
*                                             ENUMS
==================================================================================================*/

/*==================================================================================================
*                                STRUCTURES AND OTHER TYPEDEFS
==================================================================================================*/

/*==================================================================================================
*                                GLOBAL VARIABLE DECLARATIONS
==================================================================================================*/

/*==================================================================================================
*                                    FUNCTION PROTOTYPES
==================================================================================================*/

#define MCU_START_SEC_CODE
#include "Mcu_MemMap.h"

void ReportPowerErrors(Power_Ip_PowerReportErrorType error, uint8 u8ErrorCode);
void ReportPowerErrorsEmptyCallback(Power_Ip_PowerReportErrorType error, uint8 u8ErrorCode);

/*!
 * @brief Initializes a starting reference point for timeout
 *
 * @param[out] startTimeOut    The starting time from which elapsed time is measured
 * @param[out] elapsedTimeOut  The elapsed time to be passed to PowerTimeoutExpired
 * @param[out] timeoutTicksOut The timeout value (in ticks) to be passed to PowerTimeoutExpired
 * @param[in]  timeoutUs       The timeout value (in microseconds)
 */
void PowerStartTimeout(uint32 *startTimeOut,
                       uint32 *elapsedTimeOut,
                       uint32 *timeoutTicksOut,
                       uint32 timeoutUs);

/*!
 * @brief Checks for timeout condition.
 *
 * @param[in,out] startTimeInOut    The starting time from which elapsed time is measured
 * @param[in,out] elapsedTimeInOut  The accumulated elapsed time from the starting time reference
 * @param[in]     TimeoutTicks      The timeout limit (in ticks)
 */
boolean PowerTimeoutExpired(uint32 *startTimeInOut,
                            uint32 *elapsedTimeInOut,
                            uint32 TimeoutTicks);

#define MCU_STOP_SEC_CODE
#include "Mcu_MemMap.h"
#ifdef __cplusplus
}
#endif


#endif /* POWER_IP_PRIVATE_H */

/** @} */
