/*==================================================================================================
*   Project              : RTD AUTOSAR 4.4
*   Platform             : CORTEXM
*   Peripheral           : 
*   Dependencies         : none
*
*   Autosar Version      : 4.4.0
*   Autosar Revision     : ASR_REL_4_4_REV_0000
*   Autosar Conf.Variant :
*   SW Version           : 0.9.0
*   Build Version        : S32K3_RTD_0_9_0_P02_D2107_ASR_REL_4_4_REV_0000_20210716
*
*   (c) Copyright 2020 - 2021 NXP Semiconductors
*   All Rights Reserved.
*
*   NXP Confidential. This software is owned or controlled by NXP and may only be
*   used strictly in accordance with the applicable license terms. By expressly
*   accepting such terms or by downloading, installing, activating and/or otherwise
*   using the software, you are agreeing that you have read, and that you agree to
*   comply with and are bound by, such license terms. If you do not agree to be
*   bound by the applicable license terms, then you may not retain, install,
*   activate or otherwise use the software.
==================================================================================================*/
/**
*   @file       Power_Ip_Types.h
*   @version    0.9.0
*
*   @brief   POWER IP type header file.
*   @details POWER IP type header file.
*
*   @addtogroup POWER_DRIVER Power Ip Driver
*   @{
*/


#ifndef POWER_IP_TYPES_H
#define POWER_IP_TYPES_H


#ifdef __cplusplus
extern "C"{
#endif


/**
* @page misra_violations MISRA-C:2012 violations
*
*/


/*==================================================================================================
                                         INCLUDE FILES
 1) system and project includes
 2) needed interfaces from external units
 3) internal and external interfaces from this unit
==================================================================================================*/
#include "Power_Ip_Cfg_Defines.h"
#include "MC_ME_Types.h"
#include "MC_RGM_Types.h"
#include "PMC_Types.h"

/*==================================================================================================
                               SOURCE FILE VERSION INFORMATION
==================================================================================================*/
#define POWER_IP_TYPES_VENDOR_ID                       43
#define POWER_IP_TYPES_AR_RELEASE_MAJOR_VERSION        4
#define POWER_IP_TYPES_AR_RELEASE_MINOR_VERSION        4
#define POWER_IP_TYPES_AR_RELEASE_REVISION_VERSION     0
#define POWER_IP_TYPES_SW_MAJOR_VERSION                0
#define POWER_IP_TYPES_SW_MINOR_VERSION                9
#define POWER_IP_TYPES_SW_PATCH_VERSION                0

/*==================================================================================================
                                      FILE VERSION CHECKS
==================================================================================================*/
/* Check if Power_Ip_Types.h file and Power_Ip_Cfg_Defines.h file have same versions */
#if (POWER_IP_TYPES_VENDOR_ID  != POWER_IP_CFG_DEFINES_VENDOR_ID)
    #error "Power_Ip_Types.h and Power_Ip_Cfg_Defines.h have different vendor IDs"
#endif

/* Check if Power_Ip_Types.h file and Power_Ip_Cfg_Defines.h file are of the same Autosar version */
#if ((POWER_IP_TYPES_AR_RELEASE_MAJOR_VERSION    != POWER_IP_CFG_DEFINES_AR_RELEASE_MAJOR_VERSION) || \
     (POWER_IP_TYPES_AR_RELEASE_MINOR_VERSION    != POWER_IP_CFG_DEFINES_AR_RELEASE_MINOR_VERSION) || \
     (POWER_IP_TYPES_AR_RELEASE_REVISION_VERSION != POWER_IP_CFG_DEFINES_AR_RELEASE_REVISION_VERSION))
    #error "AutoSar Version Numbers of Power_Ip_Types.h and Power_Ip_Cfg_Defines.h are different"
#endif

/* Check if Power_Ip_Types.h file and Power_Ip_Cfg_Defines.h file are of the same Software version */
#if ((POWER_IP_TYPES_SW_MAJOR_VERSION != POWER_IP_CFG_DEFINES_SW_MAJOR_VERSION) || \
     (POWER_IP_TYPES_SW_MINOR_VERSION != POWER_IP_CFG_DEFINES_SW_MINOR_VERSION) || \
     (POWER_IP_TYPES_SW_PATCH_VERSION != POWER_IP_CFG_DEFINES_SW_PATCH_VERSION))
    #error "Software Version Numbers of Power_Ip_Types.h and Power_Ip_Cfg_Defines.h are different"
#endif

/* Check if Power_Ip_Types.h file and MC_ME_Types.h file have same versions */
#if (POWER_IP_TYPES_VENDOR_ID  != MC_ME_TYPES_VENDOR_ID)
    #error "Power_Ip_Types.h and MC_ME_Types.h have different vendor IDs"
#endif

/* Check if Power_Ip_Types.h file and MC_ME_Types.h file are of the same Autosar version */
#if ((POWER_IP_TYPES_AR_RELEASE_MAJOR_VERSION    != MC_ME_TYPES_AR_RELEASE_MAJOR_VERSION) || \
     (POWER_IP_TYPES_AR_RELEASE_MINOR_VERSION    != MC_ME_TYPES_AR_RELEASE_MINOR_VERSION) || \
     (POWER_IP_TYPES_AR_RELEASE_REVISION_VERSION != MC_ME_TYPES_AR_RELEASE_REVISION_VERSION))
    #error "AutoSar Version Numbers of Power_Ip_Types.h and MC_ME_Types.h are different"
#endif

/* Check if Power_Ip_Types.h file and MC_ME_Types.h file are of the same Software version */
#if ((POWER_IP_TYPES_SW_MAJOR_VERSION != MC_ME_TYPES_SW_MAJOR_VERSION) || \
     (POWER_IP_TYPES_SW_MINOR_VERSION != MC_ME_TYPES_SW_MINOR_VERSION) || \
     (POWER_IP_TYPES_SW_PATCH_VERSION != MC_ME_TYPES_SW_PATCH_VERSION))
    #error "Software Version Numbers of Power_Ip_Types.h and MC_ME_Types.h are different"
#endif

/* Check if Power_Ip_Types.h file and MC_RGM_Types.h file have same versions */
#if (POWER_IP_TYPES_VENDOR_ID  != MC_RGM_TYPES_VENDOR_ID)
    #error "Power_Ip_Types.h and MC_RGM_Types.h have different vendor IDs"
#endif

/* Check if Power_Ip_Types.h file and MC_RGM_Types.h file are of the same Autosar version */
#if ((POWER_IP_TYPES_AR_RELEASE_MAJOR_VERSION    != MC_RGM_TYPES_AR_RELEASE_MAJOR_VERSION) || \
     (POWER_IP_TYPES_AR_RELEASE_MINOR_VERSION    != MC_RGM_TYPES_AR_RELEASE_MINOR_VERSION) || \
     (POWER_IP_TYPES_AR_RELEASE_REVISION_VERSION != MC_RGM_TYPES_AR_RELEASE_REVISION_VERSION))
    #error "AutoSar Version Numbers of Power_Ip_Types.h and MC_RGM_Types.h are different"
#endif

/* Check if Power_Ip_Types.h file and MC_RGM_Types.h file are of the same Software version */
#if ((POWER_IP_TYPES_SW_MAJOR_VERSION != MC_RGM_TYPES_SW_MAJOR_VERSION) || \
     (POWER_IP_TYPES_SW_MINOR_VERSION != MC_RGM_TYPES_SW_MINOR_VERSION) || \
     (POWER_IP_TYPES_SW_PATCH_VERSION != MC_RGM_TYPES_SW_PATCH_VERSION))
    #error "Software Version Numbers of Power_Ip_Types.h and MC_RGM_Types.h are different"
#endif

/* Check if Power_Ip_Types.h file and PMC_Types.h file have same versions */
#if (POWER_IP_TYPES_VENDOR_ID  != PMC_TYPES_VENDOR_ID)
    #error "Power_Ip_Types.h and PMC_Types.h have different vendor IDs"
#endif

/* Check if Power_Ip_Types.h file and PMC_Types.h file are of the same Autosar version */
#if ((POWER_IP_TYPES_AR_RELEASE_MAJOR_VERSION    != PMC_TYPES_AR_RELEASE_MAJOR_VERSION) || \
     (POWER_IP_TYPES_AR_RELEASE_MINOR_VERSION    != PMC_TYPES_AR_RELEASE_MINOR_VERSION) || \
     (POWER_IP_TYPES_AR_RELEASE_REVISION_VERSION != PMC_TYPES_AR_RELEASE_REVISION_VERSION))
    #error "AutoSar Version Numbers of Power_Ip_Types.h and PMC_Types.h are different"
#endif

/* Check if Power_Ip_Types.h file and PMC_Types.h file are of the same Software version */
#if ((POWER_IP_TYPES_SW_MAJOR_VERSION != PMC_TYPES_SW_MAJOR_VERSION) || \
     (POWER_IP_TYPES_SW_MINOR_VERSION != PMC_TYPES_SW_MINOR_VERSION) || \
     (POWER_IP_TYPES_SW_PATCH_VERSION != PMC_TYPES_SW_PATCH_VERSION))
    #error "Software Version Numbers of Power_Ip_Types.h and PMC_Types.h are different"
#endif
/*==================================================================================================
*                                          CONSTANTS
==================================================================================================*/


/*==================================================================================================
                                       DEFINES AND MACROS
==================================================================================================*/
#define POWER_IP_ERR_CODE_RESERVED   0xFF

#define POWER_E_ISR_FUNC_RESET_ALT_FAILURE        ((uint8)0x03U)
#define POWER_E_ISR_DEST_RESET_ALT_FAILURE        ((uint8)0x04U)
#define POWER_E_ISR_VOLTAGE_ERROR                 ((uint8)0x06U)
#define POWER_LAST_MILE_REGULATOR_DISABLED        ((uint8)0x55U)
#define POWER_E_FLASH_HV_OPERATION_ONGOING        ((uint8)0x07U)
/*==================================================================================================
*                                             ENUMS
==================================================================================================*/


/*==================================================================================================
*                                STRUCTURES AND OTHER TYPEDEFS
==================================================================================================*/
/**
* @brief            
* @details          
*/
typedef struct
{
    /**< @brief Configuration for MC_RGM (Reset Generation Module) hardware IP. */
    const Power_Ip_MC_RGM_ConfigType * MC_RGM_pConfig;
    /**< @brief Configuration for PMC (Power Management Unit) hardware IP, part of PMC. */
    const PMC_ConfigType * PMC_pConfig;

} Power_Ip_HwIPsConfigType;


/**
* @brief            Power Modes encoding.
* @details          Supported power modes for the MCU.
*/
typedef enum
{
    POWER_IP_DEST_RESET_MODE = 0U,           /**< @brief Destructive Reset Mode. */
    POWER_IP_FUNC_RESET_MODE = 1U,           /**< @brief Functional Reset Mode. */
    POWER_IP_RESET_MODE = 2U,                /**< @brief Any reset mode. Used when the particular type of reset doesn't matter. */
    POWER_IP_CORE_WARM_RESET_MODE = 3U,      /**< @brief Core Warm Reset Mode. */
#if (defined(MCU_ENTER_LOW_POWER_MODE))
  #if (MCU_ENTER_LOW_POWER_MODE == STD_ON)
    POWER_IP_CORE_STANDBY_MODE = 13U,        /**< @brief Core Standby Mode */
    POWER_IP_SOC_PREPARE_STANDBY_MODE = 14U, /**< @brief Prepare Standby Mode */
    POWER_IP_SOC_STANDBY_MODE  = 15U,        /**< @brief StandBy Mode. */
    POWER_IP_STANDBY_MODE  = 16U,            /**< @brief Prepare Standby and StandBy Mode. */
  #endif /* (MCU_ENTER_LOW_POWER_MODE == STD_ON) */
#endif
    POWER_IP_RUN_MODE   = 32U,               /**< @brief Run Mode. */

} Power_Ip_PowerModeType;

/**
* @brief            The type Mcu_RawResetType specifies the reset reason in raw register format, read from a reset status register.
* @details          The type shall be uint8, uint16 or uint32 based on best performance.
*
* @implements Mcu_RawResetType_typedef
*
*/
typedef uint32 Power_Ip_RawResetType;   /**< @brief Destructive and Functional Reset Events Log. */

/**
* @brief            The Mcu_ModeType specifies the identification (ID) for a MCU mode, configured via configuration structure.
* @details          The type shall be uint8, uint16 or uint32.
*
* @implements     Mcu_ModeType_typedef
*
*/
typedef uint32 Power_Ip_ModeType;

/**
* @brief            
* @details          
*/
typedef struct
{
    /**< @brief The ID for Power Mode configuration. */
    Power_Ip_ModeType Power_Ip_ModeConfigId;

    /**< @brief The Power Mode name (code). */
    Power_Ip_PowerModeType Power_Ip_ePowerMode;

    /**< @brief MC_ME IP Mode settings. */
    const Power_Ip_MC_ME_ModeConfigType * MC_ME_pModeConfig;
    
#if (defined(MCU_RESET_DOMAINS_SUPPORTED))
  #if (MCU_RESET_DOMAINS_SUPPORTED == STD_ON)
    /**< @brief MC_RGM IP Mode settings. */
    const Power_Ip_MC_RGM_ModeConfigType * MC_RGM_pModeConfig;
  #endif /* (MCU_RESET_DOMAINS_SUPPORTED == STD_ON) */
#endif

} Power_Ip_ModeConfigType;


/** @brief Power ip report error types. */
typedef enum
{
    POWER_IP_REPORT_TIMEOUT_ERROR             = 0U,   /**< @brief Report Timeout Error. */
    POWER_IP_ERROR_ISR_NOTIFICATION           = 1U,   /**< @brief Notification Error. */
    POWER_IP_PMC_NOTIFICATION                 = 2U,   /**< @brief Notification PMC. */
} Power_Ip_PowerReportErrorType;

/*!
 * @brief Power report error callback structure.
 * Implements PowerReportErrorCallbackType_Class
 */
typedef void (*Power_Ip_ReportErrorsCallbackType)(Power_Ip_PowerReportErrorType error, uint8 u8ErrorCode);

typedef struct{
    volatile uint32 SCR;                               /**< System Control Register, offset: 0x0 */
} power_Ip_CM7_Type;

/*==================================================================================================
*                                GLOBAL VARIABLE DECLARATIONS
==================================================================================================*/


/*==================================================================================================
*                                    FUNCTION PROTOTYPES
==================================================================================================*/


#ifdef __cplusplus
}
#endif


#endif /* POWER_IP_TYPES_H */

/** @} */
