/*==================================================================================================
*   Project              : RTD AUTOSAR 4.4
*   Platform             : CORTEXM
*   Peripheral           : FLEXIO
*   Dependencies         : 
*
*   Autosar Version      : 4.4.0
*   Autosar Revision     : ASR_REL_4_4_REV_0000
*   Autosar Conf.Variant :
*   SW Version           : 0.9.0
*   Build Version        : S32K3_RTD_0_9_0_P02_D2107_ASR_REL_4_4_REV_0000_20210716
*
*   (c) Copyright 2020 - 2021 NXP Semiconductors
*   All Rights Reserved.
*
*   NXP Confidential. This software is owned or controlled by NXP and may only be
*   used strictly in accordance with the applicable license terms. By expressly
*   accepting such terms or by downloading, installing, activating and/or otherwise
*   using the software, you are agreeing that you have read, and that you agree to
*   comply with and are bound by, such license terms. If you do not agree to be
*   bound by the applicable license terms, then you may not retain, install,
*   activate or otherwise use the software.
==================================================================================================*/

#ifndef UART_TYPES_H
#define UART_TYPES_H

/**
*   @file
*   @defgroup uart_driver UART Driver
*   @addtogroup  uart_driver UART Driver
*   @{
*/

#ifdef __cplusplus
extern "C"
{
#endif

/*==================================================================================================
*                                        INCLUDE FILES
* 1) system and project includes
* 2) needed interfaces from external units
* 3) internal and external interfaces from this unit
==================================================================================================*/
#include "Uart_Defines.h"
#include "Uart_Ipw_Types.h"

/*==================================================================================================
*                               SOURCE FILE VERSION INFORMATION
==================================================================================================*/
#define UART_TYPES_VENDOR_ID                    43
#define UART_TYPES_AR_RELEASE_MAJOR_VERSION     4
#define UART_TYPES_AR_RELEASE_MINOR_VERSION     4
#define UART_TYPES_AR_RELEASE_REVISION_VERSION  0
#define UART_TYPES_SW_MAJOR_VERSION             0
#define UART_TYPES_SW_MINOR_VERSION             9
#define UART_TYPES_SW_PATCH_VERSION             0
/*==================================================================================================
*                                     FILE VERSION CHECKS
==================================================================================================*/
/*Checks against Uart_Defines.h */
#if (UART_TYPES_VENDOR_ID != UART_DEFINES_VENDOR_ID_CFG)
    #error "Uart_Types.h and Uart_Defines.h have different vendor ids"
#endif
#if ((UART_TYPES_AR_RELEASE_MAJOR_VERSION    != UART_DEFINES_AR_RELEASE_MAJOR_VERSION_CFG) || \
     (UART_TYPES_AR_RELEASE_MINOR_VERSION   != UART_DEFINES_AR_RELEASE_MINOR_VERSION_CFG) || \
     (UART_TYPES_AR_RELEASE_REVISION_VERSION!= UART_DEFINES_AR_RELEASE_REVISION_VERSION_CFG))
    #error "AUTOSAR Version Numbers of Uart_Types.h and Uart_Defines.h are different"
#endif
#if ((UART_TYPES_SW_MAJOR_VERSION!= UART_DEFINES_SW_MAJOR_VERSION_CFG) || \
     (UART_TYPES_SW_MINOR_VERSION!= UART_DEFINES_SW_MINOR_VERSION_CFG) || \
     (UART_TYPES_SW_PATCH_VERSION!= UART_DEFINES_SW_PATCH_VERSION_CFG) \
    )
    #error "Software Version Numbers of Uart_Types.h and Uart_Defines.h are different"
#endif

/*Checks against Uart_Ipw_Types.h */
#if (UART_TYPES_VENDOR_ID != UART_IPW_TYPES_VENDOR_ID)
    #error "Uart_Types.h and Uart_Ipw_Types.h have different vendor ids"
#endif
#if ((UART_TYPES_AR_RELEASE_MAJOR_VERSION    != UART_IPW_TYPES_AR_RELEASE_MAJOR_VERSION) || \
     (UART_TYPES_AR_RELEASE_MINOR_VERSION   != UART_IPW_TYPES_AR_RELEASE_MINOR_VERSION) || \
     (UART_TYPES_AR_RELEASE_REVISION_VERSION!= UART_IPW_TYPES_AR_RELEASE_REVISION_VERSION))
    #error "AUTOSAR Version Numbers of Uart_Types.h and Uart_Ipw_Types.h are different"
#endif
#if ((UART_TYPES_SW_MAJOR_VERSION!= UART_IPW_TYPES_SW_MAJOR_VERSION) || \
     (UART_TYPES_SW_MINOR_VERSION!= UART_IPW_TYPES_SW_MINOR_VERSION) || \
     (UART_TYPES_SW_PATCH_VERSION!= UART_IPW_TYPES_SW_PATCH_VERSION) \
    )
    #error "Software Version Numbers of Uart_Types.h and Uart_Ipw_Types.h are different"
#endif
/*==================================================================================================
*                                          CONSTANTS
==================================================================================================*/

/*==================================================================================================
*                                      DEFINES AND MACROS
==================================================================================================*/

/*==================================================================================================
*                                            ENUMS
==================================================================================================*/
#if ((UART_DEV_ERROR_DETECT == STD_ON) || (UART_RUNTIME_ERROR_DETECT == STD_ON))
/**
* @brief          Errors reported by Det_ReportError() and Det_ReportRuntimeError().
*/
typedef enum
{

    UART_E_UNINIT = 0x01,       /**< @brief Driver not initialized */
    UART_E_INVALID_CHANNEL,     /**< @brief Invalid channel given as parameter */
    UART_E_INVALID_POINTER ,    /**< @brief Invalid channel given as parameter */
    UART_E_ALREADY_INITIALIZED, /**< @brief Driver already initialized */
    UART_E_PARAM_CONFIG,      /**< @brief Invalid coreid */
    UART_E_CHANNEL_BUSY,        /**< @brief Channel is busy, therefore the service can not be executed */
    UART_E_INVALID_PARAMETER,   /**< @brief Parameter has a wrong value */
    UART_E_TIMEOUT              /**< @brief Timeout occured */

}
Uart_ErrorStatusType;
#endif /* ((UART_DEV_ERROR_DETECT == STD_ON) || (UART_RUNTIME_ERROR_DETECT == STD_ON)) */

/**
* @brief            Driver initialization status.
* @details          This enum contains the values for the driver initialization status.
*/
typedef enum
{
    UART_UNINIT = 0U,    /**< @brief Driver not initialized */
    UART_INIT ,          /**< @brief Driver ready */
}
Uart_DrvStatusType;

/**
* @brief          API functions service IDs.
* @details        Service IDs of the UART API.
*/
typedef enum
{
    UART_INIT_ID               =   (uint8)0x00U,  /**< @brief UART_Init() ID.*/
    UART_DEINIT_ID             =   (uint8)0x01U,  /**< @brief UART_DeInit() ID.*/
    UART_SETBAUDRATE_ID        =   (uint8)0x02U,  /**< @brief UART_SetBaudrate() ID.*/
    UART_GETBAUDRATE_ID        =   (uint8)0x03U,  /**< @brief UART_GetBaudrate() ID.*/
    UART_SYNCSEND_ID           =   (uint8)0x04U,  /**< @brief UART_SyncSend() ID.*/
    UART_ASYNCSEND_ID          =   (uint8)0x05U,  /**< @brief UART_AsyncReceive() ID.*/
    UART_SYNCRECEIVE_ID        =   (uint8)0x06U,  /**< @brief UART_SyncReceive() ID.*/
    UART_ASYNCRECEIVE_ID       =   (uint8)0x07U,  /**< @brief UART_AsyncSend() ID.*/
    UART_ABORT_ID              =   (uint8)0x08U,  /**< @brief UART_Abort() ID.*/
    UART_GETSTATUS_ID          =   (uint8)0x09U,  /**< @brief UART_GetStatus() ID.*/
    UART_VERSIONINFO_ID        =   (uint8)0x0AU,  /**< @brief Uart_GetVersionInfo() ID.*/
    UART_SETBUFFER_ID          =   (uint8)0x0BU   /**< @brief UART_SetBuffer() ID.*/
}
Uart_ApiFunctionIdType;

/**
* @brief          The type operation of an Uart channel.
*/
/* implements     Uart_DataDirectionType_Enumeration */
typedef enum
{
    UART_SEND    =  (uint8)0x00U, /**< @brief The sending operation */
    UART_RECEIVE =  (uint8)0x01U  /**< @brief The receiving operation */
}
Uart_DataDirectionType;

/**
* @brief          Uart operation status type
*/
typedef enum
{
    UART_NO_ERROR            =  0x00, /**< @brief Uart operation is successfull  */
    UART_OPERATION_ONGOING   =  0x01, /**< @brief Uart operation on going  */
    UART_ABORTED             =  0x02, /**< @brief Uart operation aborted  */
    E_UART_FRAMING_ERROR     =  0x03, /**< @brief Uart framing error  */
    E_UART_RX_OVERRUN_ERROR  =  0x04, /**< @brief Uart overrun error  */
    E_UART_PARITY_ERROR      =  0x05, /**< @brief Uart parity error  */
    E_UART_TIMEOUT           =  0x06, /**< @brief Uart operation has timeout  */
    E_UART_NOISE_ERROR       =  0x07, /**< @brief Uart noise error  */
    E_UART_DMA_ERROR         =  0x08, /**< @brief Uart Dma Error error  */
} Uart_StatusType;

/*==================================================================================================
*                                STRUCTURES AND OTHER TYPEDEFS
==================================================================================================*/
/**
* @brief          Uart channel configuration type structure.
* @details        This is the type of the external data structure containing
*                 the overall initialization data for one Uart Channel.
*                 A pointer to such a structure is provided to the Uart channel
*                 initialization routine for configuration of the Uart hardware
*                 channel.
*
*
*/
/* implements     Uart_ChannelConfigType_structure */
typedef struct
{
    uint8 u8UartChannelId;                           /**< @brief Uart channel configured  */
#if (UART_MULTICORE_SUPPORT == STD_ON)
    uint32 u32ChannelCoreId;                         /**< @brief Core id on which the Uart channel has been assigned  */
#endif
    uint32 u32ChannelClockFrequency;                 /**< @brief The clock frequency configured on the given channel  */
    const Uart_Ipw_HwConfigType* pUartChannelConfig; /**< @brief Pointer to a lower level channel configuration  */
} Uart_ChannelConfigType;

/**
* @brief          Uart driver configuration type structure.
* @details        This is the type of the pointer to the external data
*                 Uart Channels.
*                 A pointer of such a structure is provided to the Uart driver
*                 initialization routine for configuration of the Uart hardware
*                 channel.
*
*
*
*/
/* implements     Uart_ConfigType_structure */
typedef struct
{
#if (UART_MULTICORE_SUPPORT == STD_ON)
    uint32 u32PartitionCoreId;                                 /**< @brief Partition configured of the core  */
#endif
    /**
     * @brief     Hardware channel.
     * @details   Constant pointer of the constant external data
     *            structure containing the overall initialization data
     *            for all the configured Uart Channels.
     */
    const Uart_ChannelConfigType *Configs[UART_CH_MAX_CONFIG];
} Uart_ConfigType;

/*==================================================================================================
*                                GLOBAL VARIABLE DECLARATIONS
==================================================================================================*/

/*==================================================================================================
*                                    FUNCTION PROTOTYPES
==================================================================================================*/

#ifdef __cplusplus
}
#endif

/** @} */

#endif /* UART_Types_H */
