/*==================================================================================================
*   Project              : RTD AUTOSAR 4.4
*   Platform             : CORTEXM
*   Peripheral           : FLEXIO
*   Dependencies         : 
*
*   Autosar Version      : 4.4.0
*   Autosar Revision     : ASR_REL_4_4_REV_0000
*   Autosar Conf.Variant :
*   SW Version           : 0.9.0
*   Build Version        : S32K3_RTD_0_9_0_P02_D2107_ASR_REL_4_4_REV_0000_20210716
*
*   (c) Copyright 2020 - 2021 NXP Semiconductors
*   All Rights Reserved.
*
*   NXP Confidential. This software is owned or controlled by NXP and may only be
*   used strictly in accordance with the applicable license terms. By expressly
*   accepting such terms or by downloading, installing, activating and/or otherwise
*   using the software, you are agreeing that you have read, and that you agree to
*   comply with and are bound by, such license terms. If you do not agree to be
*   bound by the applicable license terms, then you may not retain, install,
*   activate or otherwise use the software.
==================================================================================================*/
/**
*   @file     Flexio_Uart_Ip.c
*   @defgroup flexio_uart_ip Flexio UART IPL
*   @addtogroup  flexio_uart_ip Flexio UART IPL
*   @{
*/

#ifdef __cplusplus
extern "C"{
#endif

/*==================================================================================================
*                                          INCLUDE FILES
* 1) system and project includes
* 2) needed interfaces from external units
* 3) internal and external interfaces from this unit
==================================================================================================*/
#include "Flexio_Uart_Ip_HwAccess.h"
#include "Flexio_Uart_Ip_Irq.h"
#include "Flexio_Uart_Ip.h"
#include "SchM_Uart.h"
#ifdef FLEXIO_UART_IP_DEV_ERROR_DETECT
    #if (FLEXIO_UART_IP_DEV_ERROR_DETECT == STD_ON)
#include "devassert.h"
    #endif
#endif
#if (FLEXIO_UART_IP_HAS_DMA_ENABLED == STD_ON)
#include "Dma_Ip.h"
#endif
/*==================================================================================================
*                                 SOURCE FILE VERSION INFORMATION
==================================================================================================*/

#define FLEXIO_UART_IP_VENDOR_ID_C                      43
#define FLEXIO_UART_IP_AR_RELEASE_MAJOR_VERSION_C       4
#define FLEXIO_UART_IP_AR_RELEASE_MINOR_VERSION_C       4
#define FLEXIO_UART_IP_AR_RELEASE_REVISION_VERSION_C    0
#define FLEXIO_UART_IP_SW_MAJOR_VERSION_C               0
#define FLEXIO_UART_IP_SW_MINOR_VERSION_C               9
#define FLEXIO_UART_IP_SW_PATCH_VERSION_C               0

/*==================================================================================================
*                                       FILE VERSION CHECKS
==================================================================================================*/
/* Checks against Flexio_Uart_Ip.h */
#if (FLEXIO_UART_IP_VENDOR_ID_C != FLEXIO_UART_IP_VENDOR_ID)
    #error "Flexio_Uart_Ip.c and Flexio_Uart_Ip.h have different vendor ids"
#endif
#if ((FLEXIO_UART_IP_AR_RELEASE_MAJOR_VERSION_C    != FLEXIO_UART_IP_AR_RELEASE_MAJOR_VERSION) || \
     (FLEXIO_UART_IP_AR_RELEASE_MINOR_VERSION_C    != FLEXIO_UART_IP_AR_RELEASE_MINOR_VERSION) || \
     (FLEXIO_UART_IP_AR_RELEASE_REVISION_VERSION_C != FLEXIO_UART_IP_AR_RELEASE_REVISION_VERSION))
     #error "AUTOSAR Version Numbers of Flexio_Uart_Ip.c and Flexio_Uart_Ip.h are different"
#endif
#if ((FLEXIO_UART_IP_SW_MAJOR_VERSION_C != FLEXIO_UART_IP_SW_MAJOR_VERSION) || \
     (FLEXIO_UART_IP_SW_MINOR_VERSION_C != FLEXIO_UART_IP_SW_MINOR_VERSION) || \
     (FLEXIO_UART_IP_SW_PATCH_VERSION_C != FLEXIO_UART_IP_SW_PATCH_VERSION))
    #error "Software Version Numbers of Flexio_Uart_Ip.c and Flexio_Uart_Ip.h are different"
#endif

/* Checks against Flexio_Uart_Ip_HwAccess.h */
#if (FLEXIO_UART_IP_VENDOR_ID_C != FLEXIO_UART_IP_HWACCESS_VENDOR_ID)
    #error "Flexio_Uart_Ip.c and Flexio_Uart_Ip_Types.h have different vendor ids"
#endif
#if ((FLEXIO_UART_IP_AR_RELEASE_MAJOR_VERSION_C    != FLEXIO_UART_IP_HWACCESS_AR_RELEASE_MAJOR_VERSION) || \
     (FLEXIO_UART_IP_AR_RELEASE_MINOR_VERSION_C    != FLEXIO_UART_IP_HWACCESS_AR_RELEASE_MINOR_VERSION) || \
     (FLEXIO_UART_IP_AR_RELEASE_REVISION_VERSION_C != FLEXIO_UART_IP_HWACCESS_AR_RELEASE_REVISION_VERSION))
     #error "AUTOSAR Version Numbers of Flexio_Uart_Ip.c and Flexio_Uart_Ip_Types.h are different"
#endif
#if ((FLEXIO_UART_IP_SW_MAJOR_VERSION_C != FLEXIO_UART_IP_HWACCESS_SW_MAJOR_VERSION) || \
     (FLEXIO_UART_IP_SW_MINOR_VERSION_C != FLEXIO_UART_IP_HWACCESS_SW_MINOR_VERSION) || \
     (FLEXIO_UART_IP_SW_PATCH_VERSION_C != FLEXIO_UART_IP_HWACCESS_SW_PATCH_VERSION))
    #error "Software Version Numbers of Flexio_Uart_Ip.c and Flexio_Uart_Ip_Types.h are different"
#endif

/*==================================================================================================
*                           LOCAL TYPEDEFS (STRUCTURES, UNIONS, ENUMS)
==================================================================================================*/

/*==================================================================================================
*                                          LOCAL MACROS
==================================================================================================*/
#ifdef FLEXIO_UART_IP_DEV_ERROR_DETECT
    #if (FLEXIO_UART_IP_DEV_ERROR_DETECT == STD_ON)
        #define FLEXIO_UART_DEV_ASSERT(x)      DevAssert(x)
    #else
        #define FLEXIO_UART_DEV_ASSERT(x)      (void)(x)
    #endif
#endif

/* Shifters/Timers used for UART simulation The parameter x represents the
   u8ResourceIndex value for the current driver u32Channel */
#define TX_SHIFTER(x)    (x)
#define RX_SHIFTER(x)    (x)
#define TX_TIMER(x)      (x)
#define RX_TIMER(x)      (x)
/* @brief Address of the least significant byte or word in a 32-bit register (depends on endianness) */
#define FLEXIO_UART_IP_LSBW_ADDR(reg)  ((uint32)(&(reg)))

#if (FLEXIO_UART_IP_HAS_DMA_ENABLED == STD_ON)
    #define FLEXIO_UART_DMA_CONFIG_LIST_DIMENSION             (10U)
    #define FLEXIO_UART_DMA_LEAST_CONFIG_LIST_DIMENSION       (2U)
#endif
/*==================================================================================================
*                                         LOCAL CONSTANTS
==================================================================================================*/

/*==================================================================================================
                                       LOCAL VARIABLES
==================================================================================================*/
#define UART_START_SEC_VAR_NO_INIT_UNSPECIFIED_NO_CACHEABLE
/* @violates @ref Uart_c_REF_1 This violation is not fixed since the inclusion of Uart_MemMap.h is as per AUTOSAR requirement*/
#include "Uart_MemMap.h"

/** @brief Pointer to flexio runtime state structure */
Flexio_Uart_Ip_StateStructureType Flexio_Uart_Ip_apStateStructure[FEATURE_FLEXIO_MAX_SHIFTER_COUNT];
#define UART_STOP_SEC_VAR_NO_INIT_UNSPECIFIED_NO_CACHEABLE
/* @violates @ref Uart_c_REF_1 This violation is not fixed since the inclusion of Uart_MemMap.h is as per AUTOSAR requirement*/
#include "Uart_MemMap.h"

#ifdef FLEXIO_UART_IP_IS_USING
    #if (FLEXIO_UART_IP_IS_USING == STD_ON)
        #define UART_START_SEC_CONST_UNSPECIFIED
        /* @violates @ref Uart_c_REF_1 This violation is not fixed since the inclusion of Uart_MemMap.h is as per AUTOSAR requirement*/
        #include "Uart_MemMap.h"
        /** @brief Table of base addresses for FLEXIO instances. */
        static FLEXIO_Type * const Flexio_Uart_Ip_apBases[FLEXIO_INSTANCE_COUNT] = FLEXIO_BASE_PTRS;
        #define UART_STOP_SEC_CONST_UNSPECIFIED
        /* @violates @ref Uart_c_REF_1 This violation is not fixed since the inclusion of Uart_MemMap.h is as per AUTOSAR requirement*/
        #include "Uart_MemMap.h"
    #endif
#endif /* #ifdef FLEXIO_UART_IP_IS_USING */
/*==================================================================================================
*                                        GLOBAL CONSTANTS
==================================================================================================*/
#define UART_START_SEC_VAR_NO_INIT_UNSPECIFIED
/* @violates @ref Uart_c_REF_1 This violation is not fixed since the inclusion of Uart_MemMap.h is as per AUTOSAR requirement*/
#include "Uart_MemMap.h"

/** @brief User config structure. */
const Flexio_Uart_Ip_UserConfigType* Flexio_Uart_Ip_apUserConfig[FEATURE_FLEXIO_MAX_SHIFTER_COUNT];

#ifdef FLEXIO_UART_IP_IS_USING
    #if (FLEXIO_UART_IP_IS_USING == STD_ON)
        /** @brief Array of UART driver runtime state structures */
        static Flexio_Uart_Ip_StateStructureType * Flexio_Uart_Ip_apStateStructuresArray[FEATURE_FLEXIO_MAX_SHIFTER_COUNT];
    #endif /* (FLEXIO_UART_IP_IS_USING == STD_ON) */
#endif /* #ifdef FLEXIO_UART_IP_IS_USING */

#define UART_STOP_SEC_VAR_NO_INIT_UNSPECIFIED
/* @violates @ref Uart_c_REF_1 This violation is not fixed since the inclusion of Uart_MemMap.h is as per AUTOSAR requirement*/
#include "Uart_MemMap.h"
/*==================================================================================================
*                                        GLOBAL VARIABLES
==================================================================================================*/

/*==================================================================================================
*                                    LOCAL FUNCTION PROTOTYPES
==================================================================================================*/

/*==================================================================================================
*                                         LOCAL FUNCTIONS
==================================================================================================*/
#ifdef FLEXIO_UART_IP_IS_USING
#if (FLEXIO_UART_IP_IS_USING == STD_ON)
#define UART_START_SEC_CODE
/* @violates @ref Uart_c_REF_1 This violation is not fixed since the inclusion of Uart_MemMap.h is as per AUTOSAR requirement*/
#include "Uart_MemMap.h"
static void Flexio_Uart_Ip_ConfigureTx(uint32 u32Channel,
                                       Flexio_Uart_Ip_BitCountPerCharType eBitCount,
                                       uint8 u8Divider,
                                       uint8 u8DataPin,
                                       Flexio_Uart_Ip_TimerDecrementType eTimerDec);

static void Flexio_Uart_Ip_ConfigureRx(uint32 u32Channel,
                                       Flexio_Uart_Ip_BitCountPerCharType eBitCount,
                                       uint8 u8Divider,
                                       uint8 u8DataPin,
                                       Flexio_Uart_Ip_TimerDecrementType eTimerDec);
static void Flexio_Uart_Ip_StopTransfer(uint32 u32Channel);
static void Flexio_Uart_Ip_CheckTxOperation(uint32 u32Channel,
                                            uint8 u8ShifterMaskFlag,
                                            uint8 u8TimerMaskFlag);
static void Flexio_Uart_Ip_ReadData(uint32 u32Channel);
static void Flexio_Uart_Ip_CheckRxOperation(uint32 u32Channel,
                                            uint8 u8ShifterMaskFlag,
                                            uint8 u8ShifterErrMaskFlag);
static void Flexio_Uart_Ip_EnableTransfer(uint32 u32Channel);
static void Flexio_Uart_Ip_EndTransfer(uint32 u32Channel);
static void Flexio_Uart_Ip_WriteData(uint32 u32Channel);
#if (FLEXIO_UART_IP_HAS_DMA_ENABLED == STD_ON)
static Flexio_Uart_Ip_StatusType Flexio_Uart_Ip_StartSendDataUsingDma(uint32 u32Channel,
                                                                      const uint8 * pTxData,
                                                                      uint32 u32TxSize);
static Flexio_Uart_Ip_StatusType Flexio_Uart_Ip_StartReceiveDataUsingDma(uint32 u32Channel,
                                                                         uint8 * pRxBuff,
                                                                         uint32 u32RxSize);
#endif /*(FLEXIO_UART_IP_HAS_DMA_ENABLED == STD_ON)*/
/*==================================================================================================
*                                        GLOBAL FUNCTIONS
==================================================================================================*/
/*FUNCTION**********************************************************************
 *
 * Function Name : Flexio_Uart_Ip_OperationHandler
 * Description   : Implementation of FLEXIO interrupt handler named in start-up code
 *
 *END**************************************************************************/
/* implements     Flexio_Uart_Ip_IrqHandlerHandler_Activity*/
void Flexio_Uart_Ip_IrqHandler(uint8 u8Channel, uint8 u8ShifterMaskFlag, uint8 u8ShifterErrorMaskFlag, uint8 u8TimerMaskFlag)
{
    const Flexio_Uart_Ip_UserConfigType *pUartUserCfg;
    pUartUserCfg = (Flexio_Uart_Ip_UserConfigType*) Flexio_Uart_Ip_apUserConfig[u8Channel];
      /* There is an event for the current instance - call the isr */
    if (pUartUserCfg->eDirection == FLEXIO_UART_IP_DIRECTION_TX)
    {
        Flexio_Uart_Ip_CheckTxOperation((uint32)u8Channel,
                                        u8ShifterMaskFlag,
                                        u8TimerMaskFlag);
    }
    else
    {
        Flexio_Uart_Ip_CheckRxOperation((uint32)u8Channel,
                                        u8ShifterMaskFlag,
                                        u8ShifterErrorMaskFlag);
    }
}


/*FUNCTION**********************************************************************
 *
 * Function Name : Flexio_Uart_Ip_ConfigureTx
 * Description   : configures the FLEXIO module for UART
 *
 *END**************************************************************************/
static void Flexio_Uart_Ip_ConfigureTx(uint32 u32Channel,
                                       Flexio_Uart_Ip_BitCountPerCharType eBitCount,
                                       uint8 u8Divider,
                                       uint8 u8DataPin,
                                       Flexio_Uart_Ip_TimerDecrementType eTimerDec)
{
    FLEXIO_Type *pBase;
    uint16 u16Bits;
    pBase = Flexio_Uart_Ip_apBases[FLEXIO_HW_INSTANCE];
    u16Bits = (uint16)eBitCount;

    /* Configure tx shifter */
    Flexio_Uart_Ip_SetShifterConfig(pBase,
                            TX_SHIFTER(u32Channel),
                            FLEXIO_SHIFTER_START_BIT_0,
                            FLEXIO_SHIFTER_STOP_BIT_1,
                            FLEXIO_SHIFTER_SOURCE_PIN);
    Flexio_Uart_Ip_SetShifterControl(pBase,
                             TX_SHIFTER(u32Channel),
                             FLEXIO_SHIFTER_MODE_TRANSMIT,
                             u8DataPin,             /* Output on tx pin */
                             FLEXIO_PIN_POLARITY_HIGH,
                             FLEXIO_PIN_CONFIG_OUTPUT,
                             TX_TIMER(u32Channel),
                             FLEXIO_TIMER_POLARITY_POSEDGE);

    /* (Divider = InputClock / (BaudRate * 32U or 512U depend on baudrate) - 1; */
    Flexio_Uart_Ip_SetTimerCompare(pBase, TX_TIMER(u32Channel), (uint16)((((uint16)(u16Bits << 1U) - 1U) << 8U) + u8Divider));

    /* Configure tx timer */
    Flexio_Uart_Ip_SetTimerConfig(pBase,
                          TX_TIMER(u32Channel),
                          FLEXIO_TIMER_START_BIT_ENABLED,
                          FLEXIO_TIMER_STOP_BIT_TIM_DIS,
                          FLEXIO_TIMER_ENABLE_TRG_HIGH,         /* Enable when Tx data is available */
                          FLEXIO_TIMER_DISABLE_TIM_CMP,
                          FLEXIO_TIMER_RESET_NEVER,
                          eTimerDec, /* Decrement on FlexIO clock */
                          FLEXIO_TIMER_INITOUT_ONE);


    Flexio_Uart_Ip_SetTimerControl(pBase,
                           TX_TIMER(u32Channel),
                           (uint8)((uint8)(TX_SHIFTER(u32Channel) << 2U) + 1U), /* Trigger on tx shifter status flag */
                           FLEXIO_TRIGGER_POLARITY_LOW,
                           FLEXIO_TRIGGER_SOURCE_INTERNAL,
                           0U,                                     /* Pin unused */
                           FLEXIO_PIN_POLARITY_HIGH,
                           FLEXIO_PIN_CONFIG_DISABLED,
                           FLEXIO_TIMER_MODE_DISABLED);
}


/*FUNCTION**********************************************************************
 *
 * Function Name : Flexio_Uart_Ip_ConfigureRx
 * Description   : configures the FLEXIO module for UART
 *
 *END**************************************************************************/
static void Flexio_Uart_Ip_ConfigureRx(uint32 u32Channel,
                                       Flexio_Uart_Ip_BitCountPerCharType eBitCount,
                                       uint8 u8Divider,
                                       uint8 u8DataPin,
                                       Flexio_Uart_Ip_TimerDecrementType eTimerDec)
{
    FLEXIO_Type *pBase;
    uint16 u16Bits;
    pBase = Flexio_Uart_Ip_apBases[FLEXIO_HW_INSTANCE];
    u16Bits = (uint16)eBitCount;

    /* Configure rx shifter */
    Flexio_Uart_Ip_SetShifterConfig(pBase,
                            RX_SHIFTER(u32Channel),
                            FLEXIO_SHIFTER_START_BIT_0,
                            FLEXIO_SHIFTER_STOP_BIT_1,
                            FLEXIO_SHIFTER_SOURCE_PIN);
    Flexio_Uart_Ip_SetShifterControl(pBase,
                             RX_SHIFTER(u32Channel),
                             FLEXIO_SHIFTER_MODE_DISABLED,
                             u8DataPin,             /* Input from rx pin */
                             FLEXIO_PIN_POLARITY_HIGH,
                             FLEXIO_PIN_CONFIG_DISABLED,
                             RX_TIMER(u32Channel),
                             FLEXIO_TIMER_POLARITY_NEGEDGE);


    /* (Divider = InputClock / (BaudRate * 32U or 512U depend on baudrate) - 1; */
    Flexio_Uart_Ip_SetTimerCompare(pBase, RX_TIMER(u32Channel), (uint16)((((uint16)(u16Bits << 1U) - 1U) << 8U) + u8Divider));

    /* Configure rx timer */
    Flexio_Uart_Ip_SetTimerConfig(pBase,
                          RX_TIMER(u32Channel),
                          FLEXIO_TIMER_START_BIT_ENABLED,
                          FLEXIO_TIMER_STOP_BIT_TIM_DIS,
                          FLEXIO_TIMER_ENABLE_PIN_POSEDGE,            /* Enable when data is available */
                          FLEXIO_TIMER_DISABLE_TIM_CMP,
                          FLEXIO_TIMER_RESET_PIN_RISING,
                          eTimerDec,                                   /* Decrement on FlexIO clock */
                          FLEXIO_TIMER_INITOUT_ONE_RESET);

    Flexio_Uart_Ip_SetTimerControl(pBase,
                           RX_TIMER(u32Channel),
                           0U,                                            /* Trigger unused */
                           FLEXIO_TRIGGER_POLARITY_HIGH,
                           FLEXIO_TRIGGER_SOURCE_EXTERNAL,
                           u8DataPin,                                   /* Input from rx pin */
                           FLEXIO_PIN_POLARITY_LOW,
                           FLEXIO_PIN_CONFIG_DISABLED,
                           FLEXIO_TIMER_MODE_DISABLED);
}

/*FUNCTION**********************************************************************
 *
 * Function Name : Flexio_Uart_Ip_EndTransfer
 * Description   : End the current transfer
 *
 *END**************************************************************************/
static void Flexio_Uart_Ip_EndTransfer(uint32 u32Channel)
{
    FLEXIO_Type *pBase;
    Flexio_Uart_Ip_StateStructureType * pUartState;
    pUartState = (Flexio_Uart_Ip_StateStructureType *)Flexio_Uart_Ip_apStateStructuresArray[u32Channel];
    const Flexio_Uart_Ip_UserConfigType *pUartUserCfg;
    pUartUserCfg = (Flexio_Uart_Ip_UserConfigType*) Flexio_Uart_Ip_apUserConfig[u32Channel];
    pBase = Flexio_Uart_Ip_apBases[FLEXIO_HW_INSTANCE];

    /* Disable transfer engine */
    switch (pUartUserCfg->eDriverType)
    {
        case FLEXIO_UART_IP_DRIVER_TYPE_INTERRUPTS:
            /* Disable interrupts for Rx / Tx shifter */
            Flexio_Mcl_Ip_SetShifterInterrupt(pBase, (uint8)(1U << TX_SHIFTER(u32Channel)), FALSE);
            Flexio_Mcl_Ip_SetShifterErrorInterrupt(pBase, (uint8)(1U << TX_SHIFTER(u32Channel)), FALSE);
            /* Disable timer interrupt  */
            Flexio_Mcl_Ip_SetTimerInterrupt(pBase, (uint8)(1U << TX_TIMER(u32Channel)), FALSE);
            break;
        case FLEXIO_UART_IP_DRIVER_TYPE_DMA:
            /* For Tx we need to disable timer interrupt */
            Flexio_Mcl_Ip_SetTimerInterrupt(pBase, (uint8)(1U << TX_TIMER(u32Channel)), FALSE);
            /* Disable the FlexIO DMA request */
            Flexio_Mcl_Ip_SetShifterDMARequest(pBase, (uint8)(1U << TX_SHIFTER(u32Channel)), FALSE);
            break;
        default:
            /* Impossible type - do nothing */
            break;
    }

    pUartState->u32RemainingBytes = 0U;
    pUartState->bDriverIdle = FALSE;
}

/*FUNCTION**********************************************************************
 *
 * Function Name : Flexio_Uart_Ip_StopTransfer
 * Description   : Forcefully stops the current transfer
 *
 *END**************************************************************************/

static void Flexio_Uart_Ip_StopTransfer(uint32 u32Channel)
{
    FLEXIO_Type *pBase;
    const Flexio_Uart_Ip_UserConfigType *pUartUserCfg;
    pUartUserCfg = (Flexio_Uart_Ip_UserConfigType*) Flexio_Uart_Ip_apUserConfig[u32Channel];
    pBase = Flexio_Uart_Ip_apBases[FLEXIO_HW_INSTANCE];

    /* Disable and re-enable timers and shifters to reset them */
    Flexio_Uart_Ip_SetTimerMode(pBase, TX_TIMER(u32Channel), FLEXIO_TIMER_MODE_DISABLED);
    Flexio_Uart_Ip_SetShifterMode(pBase, TX_SHIFTER(u32Channel), FLEXIO_SHIFTER_MODE_DISABLED);

    /* Clear any leftover error flags */
    Flexio_Mcl_Ip_ClearShifterErrorStatus(pBase, TX_SHIFTER(u32Channel));

    /* End the transfer */
    Flexio_Uart_Ip_EndTransfer(u32Channel);

    /* Re-enable shifter for Tx, to ensure correct idle state */
    if (pUartUserCfg->eDirection == FLEXIO_UART_IP_DIRECTION_TX)
    {
        /* In tx mode restore start bit in case it was changed for end of transmission detection */
        Flexio_Uart_Ip_SetShifterStartBit(pBase, TX_SHIFTER(u32Channel), FLEXIO_SHIFTER_START_BIT_0);
        Flexio_Uart_Ip_SetShifterMode(pBase, TX_SHIFTER(u32Channel), FLEXIO_SHIFTER_MODE_TRANSMIT);
    }
}

/*FUNCTION**********************************************************************
 *
 * Function Name : Flexio_Uart_Ip_WriteData
 * Description   : Writes data to be transmitted by the module
 *
 *END**************************************************************************/
static void Flexio_Uart_Ip_WriteData(uint32 u32Channel)
{
    FLEXIO_Type *pBase;
    uint32 u32Data = 0UL;

    Flexio_Uart_Ip_StateStructureType * pUartState;
    pUartState = (Flexio_Uart_Ip_StateStructureType *)Flexio_Uart_Ip_apStateStructuresArray[u32Channel];
    const Flexio_Uart_Ip_UserConfigType *pUartUserCfg;
    pUartUserCfg = (Flexio_Uart_Ip_UserConfigType*) Flexio_Uart_Ip_apUserConfig[u32Channel];

    FLEXIO_UART_DEV_ASSERT(pUartState->pTxData != NULL_PTR);

    pBase = Flexio_Uart_Ip_apBases[FLEXIO_HW_INSTANCE];

    /* Read data from user buffer and update tx buffer pointer and remaining bytes count */
    if (pUartUserCfg->eBitCount == 8U)
    {
        u32Data = (uint32)(*(uint8 *)pUartState->pTxData);
        pUartState->pTxData++;
        pUartState->u32RemainingBytes -= 1U;
    }

    Flexio_Uart_Ip_WriteShifterBuffer(pBase, TX_SHIFTER(u32Channel), u32Data, FLEXIO_SHIFTER_RW_MODE_NORMAL);
}


/*FUNCTION**********************************************************************
 *
 * Function Name : Flexio_Uart_Ip_CheckTxOperation
 * Description   : Check status of the UART transmission. This function can be
 *                 called either in an interrupt routine or directly in polling
 *                 mode to advance the transfer.
 *
 *END**************************************************************************/

static void Flexio_Uart_Ip_CheckTxOperation(uint32 u32Channel,
                                         uint8 u8ShifterMaskFlag,
                                         uint8 u8TimerMaskFlag)
{
    FLEXIO_Type *pBase;
    Flexio_Uart_Ip_StateStructureType * pUartState;
    pUartState = (Flexio_Uart_Ip_StateStructureType *)Flexio_Uart_Ip_apStateStructuresArray[u32Channel];
    const Flexio_Uart_Ip_UserConfigType *pUartUserCfg;
    pUartUserCfg = (Flexio_Uart_Ip_UserConfigType*) Flexio_Uart_Ip_apUserConfig[u32Channel];
    pBase = Flexio_Uart_Ip_apBases[FLEXIO_HW_INSTANCE];
    uint32 u32ResourceMask = (1UL << u32Channel);

    /* No need to check for Tx underflow since timer is controlled by the shifter status flag */
    /* Check for transfer end */
    if (pUartState->u32RemainingBytes == 0U)
    {
        if (((uint32)u8TimerMaskFlag & u32ResourceMask) != 0U)
        {
            /*In case the Interrupt is spurious and the ISR shall only clear the interrupt status flag and return immediately*/
            if(!Flexio_Uart_Ip_GetTimerInterrupt(pBase,TX_TIMER(u32Channel)))
            {
               /* Clear timer status */
               Flexio_Mcl_Ip_ClearTimerStatus(pBase, TX_TIMER(u32Channel));
               return;
            }

            pUartState->u8TxFlush--;
            if (pUartState->u8TxFlush == 0U)
            {
                /* Clear Shifter status for transmit mode*/
                Flexio_Mcl_Ip_ClearShifterStatus(pBase, TX_SHIFTER(u32Channel));
                /* Done flushing the Tx buffer, end transfer */
                /* Record success if there was no error */
                if (pUartState->eStatus == FLEXIO_UART_IP_STATUS_BUSY)
                {
                    pUartState->eStatus = FLEXIO_UART_IP_STATUS_SUCCESS;
                }
                Flexio_Uart_Ip_StopTransfer(u32Channel);
                /* Call callback to announce the end transfer event to the user */
                if (pUartUserCfg->pfCallback != NULL_PTR)
                {
                    pUartUserCfg->pfCallback(u32Channel, pUartState, FLEXIO_UART_IP_EVENT_END_TRANSFER, pUartUserCfg->pCallbackParam);
                }
            }
            else if (Flexio_Mcl_Ip_GetShifterStatus(pBase, TX_SHIFTER(u32Channel)))
            {
                /* u8TxFlush == 1, but last byte was already transferred from buffer to shifter. There is a
                   danger that the transmission is over and we end up never reporting the end event.
                   To avoid this, send one extra dummy byte */
                /* Set start bit to 1 and send an 0xFF byte, this way the line will appear idle */
                Flexio_Uart_Ip_SetShifterStartBit(pBase, TX_SHIFTER(u32Channel), FLEXIO_SHIFTER_START_BIT_1);
                Flexio_Uart_Ip_WriteShifterBuffer(pBase, TX_SHIFTER(u32Channel), 0xFFFFFFFFUL, FLEXIO_SHIFTER_RW_MODE_NORMAL);
            }
            else
            {
                /* u8TxFlush == 1, and last byte was not yet transferred from buffer to shifter.
                   No need to do anything, just wait for the next timer event. */
            }
        }
    }
    /* Check if transmitter needs more data */
    else if ((((uint32)u8ShifterMaskFlag & u32ResourceMask) != 0U) && (pUartState->u32RemainingBytes > 0U))
    {
        /*In case the Interrupt is spurious and the ISR shall only clear the interrupt status flag and return immediately*/
        if(!Flexio_Uart_Ip_GetShifterInterrupt(pBase, TX_SHIFTER(u32Channel)))
        {
           /* Clear Shifter status for transmit mode*/
           Flexio_Mcl_Ip_ClearShifterStatus(pBase, TX_SHIFTER(u32Channel));
           return;
        }
        Flexio_Uart_Ip_WriteData(u32Channel);
        if (pUartState->u32RemainingBytes == 0U)
        {
            /* Out of data, call callback to allow user to provide a new buffer */
            if (pUartUserCfg->pfCallback != NULL_PTR)
            {
                pUartUserCfg->pfCallback(u32Channel, pUartState, FLEXIO_UART_IP_EVENT_TX_EMPTY, pUartUserCfg->pCallbackParam);
            }
        }
        if (pUartState->u32RemainingBytes == 0U)
        {
            /* No more data, transmission will stop after the last bytes are sent.
               The timer event will indicate when the send is complete */
            if (pUartUserCfg->eDriverType == FLEXIO_UART_IP_DRIVER_TYPE_INTERRUPTS)
            {
                /* Transmission completed; disable interrupt */
                Flexio_Mcl_Ip_SetShifterInterrupt(pBase, (uint8)(1U << TX_SHIFTER(u32Channel)), FALSE);
                /* Enable timer interrupt to ensure that transfer is completed */
                Flexio_Mcl_Ip_SetTimerInterrupt(pBase, (uint8)(1U << TX_TIMER(u32Channel)), TRUE);
            }
        }
    }
    else
    {
        /* No relevant events - nothing to do */
    }
}

/*FUNCTION**********************************************************************
 *
 * Function Name : Flexio_Uart_Ip_ReadData
 * Description   : reads data received by the module
 *
 *END**************************************************************************/

static void Flexio_Uart_Ip_ReadData(uint32 u32Channel)
{
    const FLEXIO_Type *pBase;
    uint32 u32Data;
    Flexio_Uart_Ip_StateStructureType * pUartState;
    pUartState = (Flexio_Uart_Ip_StateStructureType *)Flexio_Uart_Ip_apStateStructuresArray[u32Channel];
    const Flexio_Uart_Ip_UserConfigType *pUartUserCfg;
    pUartUserCfg = (Flexio_Uart_Ip_UserConfigType*) Flexio_Uart_Ip_apUserConfig[u32Channel];
    FLEXIO_UART_DEV_ASSERT(pUartState->u32RemainingBytes > 0U);
    FLEXIO_UART_DEV_ASSERT(pUartState->pRxData != NULL_PTR);

    pBase = Flexio_Uart_Ip_apBases[FLEXIO_HW_INSTANCE];

    /* Read data from shifter buffer */
    u32Data = Flexio_Uart_Ip_ReadShifterBuffer(pBase, RX_SHIFTER(u32Channel), FLEXIO_SHIFTER_RW_MODE_NORMAL);
    u32Data >>= 32U - (uint32)(pUartUserCfg->eBitCount);

    if (pUartUserCfg->eBitCount == 8U)
    {
        *(uint8 *)pUartState->pRxData = (uint8)u32Data;
        /* Update rx buffer pointer and remaining bytes count */
        pUartState->pRxData ++;
        pUartState->u32RemainingBytes -= 1U;
    }
}

/*FUNCTION**********************************************************************
 *
 * Function Name : Flexio_Uart_Ip_CheckRxOperation
 * Description   : Check status of the UART reception. This function can be
 *                 called either in an interrupt routine or directly in polling
 *                 mode to advance the transfer.
 *
 *END**************************************************************************/

static void Flexio_Uart_Ip_CheckRxOperation(uint32 u32Channel,
                                         uint8 u8ShifterMaskFlag,
                                         uint8 u8ShifterErrMaskFlag)
{
    FLEXIO_Type * pBase;
    Flexio_Uart_Ip_StateStructureType * pUartState;
    pUartState = (Flexio_Uart_Ip_StateStructureType *)Flexio_Uart_Ip_apStateStructuresArray[u32Channel];
    const Flexio_Uart_Ip_UserConfigType *pUartUserCfg;
    pUartUserCfg = (Flexio_Uart_Ip_UserConfigType*) Flexio_Uart_Ip_apUserConfig[u32Channel];
    pBase = Flexio_Uart_Ip_apBases[FLEXIO_HW_INSTANCE];
    uint32 u32ResourceMask = (1UL << u32Channel);

    /* Check for errors */
    if (((uint32)u8ShifterErrMaskFlag & u32ResourceMask) != 0U)
    {
        /*In case the Interrupt is spurious and the ISR shall only clear the interrupt status flag and return immediately*/
        if(!Flexio_Uart_Ip_GetShifterErrorInterrupt(pBase, RX_SHIFTER(u32Channel)))
        {
            /*Clear Error Status flag*/
            Flexio_Mcl_Ip_ClearShifterErrorStatus(pBase, RX_SHIFTER(u32Channel));
            return;
        }
        pUartState->eStatus = FLEXIO_UART_IP_STATUS_RX_OVERRUN;
        pUartState->u32RemainingBytes = 0U;
        /* Invoke callback if there is one */
        if (pUartUserCfg->pfErrorCallback != NULL_PTR)
        {
            pUartUserCfg->pfErrorCallback(u32Channel, pUartState, FLEXIO_UART_IP_EVENT_ERROR, pUartUserCfg->pErrorCallbackParam);
        }
        /* Continue processing events */
    }
    /* Check if data was received */
    else if (((uint32)u8ShifterMaskFlag & u32ResourceMask) != 0U)
    {
        /*In case the Interrupt is spurious and the ISR shall only clear the interrupt status flag and return immediately*/
        if(!Flexio_Uart_Ip_GetShifterInterrupt(pBase, RX_SHIFTER(u32Channel)))
        {
             /*Clear Shifter Status flag*/
            Flexio_Mcl_Ip_ClearShifterStatus(pBase, RX_SHIFTER(u32Channel));
            return;
        }
        Flexio_Uart_Ip_ReadData(u32Channel);
        if (pUartState->u32RemainingBytes == 0U)
        {
            /* Out of data, call callback to allow user to provide a new buffer */
            if (pUartUserCfg->pfCallback != NULL_PTR)
            {
                pUartUserCfg->pfCallback(u32Channel, pUartState, FLEXIO_UART_IP_EVENT_RX_FULL, pUartUserCfg->pCallbackParam);
            }
        }
    }
    else
    {
        /* No events - nothing to do */
    }
    /* Check if transfer is over */
    if (pUartState->u32RemainingBytes == 0U)
    {
        /* Record success if there was no error */
        if (pUartState->eStatus == FLEXIO_UART_IP_STATUS_BUSY)
        {
            pUartState->eStatus = FLEXIO_UART_IP_STATUS_SUCCESS;
        }
        /* Discard any leftover rx data */
        Flexio_Mcl_Ip_ClearShifterStatus(pBase, RX_SHIFTER(u32Channel));
        /* End transfer */
        Flexio_Uart_Ip_StopTransfer(u32Channel);
        /* Call callback to announce the event to the user */
        if (pUartUserCfg->pfCallback != NULL_PTR)
        {
            pUartUserCfg->pfCallback(u32Channel, pUartState, FLEXIO_UART_IP_EVENT_END_TRANSFER, pUartUserCfg->pCallbackParam);
        }
    }
}

/*FUNCTION**********************************************************************
 *
 * Function Name : Flexio_Uart_Ip_EnableTransfer
 * Description   : Enables timers and shifters to start a transfer
 *
 *END**************************************************************************/

static void Flexio_Uart_Ip_EnableTransfer(uint32 u32Channel)
{
    FLEXIO_Type * pBase;
    const Flexio_Uart_Ip_UserConfigType *pUartUserCfg;
    pUartUserCfg = (Flexio_Uart_Ip_UserConfigType*) Flexio_Uart_Ip_apUserConfig[u32Channel];
    pBase = Flexio_Uart_Ip_apBases[FLEXIO_HW_INSTANCE];

    /* Enable timers and shifters */
    if (pUartUserCfg->eDirection == FLEXIO_UART_IP_DIRECTION_RX)
    {
        /* In rx mode, discard any leftover rx. data */
        Flexio_Mcl_Ip_ClearShifterStatus(pBase, RX_SHIFTER(u32Channel));
        Flexio_Uart_Ip_SetShifterMode(pBase, RX_SHIFTER(u32Channel), FLEXIO_SHIFTER_MODE_RECEIVE);
    }
    Flexio_Uart_Ip_SetTimerMode(pBase, TX_TIMER(u32Channel), FLEXIO_TIMER_MODE_8BIT_BAUD);
}

#if (FLEXIO_UART_IP_HAS_DMA_ENABLED == STD_ON)

/*FUNCTION**********************************************************************
 *
 * Function Name : Flexio_Uart_Ip_StartSendDataUsingDma
 * Description   : Initiate (start) a transmit by beginning the process of
 * sending data using DMA transfers.
 * This is not a public API as it is called from other driver functions.
 *
 *END**************************************************************************/
static Flexio_Uart_Ip_StatusType Flexio_Uart_Ip_StartSendDataUsingDma(uint32 u32Channel,
                                                                      const uint8 * pTxData,
                                                                      uint32 u32TxSize)
{
    Flexio_Uart_Ip_StateStructureType * pUartState;
    const Flexio_Uart_Ip_UserConfigType *pUserConfig;
    FLEXIO_Type * pBase;
    Dma_Ip_LogicChannelTransferListType dmaTransferList[FLEXIO_UART_DMA_CONFIG_LIST_DIMENSION];
    Dma_Ip_ReturnType eDmaReturnStatus;

    pUartState = (Flexio_Uart_Ip_StateStructureType *)Flexio_Uart_Ip_apStateStructuresArray[u32Channel];
    pUserConfig = (Flexio_Uart_Ip_UserConfigType*) Flexio_Uart_Ip_apUserConfig[u32Channel];
    pBase = Flexio_Uart_Ip_apBases[FLEXIO_HW_INSTANCE];

    /* Set up parameters for Dma_Ip_LogicChannelTransferListType */
    dmaTransferList[0].Param = DMA_IP_CH_SET_SOURCE_ADDRESS;
    dmaTransferList[0].Value = (uint32)pTxData;
    dmaTransferList[1].Param = DMA_IP_CH_SET_DESTINATION_ADDRESS;
    dmaTransferList[1].Value = FLEXIO_UART_IP_LSBW_ADDR(pBase->SHIFTBUF[u32Channel]);
    dmaTransferList[2].Param = DMA_IP_CH_SET_SOURCE_SIGNED_OFFSET;
    dmaTransferList[2].Value = 1;
    dmaTransferList[3].Param = DMA_IP_CH_SET_DESTINATION_SIGNED_OFFSET;
    dmaTransferList[3].Value = 0;
    dmaTransferList[4].Param = DMA_IP_CH_SET_MAJORLOOP_COUNT;
    dmaTransferList[4].Value = u32TxSize;
    dmaTransferList[5].Param = DMA_IP_CH_SET_MINORLOOP_SIZE;
    dmaTransferList[5].Value = 1;
    dmaTransferList[6].Param = DMA_IP_CH_SET_DESTINATION_TRANSFER_SIZE;
    dmaTransferList[6].Value = DMA_IP_TRANSFER_SIZE_1_BYTE;
    dmaTransferList[7].Param = DMA_IP_CH_SET_SOURCE_TRANSFER_SIZE;
    dmaTransferList[7].Value = DMA_IP_TRANSFER_SIZE_1_BYTE;
    dmaTransferList[8].Param = DMA_IP_CH_SET_CONTROL_EN_MAJOR_INTERRUPT;
    dmaTransferList[8].Value = 1;
    dmaTransferList[9].Param = DMA_IP_CH_SET_CONTROL_DIS_AUTO_REQUEST;
    dmaTransferList[9].Value = 1;

    /* Update state structure */
    pUartState->pTxData = pTxData;
    pUartState->u32RemainingBytes = 0U;
    pUartState->eStatus = FLEXIO_UART_IP_STATUS_BUSY;

    /* Configure the transfer control descriptor for the DMA channel */
    eDmaReturnStatus = Dma_Ip_SetLogicChannelTransferList(pUserConfig->u8DmaChannel, dmaTransferList, FLEXIO_UART_DMA_CONFIG_LIST_DIMENSION);
    FLEXIO_UART_DEV_ASSERT(eDmaReturnStatus == DMA_IP_STATUS_SUCCESS);

    /* Start the DMA channel */
    eDmaReturnStatus = Dma_Ip_SetLogicChannelCommand(pUserConfig->u8DmaChannel, DMA_IP_CH_SET_HARDWARE_REQUEST);
    FLEXIO_UART_DEV_ASSERT(eDmaReturnStatus == DMA_IP_STATUS_SUCCESS);
    /* Enable FlexIO DMA requests */
    Flexio_Mcl_Ip_SetShifterDMARequest(pBase, (uint8)(1U << TX_SHIFTER(u32Channel)), TRUE);

    return FLEXIO_UART_IP_STATUS_SUCCESS;
}

/*FUNCTION**********************************************************************
 *
 * Function Name : Flexio_Uart_Ip_StartReceiveDataUsingDma
 * Description   : Initiate (start) a transmit by beginning the process of
 * receiving data using DMA transfers.
 * This is not a public API as it is called from other driver functions.
 *
 *END**************************************************************************/
static Flexio_Uart_Ip_StatusType Flexio_Uart_Ip_StartReceiveDataUsingDma(uint32 u32Channel,
                                                                         uint8 * pRxBuff,
                                                                         uint32 u32RxSize)
{
    Flexio_Uart_Ip_StateStructureType * pUartState;
    const Flexio_Uart_Ip_UserConfigType *pUserConfig;
    FLEXIO_Type * pBase;
    Dma_Ip_LogicChannelTransferListType dmaTransferList[FLEXIO_UART_DMA_CONFIG_LIST_DIMENSION];
    Dma_Ip_ReturnType eDmaReturnStatus;

    pUartState = (Flexio_Uart_Ip_StateStructureType *)Flexio_Uart_Ip_apStateStructuresArray[u32Channel];
    pUserConfig = (Flexio_Uart_Ip_UserConfigType*) Flexio_Uart_Ip_apUserConfig[u32Channel];
    pBase = Flexio_Uart_Ip_apBases[FLEXIO_HW_INSTANCE];

    /* Set up parameters for Dma_Ip_LogicChannelTransferListType */
    dmaTransferList[0].Param = DMA_IP_CH_SET_SOURCE_ADDRESS;
    dmaTransferList[0].Value = FLEXIO_UART_IP_LSBW_ADDR(pBase->SHIFTBUF[u32Channel]) + 3U;
    dmaTransferList[1].Param = DMA_IP_CH_SET_DESTINATION_ADDRESS;
    dmaTransferList[1].Value = (uint32)pRxBuff;
    dmaTransferList[2].Param = DMA_IP_CH_SET_SOURCE_SIGNED_OFFSET;
    dmaTransferList[2].Value = 0;
    dmaTransferList[3].Param = DMA_IP_CH_SET_DESTINATION_SIGNED_OFFSET;
    dmaTransferList[3].Value = 1;
    dmaTransferList[4].Param = DMA_IP_CH_SET_MAJORLOOP_COUNT;
    dmaTransferList[4].Value = u32RxSize;
    dmaTransferList[5].Param = DMA_IP_CH_SET_MINORLOOP_SIZE;
    dmaTransferList[5].Value = 1;
    dmaTransferList[6].Param = DMA_IP_CH_SET_DESTINATION_TRANSFER_SIZE;
    dmaTransferList[6].Value = DMA_IP_TRANSFER_SIZE_1_BYTE;
    dmaTransferList[7].Param = DMA_IP_CH_SET_SOURCE_TRANSFER_SIZE;
    dmaTransferList[7].Value = DMA_IP_TRANSFER_SIZE_1_BYTE;
    dmaTransferList[8].Param = DMA_IP_CH_SET_CONTROL_EN_MAJOR_INTERRUPT;
    dmaTransferList[8].Value = 1;
    dmaTransferList[9].Param = DMA_IP_CH_SET_CONTROL_DIS_AUTO_REQUEST;
    dmaTransferList[9].Value = 1;

    /* Update state structure */
    pUartState->pRxData = pRxBuff;
    pUartState->u32RemainingBytes = 0U;
    pUartState->eStatus = FLEXIO_UART_IP_STATUS_BUSY;

    /* Configure the transfer control descriptor for the DMA channel */
    eDmaReturnStatus = Dma_Ip_SetLogicChannelTransferList(pUserConfig->u8DmaChannel, dmaTransferList, FLEXIO_UART_DMA_CONFIG_LIST_DIMENSION);
    FLEXIO_UART_DEV_ASSERT(eDmaReturnStatus == DMA_IP_STATUS_SUCCESS);

    /* Start the DMA channel */
    eDmaReturnStatus = Dma_Ip_SetLogicChannelCommand(pUserConfig->u8DmaChannel, DMA_IP_CH_SET_HARDWARE_REQUEST);
    FLEXIO_UART_DEV_ASSERT(eDmaReturnStatus == DMA_IP_STATUS_SUCCESS);

    /* Enable FlexIO DMA requests */
    Flexio_Mcl_Ip_SetShifterDMARequest(pBase, (uint8)(1U << RX_SHIFTER(u32Channel)), TRUE);

    return FLEXIO_UART_IP_STATUS_SUCCESS;
}
#endif

/*==================================================================================================
*                                        GLOBAL FUNCTIONS
==================================================================================================*/

/*FUNCTION**********************************************************************
 *
 * Function Name : Flexio_Uart_Ip_Init
 * Description   : Initialize the FLEXIO_UART driver
 * Implements : Flexio_Uart_Ip_Init_Activity
 *
 *END**************************************************************************/
/* implements     Flexio_Uart_Ip_Init_Activity*/
void Flexio_Uart_Ip_Init(uint32 u32Channel,
                         const Flexio_Uart_Ip_UserConfigType * pUserConfig)
{
    Flexio_Uart_Ip_StateStructureType *pUartStatePtr;
    pUartStatePtr = pUserConfig->pStateStruct;

    FLEXIO_UART_DEV_ASSERT(Flexio_Uart_Ip_apStateStructuresArray[u32Channel] == NULL_PTR);
    FLEXIO_UART_DEV_ASSERT(u32Channel < FEATURE_FLEXIO_MAX_SHIFTER_COUNT);
    Flexio_Uart_Ip_apStateStructuresArray[u32Channel] = pUserConfig->pStateStruct;
    Flexio_Uart_Ip_apUserConfig[u32Channel] =  pUserConfig;

    /* Initialize driver-specific context structure */
    pUartStatePtr->pRxData = NULL_PTR;
    pUartStatePtr->pTxData = NULL_PTR;
    pUartStatePtr->u32RemainingBytes = 0U;
    pUartStatePtr->u32BaudRate = pUserConfig->u32BaudRate;
    pUartStatePtr->eStatus = FLEXIO_UART_IP_STATUS_SUCCESS;
    pUartStatePtr->bDriverIdle = FALSE;

    if (pUserConfig->eDirection == FLEXIO_UART_IP_DIRECTION_TX)
    {
        /* Configure device for UART Tx mode */
        Flexio_Uart_Ip_ConfigureTx(u32Channel,
                           pUserConfig->eBitCount,
                           pUserConfig->u8Divider,
                           pUserConfig->u8DataPin,
                           pUserConfig->eTimerDec);
    }
    else
    {
        /* Configure device for UART Rx mode */
        Flexio_Uart_Ip_ConfigureRx(u32Channel,
                           pUserConfig->eBitCount,
                           pUserConfig->u8Divider,
                           pUserConfig->u8DataPin,
                           pUserConfig->eTimerDec);
    }
}


/*FUNCTION**********************************************************************
 *
 * Function Name : Flexio_Uart_Ip_GetBaudRate
 * Description   : Returns the FLEXIO configured baud rate.
 *
 * Implements    : Flexio_Uart_Ip_GetBaudRate_Activity
 *END**************************************************************************/
/* implements     Flexio_Uart_Ip_GetBaudRate_Activity*/
void Flexio_Uart_Ip_GetBaudRate(uint32 u32Channel, uint32 * pConfiguredBaudRate)
{
    FLEXIO_UART_DEV_ASSERT(u32Channel < FEATURE_FLEXIO_MAX_SHIFTER_COUNT);
    FLEXIO_UART_DEV_ASSERT(pConfiguredBaudRate != NULL_PTR);
    Flexio_Uart_Ip_StateStructureType * pUartState;
    pUartState = (Flexio_Uart_Ip_StateStructureType *)Flexio_Uart_Ip_apStateStructuresArray[u32Channel];

    *pConfiguredBaudRate = pUartState->u32BaudRate;
}



/*FUNCTION**********************************************************************
 *
 * Function Name : Flexio_Uart_Ip_Deinit
 * Description   : De-initialize the FLEXIO_UART driver
 * Implements : Flexio_Uart_Ip_Deinit_Activity
 *
 *END**************************************************************************/
/* implements     Flexio_Uart_Ip_Deinit_Activity*/
void Flexio_Uart_Ip_Deinit(uint32 u32Channel)
{
    FLEXIO_UART_DEV_ASSERT(u32Channel < FEATURE_FLEXIO_MAX_SHIFTER_COUNT);
    uint32 u32StartTime;
    FLEXIO_Type *pBase;

    pBase = Flexio_Uart_Ip_apBases[FLEXIO_HW_INSTANCE];
    FLEXIO_UART_DEV_ASSERT(Flexio_Uart_Ip_apStateStructuresArray[u32Channel] != NULL_PTR);
    u32StartTime = OsIf_GetCounter(FLEXIO_UART_IP_TIMEOUT_TYPE);
    /* Wait until the data is completely shifted out of shift register */
    while ((Flexio_Uart_Ip_GetStatus(u32Channel, NULL_PTR) == FLEXIO_UART_IP_STATUS_BUSY) && \
           !Flexio_Uart_Ip_CheckTimeout(u32StartTime, (uint32)FLEXIO_UART_IP_TIMEOUT_VALUE_US))
    {}
    /* Clear our saved pointer to the state structure */
    Flexio_Uart_Ip_apStateStructuresArray[u32Channel] = NULL_PTR;
    /* Disable interrupts for Tx or Rx shifter */
    Flexio_Mcl_Ip_SetShifterInterrupt(pBase, (uint8)(1U << u32Channel), FALSE);
    Flexio_Mcl_Ip_SetShifterErrorInterrupt(pBase, (uint8)(1U << u32Channel), FALSE);
    /* Reset FlexIO module */
    Flexio_Mcl_Ip_SetSoftwareReset(pBase, TRUE);
  /* Clear reset bit */
  Flexio_Mcl_Ip_SetSoftwareReset(pBase, FALSE);
}

/*FUNCTION**********************************************************************
 *
 * Function Name : Flexio_Uart_Ip_AsyncSend
 * Description   : Perform a Async UART transmission
 * Implements : Flexio_Uart_Ip_AsyncSendData_Activity
 *
 *END**************************************************************************/
 /* implements     Flexio_Uart_Ip_AsyncSend_Activity*/
Flexio_Uart_Ip_StatusType Flexio_Uart_Ip_AsyncSend(uint32 u32Channel,
                                                  const uint8 * pTxBuff,
                                                  uint32 u32TxSize)
{
    FLEXIO_UART_DEV_ASSERT(u32Channel < FEATURE_FLEXIO_MAX_SHIFTER_COUNT);
    FLEXIO_Type *pBase;
    Flexio_Uart_Ip_StateStructureType * pUartState;
    pUartState = (Flexio_Uart_Ip_StateStructureType *)Flexio_Uart_Ip_apStateStructuresArray[u32Channel];
    const Flexio_Uart_Ip_UserConfigType *pUartUserCfg;
    pUartUserCfg = (Flexio_Uart_Ip_UserConfigType*) Flexio_Uart_Ip_apUserConfig[u32Channel];

    FLEXIO_UART_DEV_ASSERT(u32Channel < FEATURE_FLEXIO_MAX_SHIFTER_COUNT);
    FLEXIO_UART_DEV_ASSERT(pUartState != NULL_PTR);
    FLEXIO_UART_DEV_ASSERT(pTxBuff != NULL_PTR);
    FLEXIO_UART_DEV_ASSERT(u32TxSize > 0U);
    FLEXIO_UART_DEV_ASSERT(pUartUserCfg->eDirection == FLEXIO_UART_IP_DIRECTION_TX);
    /* If 2 bytes per word are needed, then the size must be even */
    FLEXIO_UART_DEV_ASSERT((pUartUserCfg->eBitCount <= 8U) || ((u32TxSize & 1U) == 0U));
    /* Check if driver is busy */
    SchM_Enter_Uart_UART_EXCLUSIVE_AREA_04();
    if (pUartState->bDriverIdle)
    {
        SchM_Exit_Uart_UART_EXCLUSIVE_AREA_04();
        return FLEXIO_UART_IP_STATUS_BUSY;
    }
    pUartState->bDriverIdle = TRUE;
    SchM_Exit_Uart_UART_EXCLUSIVE_AREA_04();
    pBase = Flexio_Uart_Ip_apBases[FLEXIO_HW_INSTANCE];

    pUartState->pTxData = (uint8 *)pTxBuff;
    pUartState->u32RemainingBytes = u32TxSize;
    pUartState->eStatus = FLEXIO_UART_IP_STATUS_BUSY;
    /* Number of bytes to flush after the last byte is copied in the tx shifter buffer */
    pUartState->u8TxFlush = (uint8)((u32TxSize == 1U) ? 1U : 2U);

    /* Enable timers and shifters */
    Flexio_Uart_Ip_EnableTransfer(u32Channel);
    /* Enable transfer engine */
    switch (pUartUserCfg->eDriverType)
    {
        case FLEXIO_UART_IP_DRIVER_TYPE_INTERRUPTS:

            /* Enable interrupts for Tx shifter */
            Flexio_Mcl_Ip_SetShifterInterrupt(pBase, (uint8)(1U << TX_SHIFTER(u32Channel)), TRUE);
            Flexio_Mcl_Ip_SetShifterErrorInterrupt(pBase, (uint8)(1U << TX_SHIFTER(u32Channel)), TRUE);
            break;
#if (FLEXIO_UART_IP_HAS_DMA_ENABLED == STD_ON)
        case FLEXIO_UART_IP_DRIVER_TYPE_DMA:
            Flexio_Uart_Ip_StartSendDataUsingDma(u32Channel, pTxBuff, u32TxSize);
            break;
#endif
        default:
            /* Impossible type - do nothing */
            break;
    }

    return FLEXIO_UART_IP_STATUS_SUCCESS;
}

/*FUNCTION**********************************************************************
 *
 * Function Name : Flexio_Uart_Ip_GetStatus
 * Description   : Get the status of the current non-blocking UART transaction
 * Implements : Flexio_Uart_Ip_GetStatus_Activity
 *
 *END**************************************************************************/
Flexio_Uart_Ip_StatusType Flexio_Uart_Ip_GetStatus(uint32 u32Channel, uint32 *pBytesRemaining)
{
    FLEXIO_UART_DEV_ASSERT(u32Channel < FEATURE_FLEXIO_MAX_SHIFTER_COUNT);
    Flexio_Uart_Ip_StateStructureType * pUartState;
    pUartState = (Flexio_Uart_Ip_StateStructureType *)Flexio_Uart_Ip_apStateStructuresArray[u32Channel];
    const Flexio_Uart_Ip_UserConfigType *pUartUserCfg;
    pUartUserCfg = (Flexio_Uart_Ip_UserConfigType*) Flexio_Uart_Ip_apUserConfig[u32Channel];
#if (FLEXIO_UART_IP_HAS_DMA_ENABLED == STD_ON)
    const Dma_Ip_LogicChannelInfoParamType eDmaLogicChnParam = DMA_IP_CH_GET_CURRENT_ITER_COUNT;
#endif

    FLEXIO_UART_DEV_ASSERT(pUartState != NULL_PTR);

    if (pBytesRemaining != NULL_PTR)
    {
        if (pUartState->bDriverIdle)
        {
            /* Fill in the bytes not transferred yet. */
            if (pUartUserCfg->eDriverType == FLEXIO_UART_IP_DRIVER_TYPE_INTERRUPTS)
            {
                *pBytesRemaining = pUartState->u32RemainingBytes;
            }
#if (FLEXIO_UART_IP_HAS_DMA_ENABLED == STD_ON)
            else
            {
                /* In DMA-based communication, the remaining bytes are retrieved
                 * from the current DMA major loop count
                 */
                Dma_Ip_GetLogicChannelParam(pUartUserCfg->u8DmaChannel, eDmaLogicChnParam, pBytesRemaining);
            }
#endif
        }
        else
        {
            *pBytesRemaining = 0;
        }
    }

        return pUartState->eStatus;
}

/*FUNCTION**********************************************************************
 *
 * Function Name : Flexio_Uart_Ip_SyncSend
 * Description   : Perform a non-blocking UART transmission
 * Implements : Flexio_Uart_Ip_SyncSendData_Activity
 *
 *END**************************************************************************/
/* implements     Flexio_Uart_Ip_SyncSend_Activity*/
Flexio_Uart_Ip_StatusType Flexio_Uart_Ip_SyncSend(uint32 u32Channel,
                                                  const uint8 * pTxBuff,
                                                  uint32 u32TxSize)
{
    FLEXIO_UART_DEV_ASSERT(u32Channel < FEATURE_FLEXIO_MAX_SHIFTER_COUNT);
    FLEXIO_UART_DEV_ASSERT(pTxBuff != NULL_PTR);
    FLEXIO_UART_DEV_ASSERT(u32TxSize > 0U);
    FLEXIO_Type *pBase;
    uint32 u32StartTime;
    uint32 u32ElapsedTime;
    uint32 u32TimeoutTicks;
    Flexio_Uart_Ip_StateStructureType * pUartState;
    const Flexio_Uart_Ip_UserConfigType *pUartUserCfg;

    pUartState = (Flexio_Uart_Ip_StateStructureType *)Flexio_Uart_Ip_apStateStructuresArray[u32Channel];
    pUartUserCfg = (Flexio_Uart_Ip_UserConfigType*) Flexio_Uart_Ip_apUserConfig[u32Channel];
    pBase = Flexio_Uart_Ip_apBases[FLEXIO_HW_INSTANCE];
    FLEXIO_UART_DEV_ASSERT(pUartState != NULL_PTR);
    FLEXIO_UART_DEV_ASSERT(pUartUserCfg->eDirection == FLEXIO_UART_IP_DIRECTION_TX);
    /* If 2 bytes per word are needed, then the size must be even */
    FLEXIO_UART_DEV_ASSERT((pUartUserCfg->eBitCount <= 8U) || ((u32TxSize & 1U) == 0U));

    /* Check if driver is busy */
    SchM_Enter_Uart_UART_EXCLUSIVE_AREA_05();
    if (pUartState->bDriverIdle)
    {
        SchM_Exit_Uart_UART_EXCLUSIVE_AREA_05();
        return FLEXIO_UART_IP_STATUS_BUSY;
    }
    pUartState->bDriverIdle = TRUE;
    SchM_Exit_Uart_UART_EXCLUSIVE_AREA_05();
    pUartState->pTxData = (uint8 *)pTxBuff;
    pUartState->u32RemainingBytes = u32TxSize;

    pUartState->eStatus = FLEXIO_UART_IP_STATUS_BUSY;
    /* Disable interrupts for Tx shifter */
    Flexio_Mcl_Ip_SetShifterInterrupt(pBase, (uint8)(1U << TX_SHIFTER(u32Channel)), FALSE);
    Flexio_Mcl_Ip_SetShifterErrorInterrupt(pBase, (uint8)(1U << TX_SHIFTER(u32Channel)), FALSE);

    /* Number of bytes to flush after the last byte is copied in the tx shifter buffer */
    pUartState->u8TxFlush = (uint8)((u32TxSize == 1U) ? 1U : 2U);

    /* Enable timers and shifters */
    Flexio_Uart_Ip_EnableTransfer(u32Channel);
    u32StartTime = OsIf_GetCounter(FLEXIO_UART_IP_TIMEOUT_TYPE);

    while ((pUartState->u32RemainingBytes > 0U) && \
            !Flexio_Uart_Ip_CheckTimeout(u32StartTime, (uint32)FLEXIO_UART_IP_TIMEOUT_VALUE_US))
    {
        Flexio_Uart_Ip_WriteData(u32Channel);
        while (!Flexio_Mcl_Ip_GetShifterStatus(pBase, u32Channel)  && \
               !Flexio_Uart_Ip_CheckTimeout(u32StartTime, (uint32)FLEXIO_UART_IP_TIMEOUT_VALUE_US))
        {}
    }

    u32ElapsedTime  = OsIf_GetElapsed(&u32StartTime, FLEXIO_UART_IP_TIMEOUT_TYPE);
    u32TimeoutTicks = OsIf_MicrosToTicks((uint32)FLEXIO_UART_IP_TIMEOUT_VALUE_US, FLEXIO_UART_IP_TIMEOUT_TYPE);

    /* Check if Timeout occur */
    if (u32ElapsedTime >= u32TimeoutTicks)
    {
        pUartState->eStatus = FLEXIO_UART_IP_STATUS_TIMEOUT;
    }

    /* Check other success receiving case*/
    if (pUartState->eStatus == FLEXIO_UART_IP_STATUS_BUSY)
    {
        pUartState->eStatus = FLEXIO_UART_IP_STATUS_SUCCESS;
    }
    pUartState->bDriverIdle = FALSE;
    return pUartState->eStatus;
}

/*FUNCTION**********************************************************************
 *
 * Function Name : Flexio_Uart_Ip_AsyncReceive
 * Description   : Perform a non-blocking UART reception
 * Implements : Flexio_Uart_Ip_AsyncReceive_Activity
 *
 *END**************************************************************************/
/* implements     Flexio_Uart_Ip_AsyncReceive_Activity*/
Flexio_Uart_Ip_StatusType Flexio_Uart_Ip_AsyncReceive(uint32 u32Channel,
                                                      uint8 * pRxBuff,
                                                      uint32 u32RxSize)
{
    FLEXIO_UART_DEV_ASSERT(u32Channel < FEATURE_FLEXIO_MAX_SHIFTER_COUNT);
    FLEXIO_Type *pBase;
    Flexio_Uart_Ip_StateStructureType * pUartState;
    pUartState = (Flexio_Uart_Ip_StateStructureType *)Flexio_Uart_Ip_apStateStructuresArray[u32Channel];
    const Flexio_Uart_Ip_UserConfigType *pUartUserCfg;
    pUartUserCfg = (Flexio_Uart_Ip_UserConfigType*) Flexio_Uart_Ip_apUserConfig[u32Channel];

    FLEXIO_UART_DEV_ASSERT(pUartState != NULL_PTR);
    FLEXIO_UART_DEV_ASSERT(pRxBuff != NULL_PTR);
    FLEXIO_UART_DEV_ASSERT(u32RxSize > 0U);
    FLEXIO_UART_DEV_ASSERT(pUartUserCfg->eDirection == FLEXIO_UART_IP_DIRECTION_RX);
    /* If 2 bytes per word are needed, then the size must be even */
    FLEXIO_UART_DEV_ASSERT((pUartUserCfg->eBitCount <= 8U) || ((u32RxSize & 1U) == 0U));
    /* Check if driver is busy */
    SchM_Enter_Uart_UART_EXCLUSIVE_AREA_06();
    if (pUartState->bDriverIdle)
    {
        SchM_Exit_Uart_UART_EXCLUSIVE_AREA_06();
        return FLEXIO_UART_IP_STATUS_BUSY;
    }
    pUartState->bDriverIdle = TRUE;
    SchM_Exit_Uart_UART_EXCLUSIVE_AREA_06();

    pBase = Flexio_Uart_Ip_apBases[FLEXIO_HW_INSTANCE];
    pUartState->pRxData = pRxBuff;
    pUartState->u32RemainingBytes = u32RxSize;
    pUartState->eStatus = FLEXIO_UART_IP_STATUS_BUSY;

    /* Enable timers and shifters */
    Flexio_Uart_Ip_EnableTransfer(u32Channel);
    /* Enable transfer engine */
    switch (pUartUserCfg->eDriverType)
    {
        case FLEXIO_UART_IP_DRIVER_TYPE_INTERRUPTS:
        /* Enable interrupts for Rx shifter */
        Flexio_Mcl_Ip_SetShifterInterrupt(pBase, (uint8)(1U << RX_SHIFTER(u32Channel)), TRUE);
        Flexio_Mcl_Ip_SetShifterErrorInterrupt(pBase, (uint8)(1U << RX_SHIFTER(u32Channel)), TRUE);
        break;
#if (FLEXIO_UART_IP_HAS_DMA_ENABLED == STD_ON)
        case FLEXIO_UART_IP_DRIVER_TYPE_DMA:
            Flexio_Uart_Ip_StartReceiveDataUsingDma(u32Channel, pRxBuff, u32RxSize);
            break;
#endif
        default:
            /* Impossible type - do nothing */
            break;
    }

    return FLEXIO_UART_IP_STATUS_SUCCESS;
}

/*FUNCTION**********************************************************************
 *
 * Function Name : Flexio_Uart_Ip_SyncReceive
 * Description   : Perform a non-blocking UART reception
 * Implements : Flexio_Uart_Ip_SyncReceive_Activity
 *
 *END**************************************************************************/
/* implements     Flexio_Uart_Ip_SyncReceive_Activity*/
Flexio_Uart_Ip_StatusType Flexio_Uart_Ip_SyncReceive(uint32 u32Channel,
                                                     uint8 * pRxBuff,
                                                     uint32 u32RxSize)
{
    FLEXIO_UART_DEV_ASSERT(u32Channel < FEATURE_FLEXIO_MAX_SHIFTER_COUNT);
    FLEXIO_UART_DEV_ASSERT(pRxBuff != NULL_PTR);
    FLEXIO_UART_DEV_ASSERT(u32RxSize > 0U);

    Flexio_Uart_Ip_StateStructureType * pUartState;
    pUartState = (Flexio_Uart_Ip_StateStructureType *)Flexio_Uart_Ip_apStateStructuresArray[u32Channel];
    const Flexio_Uart_Ip_UserConfigType *pUartUserCfg;
    pUartUserCfg = (Flexio_Uart_Ip_UserConfigType*) Flexio_Uart_Ip_apUserConfig[u32Channel];
    uint32 u32StartTime;
    uint32 u32ElapsedTime;
    uint32 u32TimeoutTicks;
    FLEXIO_Type *pBase;
    pBase = Flexio_Uart_Ip_apBases[FLEXIO_HW_INSTANCE];
    FLEXIO_UART_DEV_ASSERT(pUartState != NULL_PTR);
    FLEXIO_UART_DEV_ASSERT(pUartUserCfg->eDirection == FLEXIO_UART_IP_DIRECTION_RX);
    /* If 2 bytes per word are needed, then the size must be even */
    FLEXIO_UART_DEV_ASSERT((pUartUserCfg->eBitCount <= 8U) || ((u32RxSize & 1U) == 0U));

    /* Check if driver is busy */
    SchM_Enter_Uart_UART_EXCLUSIVE_AREA_07();
    if (pUartState->bDriverIdle)
    {
        SchM_Exit_Uart_UART_EXCLUSIVE_AREA_07();
        return FLEXIO_UART_IP_STATUS_BUSY;
    }
    pUartState->bDriverIdle = TRUE;
    SchM_Exit_Uart_UART_EXCLUSIVE_AREA_07();
    pUartState->pRxData = pRxBuff;
    pUartState->u32RemainingBytes = u32RxSize;
    pUartState->eStatus = FLEXIO_UART_IP_STATUS_BUSY;
    /* Disable interrupts for Rx shifter */
    Flexio_Mcl_Ip_SetShifterInterrupt(pBase, (uint8)(1U << RX_SHIFTER(u32Channel)), FALSE);
    Flexio_Mcl_Ip_SetShifterErrorInterrupt(pBase, (uint8)(1U << RX_SHIFTER(u32Channel)), FALSE);
    /* Enable timers and shifters */
    Flexio_Uart_Ip_EnableTransfer(u32Channel);

    u32StartTime = OsIf_GetCounter(FLEXIO_UART_IP_TIMEOUT_TYPE);

    while ((pUartState->u32RemainingBytes > 0U) && \
            !Flexio_Uart_Ip_CheckTimeout(u32StartTime, (uint32)FLEXIO_UART_IP_TIMEOUT_VALUE_US))
    {
        while (!Flexio_Mcl_Ip_GetShifterStatus(pBase, u32Channel)  && \
               !Flexio_Uart_Ip_CheckTimeout(u32StartTime, (uint32)FLEXIO_UART_IP_TIMEOUT_VALUE_US))
        {}
        /* For SMOD=Receive, indicates Shifter was ready to store new data into SHIFTBUF before the previous data
        was read from SHIFTBUF (SHIFTBUF Overrun), or indicates that the received start or stop bit does not
        match the expected value. */
        if (Flexio_Mcl_Ip_GetShifterErrorStatus(pBase, RX_SHIFTER(u32Channel)))
        {
            pUartState->eStatus = FLEXIO_UART_IP_STATUS_ERROR;
            Flexio_Mcl_Ip_ClearShifterErrorStatus(pBase, RX_SHIFTER(u32Channel));
            break;
        }
        /* Read only data from SHIFBUF when shifter status flag is set */
        Flexio_Uart_Ip_ReadData(u32Channel);
    }

    /* Check other success receiving case*/
    if (FLEXIO_UART_IP_STATUS_BUSY == pUartState->eStatus)
    {
        u32ElapsedTime  = OsIf_GetElapsed(&u32StartTime, FLEXIO_UART_IP_TIMEOUT_TYPE);
        u32TimeoutTicks = OsIf_MicrosToTicks((uint32)FLEXIO_UART_IP_TIMEOUT_VALUE_US, FLEXIO_UART_IP_TIMEOUT_TYPE);
        /* Check if Timeout occur */
        if (u32ElapsedTime >= u32TimeoutTicks)
        {
            pUartState->eStatus = FLEXIO_UART_IP_STATUS_TIMEOUT;
        }
        else
        {
            pUartState->eStatus = FLEXIO_UART_IP_STATUS_SUCCESS;
        }
    }

    pUartState->bDriverIdle = FALSE;
    return pUartState->eStatus;
}
#if (FLEXIO_UART_IP_HAS_DMA_ENABLED == STD_ON)

/*FUNCTION**********************************************************************
 *
 * Function Name : Flexio_Uart_Ip_CompleteSendUsingDma
 * Description   : Finish up a transmit by completing the process of sending
 * data and disabling the DMA requests. This is a callback for DMA major loop
 * completion, so it must match the DMA callback signature.
 *
 *END**************************************************************************/
void Flexio_Uart_Ip_CompleteSendUsingDma(uint32 u32Channel)
{
    FLEXIO_UART_DEV_ASSERT(u32Channel < FEATURE_FLEXIO_MAX_SHIFTER_COUNT);
    Flexio_Uart_Ip_StateStructureType * pUartState;
    const Flexio_Uart_Ip_UserConfigType * pUartUserCfg;
    FLEXIO_Type * pBase;
    Dma_Ip_LogicChannelTransferListType dmaTransferList[FLEXIO_UART_DMA_CONFIG_LIST_DIMENSION];
    Dma_Ip_ReturnType eDmaReturnStatus;
    Dma_Ip_LogicChannelStatusType dmaStatus;

    pUartState = (Flexio_Uart_Ip_StateStructureType *)Flexio_Uart_Ip_apStateStructuresArray[u32Channel];
    pUartUserCfg = (Flexio_Uart_Ip_UserConfigType*) Flexio_Uart_Ip_apUserConfig[u32Channel];
    pBase = Flexio_Uart_Ip_apBases[FLEXIO_HW_INSTANCE];

    /* Get Dma Ip Logic Channel Status */
    (void)Dma_Ip_GetLogicChannelStatus(pUartUserCfg->u8DmaChannel, &dmaStatus);
    if (dmaStatus.ChStateValue == DMA_IP_CH_ERROR_STATE)
    {
        /* Reset the Dma Channel Error status. */
        eDmaReturnStatus = Dma_Ip_SetLogicChannelCommand(pUartUserCfg->u8DmaChannel, DMA_IP_CH_CLEAR_ERROR);

        FLEXIO_UART_DEV_ASSERT(eDmaReturnStatus == DMA_IP_STATUS_SUCCESS);
        /* Update transmit status */
        pUartState->eStatus = FLEXIO_UART_IP_STATUS_ERROR;
        /* Invoke callback if there is one */
        if (pUartUserCfg->pfErrorCallback != NULL_PTR)
        {
            pUartUserCfg->pfErrorCallback(u32Channel, pUartState, FLEXIO_UART_IP_EVENT_ERROR, pUartUserCfg->pErrorCallbackParam);
        }
    }

    /* Invoke the callback when the buffer is finished;
     * Application can provide another buffer inside the callback by calling Flexio_Uart_Ip_SetTxBuffer */
    if (pUartState->eStatus == FLEXIO_UART_IP_STATUS_BUSY)
    {
        /* Invoke callback if there is one (callback may reset the tx buffer for continuous transmission)*/
        if (pUartUserCfg->pfCallback != NULL_PTR)
        {
            pUartUserCfg->pfCallback(u32Channel, pUartState, FLEXIO_UART_IP_EVENT_TX_EMPTY, pUartUserCfg->pCallbackParam);
        }
    }

    /* If the callback has updated the tx buffer, update the DMA descriptor to continue the transfer;
     * otherwise, stop the current transfer.
     */
    if ((pUartState->u32RemainingBytes > 0U) && (pUartState->eStatus != FLEXIO_UART_IP_STATUS_ERROR))
    {
        /* Set up parameters for Dma_Ip_LogicChannelTransferListType */
        dmaTransferList[0].Param = DMA_IP_CH_SET_SOURCE_ADDRESS;
        dmaTransferList[0].Value = (uint32)(pUartState->pTxData);
        /* Update tx size and major loop count parameters for Dma_Ip_LogicChannelTransferListType */
        dmaTransferList[1].Param = DMA_IP_CH_SET_MAJORLOOP_COUNT;
        dmaTransferList[1].Value = pUartState->u32RemainingBytes;

        /* Re-configure the transfer control descriptor for the DMA channel */
        eDmaReturnStatus = Dma_Ip_SetLogicChannelTransferList(pUartUserCfg->u8DmaChannel, dmaTransferList, FLEXIO_UART_DMA_LEAST_CONFIG_LIST_DIMENSION);
        FLEXIO_UART_DEV_ASSERT(eDmaReturnStatus == DMA_IP_STATUS_SUCCESS);

        /* Now that this tx is set up, clear remaining bytes count */
        pUartState->u32RemainingBytes = 0U;

        /* Re-start the channel */
        eDmaReturnStatus = Dma_Ip_SetLogicChannelCommand(pUartUserCfg->u8DmaChannel, DMA_IP_CH_SET_HARDWARE_REQUEST);
        FLEXIO_UART_DEV_ASSERT(eDmaReturnStatus == DMA_IP_STATUS_SUCCESS);
    }
    else
    {
        /* Disable tx DMA requests for the current instance */
        Flexio_Mcl_Ip_SetShifterDMARequest(pBase, (uint8)(1U << TX_TIMER(u32Channel)), FALSE);

        /* For Tx we need to disable timer interrupt */
        Flexio_Mcl_Ip_SetTimerInterrupt(pBase, (uint8)(1U << TX_TIMER(u32Channel)), FALSE);

        /* Update the busy flag */
        pUartState->bDriverIdle = FALSE;

        /* If the current reception hasn't been aborted, update the status */
        if (pUartState->eStatus == FLEXIO_UART_IP_STATUS_BUSY)
        {
            pUartState->eStatus = FLEXIO_UART_IP_STATUS_SUCCESS;
            /* Invoke callback if there is one */
            if (pUartUserCfg->pfCallback != NULL_PTR)
            {
                pUartUserCfg->pfCallback(u32Channel, pUartState, FLEXIO_UART_IP_EVENT_END_TRANSFER, pUartUserCfg->pCallbackParam);
            }
        }
    }
}

/*FUNCTION**********************************************************************
 *
 * Function Name : Flexio_Uart_Ip_CompleteReceiveUsingDma
 * Description   : Finish up a transmit by completing the process of sending
 * data and disabling the DMA requests. This is a callback for DMA major loop
 * completion, so it must match the DMA callback signature.
 *
 *END**************************************************************************/
void Flexio_Uart_Ip_CompleteReceiveUsingDma(uint32 u32Channel)
{
    FLEXIO_UART_DEV_ASSERT(u32Channel < FEATURE_FLEXIO_MAX_SHIFTER_COUNT);
    Flexio_Uart_Ip_StateStructureType * pUartState;
    const Flexio_Uart_Ip_UserConfigType * pUartUserCfg;
    FLEXIO_Type * pBase;
    Dma_Ip_LogicChannelTransferListType dmaTransferList[FLEXIO_UART_DMA_CONFIG_LIST_DIMENSION];
    Dma_Ip_ReturnType eDmaReturnStatus;
    Dma_Ip_LogicChannelStatusType dmaStatus;

    pUartState = (Flexio_Uart_Ip_StateStructureType *)Flexio_Uart_Ip_apStateStructuresArray[u32Channel];
    pUartUserCfg = (Flexio_Uart_Ip_UserConfigType*) Flexio_Uart_Ip_apUserConfig[u32Channel];
    pBase = Flexio_Uart_Ip_apBases[FLEXIO_HW_INSTANCE];

    /* Get Dma Ip Logic Channel Status */
    (void)Dma_Ip_GetLogicChannelStatus(pUartUserCfg->u8DmaChannel, &dmaStatus);
    if (dmaStatus.ChStateValue == DMA_IP_CH_ERROR_STATE)
    {
        /* Reset the Dma Channel Error status. */
        eDmaReturnStatus = Dma_Ip_SetLogicChannelCommand(pUartUserCfg->u8DmaChannel, DMA_IP_CH_CLEAR_ERROR);

        FLEXIO_UART_DEV_ASSERT(eDmaReturnStatus == DMA_IP_STATUS_SUCCESS);
        /* Update transmit status */
        pUartState->eStatus = FLEXIO_UART_IP_STATUS_ERROR;
        /* Invoke callback if there is one */
        if (pUartUserCfg->pfErrorCallback != NULL_PTR)
        {
            pUartUserCfg->pfErrorCallback(u32Channel, pUartState, FLEXIO_UART_IP_EVENT_ERROR, pUartUserCfg->pErrorCallbackParam);
        }
    }

    /* Invoke the callback when the buffer is finished;
     * Application can provide another buffer inside the callback by calling Flexio_Uart_Ip_SetTxBuffer */
    if (pUartState->eStatus == FLEXIO_UART_IP_STATUS_BUSY)
    {
        /* Invoke callback if there is one (callback may reset the tx buffer for continuous transmission)*/
        if (pUartUserCfg->pfCallback != NULL_PTR)
        {
            pUartUserCfg->pfCallback(u32Channel, pUartState, FLEXIO_UART_IP_EVENT_RX_FULL, pUartUserCfg->pCallbackParam);
        }
    }

    /* If the callback has updated the tx buffer, update the DMA descriptor to continue the transfer;
     * otherwise, stop the current transfer.
     */
    if ((pUartState->u32RemainingBytes > 0U) && (pUartState->eStatus != FLEXIO_UART_IP_STATUS_ERROR))
    {
        /* Set up parameters for Dma_Ip_LogicChannelTransferListType */
        dmaTransferList[0].Param = DMA_IP_CH_SET_DESTINATION_ADDRESS;
        dmaTransferList[0].Value = (uint32)(pUartState->pRxData);
        /* Update tx size and major loop count parameters for Dma_Ip_LogicChannelTransferListType */
        dmaTransferList[1].Param = DMA_IP_CH_SET_MAJORLOOP_COUNT;
        dmaTransferList[1].Value = pUartState->u32RemainingBytes;

        /* Re-configure the transfer control descriptor for the DMA channel */
        eDmaReturnStatus = Dma_Ip_SetLogicChannelTransferList(pUartUserCfg->u8DmaChannel, dmaTransferList, FLEXIO_UART_DMA_LEAST_CONFIG_LIST_DIMENSION);
        FLEXIO_UART_DEV_ASSERT(eDmaReturnStatus == DMA_IP_STATUS_SUCCESS);

        /* Now that this rx is set up, clear remaining bytes count */
        pUartState->u32RemainingBytes = 0U;

        /* Re-start the channel */
        eDmaReturnStatus = Dma_Ip_SetLogicChannelCommand(pUartUserCfg->u8DmaChannel, DMA_IP_CH_SET_HARDWARE_REQUEST);
        FLEXIO_UART_DEV_ASSERT(eDmaReturnStatus == DMA_IP_STATUS_SUCCESS);
    }
    else
    {
        /* Disable interrupts for Rx / Tx shifter */
        Flexio_Mcl_Ip_SetShifterInterrupt(pBase, (uint8)(1U << TX_SHIFTER(u32Channel)), FALSE);
        Flexio_Mcl_Ip_SetShifterErrorInterrupt(pBase, (uint8)(1U << TX_SHIFTER(u32Channel)), FALSE);
        /* Disable timer interrupt  */
        Flexio_Mcl_Ip_SetTimerInterrupt(pBase, (uint8)(1U << TX_TIMER(u32Channel)), FALSE);
        /* Disable rx DMA requests for the current instance */
        Flexio_Mcl_Ip_SetShifterDMARequest(pBase, (uint8)(1U << RX_TIMER(u32Channel)), FALSE);

        /* For Rx we need to disable timer interrupt */
        Flexio_Mcl_Ip_SetTimerInterrupt(pBase, (uint8)(1U << RX_TIMER(u32Channel)), FALSE);
        /* Update the busy flag */
        pUartState->bDriverIdle = FALSE;
        /* If the current reception hasn't been aborted, update the status */
        if (pUartState->eStatus == FLEXIO_UART_IP_STATUS_BUSY)
        {
            pUartState->eStatus = FLEXIO_UART_IP_STATUS_SUCCESS;
            /* Call the callback to notify application that the transfer is complete */
            if (pUartUserCfg->pfCallback != NULL_PTR)
            {
                pUartUserCfg->pfCallback(u32Channel, pUartState, FLEXIO_UART_IP_EVENT_END_TRANSFER, pUartUserCfg->pCallbackParam);
            }
        }
    }
}
#endif

/*FUNCTION**********************************************************************
 *
 * Function Name : Flexio_Uart_Ip_AbortTransferData
 * Description   : This function terminates an non-blocking FLEXIO transmission
 * early. During a non-blocking FLEXIO transmission, the user has the option to
 * terminate the transmission early if the transmission is still in progress.
 *
 * Implements    : Flexio_Uart_Ip_AbortTransferData_Activity
 *END**************************************************************************/
Flexio_Uart_Ip_StatusType Flexio_Uart_Ip_AbortTransferData(uint32 u32Channel)
{
    FLEXIO_UART_DEV_ASSERT(u32Channel < FEATURE_FLEXIO_MAX_SHIFTER_COUNT);
    Flexio_Uart_Ip_StateStructureType * pUartState;
    const Flexio_Uart_Ip_UserConfigType *pUartUserCfg;

    pUartState = (Flexio_Uart_Ip_StateStructureType *)Flexio_Uart_Ip_apStateStructuresArray[u32Channel];
    pUartUserCfg = (Flexio_Uart_Ip_UserConfigType*)Flexio_Uart_Ip_apUserConfig[u32Channel];

    FLEXIO_UART_DEV_ASSERT(pUartState != NULL_PTR);
    FLEXIO_UART_DEV_ASSERT(pUartUserCfg != NULL_PTR);

    /* Check if a transfer is running. */
    if (!pUartState->bDriverIdle)
    {
        return FLEXIO_UART_IP_STATUS_SUCCESS;
    }

    /* Update the tx status */
    pUartState->eStatus = FLEXIO_UART_IP_STATUS_ABORTED;

    /* Stop the running transfer. */
    if (pUartUserCfg->eDriverType == FLEXIO_UART_IP_DRIVER_TYPE_INTERRUPTS)
    {
        Flexio_Uart_Ip_StopTransfer(u32Channel);
    }
#if (FLEXIO_UART_IP_HAS_DMA_ENABLED == STD_ON)
    else
    {
        /* Release the DMA channel */
        (void)Dma_Ip_SetLogicChannelCommand(pUartUserCfg->u8DmaChannel, DMA_IP_CH_CLEAR_HARDWARE_REQUEST);
        Flexio_Uart_Ip_CompleteSendUsingDma(u32Channel);
    }
#endif
    return FLEXIO_UART_IP_STATUS_SUCCESS;
}

/*FUNCTION**********************************************************************
 *
 * Function Name : Flexio_Uart_Ip_SetBaudRate
 * Description   : Configures the FLEXIO baud rate.
 * In some FLEXIO instances the user must disable the transmitter/receiver
 * before calling this function.
 * Generally, this may be applied to all FLEXIOs to ensure safe operation.
 *
 * Implements    : Flexio_Uart_Ip_SetBaudRate_Activity
 *END**************************************************************************/
Flexio_Uart_Ip_StatusType Flexio_Uart_Ip_SetBaudRate(uint32 u32Channel,
                                                     Flexio_Uart_Ip_BaudrateType u32DesiredBaudrate,
                                                     uint32 u32ClockFrequency)
{
    FLEXIO_UART_DEV_ASSERT(u32Channel < FEATURE_FLEXIO_MAX_SHIFTER_COUNT);
    FLEXIO_UART_DEV_ASSERT(u32ClockFrequency > 0U);

    Flexio_Uart_Ip_StateStructureType * pUartState;
    const Flexio_Uart_Ip_UserConfigType *pUartUserCfg;
  Flexio_Uart_Ip_TimerDecrementType eTimerDecrement;
  FLEXIO_Type * pBase;
    sint16 i16Divider;

    pUartState = (Flexio_Uart_Ip_StateStructureType *)Flexio_Uart_Ip_apStateStructuresArray[u32Channel];
    pUartUserCfg = (Flexio_Uart_Ip_UserConfigType*) Flexio_Uart_Ip_apUserConfig[u32Channel];
  pBase = Flexio_Uart_Ip_apBases[FLEXIO_HW_INSTANCE];

    if (pUartState != NULL_PTR)
    {
        /* Check if there is an ongoing transfer */
        if (pUartState->bDriverIdle == TRUE)
        {
            return FLEXIO_UART_IP_STATUS_BUSY;
        }
    }

    i16Divider = (uint8)(u32ClockFrequency / (u32DesiredBaudrate * 16U * 2U)) - 1U;

    if ((i16Divider <= 255) && (i16Divider >= 0))
    {
        eTimerDecrement = FLEXIO_TIMER_DECREMENT_FXIO_CLK_DIV_16;
        pUartState->u32BaudRate = (uint32)(u32ClockFrequency / (16U * 2U * (i16Divider + 1U)));
    }
    else
    {
        eTimerDecrement = FLEXIO_TIMER_DECREMENT_FXIO_CLK_DIV_256;
        i16Divider = (uint8)(u32ClockFrequency / (u32DesiredBaudrate * 256 * 2U)) - 1U;
        FLEXIO_UART_DEV_ASSERT((i16Divider <= 255) && (i16Divider >= 0));
        pUartState->u32BaudRate = (uint32)(u32ClockFrequency / (256U * 2U * (i16Divider + 1U)));
    }

    Flexio_Uart_Ip_SetTimerCompare(pBase, TX_TIMER(u32Channel), (uint16)((((uint16)(pUartUserCfg->eBitCount << 1U) - 1U) << 8U) + (uint8)i16Divider));

    if (pUartUserCfg->eDirection == FLEXIO_UART_IP_DIRECTION_TX)
    {
        /* Configure tx timer */
        Flexio_Uart_Ip_SetTimerConfig(pBase,
                                  TX_TIMER(u32Channel),
                                  FLEXIO_TIMER_START_BIT_ENABLED,
                                  FLEXIO_TIMER_STOP_BIT_TIM_DIS,
                                  FLEXIO_TIMER_ENABLE_TRG_HIGH,       /* Enable when Tx data is available */
                                  FLEXIO_TIMER_DISABLE_TIM_CMP,
                                  FLEXIO_TIMER_RESET_NEVER,
                                  eTimerDecrement,                    /* Decrement on FlexIO clock */
                                  FLEXIO_TIMER_INITOUT_ONE);
    }
    else
    {
        /* Configure rx timer */
        Flexio_Uart_Ip_SetTimerConfig(pBase,
                                  RX_TIMER(u32Channel),
                                  FLEXIO_TIMER_START_BIT_ENABLED,
                                  FLEXIO_TIMER_STOP_BIT_TIM_DIS,
                                  FLEXIO_TIMER_ENABLE_PIN_POSEDGE,    /* Enable when data is available */
                                  FLEXIO_TIMER_DISABLE_TIM_CMP,
                                  FLEXIO_TIMER_RESET_PIN_RISING,
                                  eTimerDecrement,                    /* Decrement on FlexIO clock */
                                  FLEXIO_TIMER_INITOUT_ONE_RESET);
    }
    return FLEXIO_UART_IP_STATUS_SUCCESS;
}

/*FUNCTION**********************************************************************
 *
 * Function Name : Flexio_Uart_Ip_SetTxBuffer
 * Description   : Sets the driver internal reference to the tx buffer.
 *                 Can be called from the tx callback to provide a different
 *                 buffer for continuous transmission.
 *
 * Implements    : Flexio_Uart_Ip_SetTxBuffer_Activity
 *END**************************************************************************/
Flexio_Uart_Ip_StatusType Flexio_Uart_Ip_SetTxBuffer(uint32 u32Channel,
                                                     const uint8 * pTxData,
                                                     uint32 u32TxSize)
{
    FLEXIO_UART_DEV_ASSERT(pTxData != NULL_PTR);
    FLEXIO_UART_DEV_ASSERT(u32TxSize > 0U);
    Flexio_Uart_Ip_StateStructureType * pUartState;

    pUartState = (Flexio_Uart_Ip_StateStructureType *)Flexio_Uart_Ip_apStateStructuresArray[u32Channel];

    pUartState->pTxData = pTxData;
    pUartState->u32RemainingBytes = u32TxSize;

    return FLEXIO_UART_IP_STATUS_SUCCESS;
}

/*FUNCTION**********************************************************************
 *
 * Function Name : Flexio_Uart_Ip_SetRxBuffer
 * Description   : Sets the driver internal reference to the rx buffer.
 *                 Can be called from the rx callback to provide a different
 *                 buffer for continuous reception.
 *
 * Implements    : Flexio_Uart_Ip_SetRxBuffer_Activity
 *END**************************************************************************/
Flexio_Uart_Ip_StatusType Flexio_Uart_Ip_SetRxBuffer(uint32 u32Channel,
                                                     uint8 * pRxData,
                                                     uint32 u32RxSize)
{
    FLEXIO_UART_DEV_ASSERT(pRxData != NULL_PTR);
    FLEXIO_UART_DEV_ASSERT(u32RxSize > 0U);
    Flexio_Uart_Ip_StateStructureType * pUartState;

    pUartState = (Flexio_Uart_Ip_StateStructureType *)Flexio_Uart_Ip_apStateStructuresArray[u32Channel];
    pUartState->pRxData = pRxData;
    pUartState->u32RemainingBytes = u32RxSize;

    return FLEXIO_UART_IP_STATUS_SUCCESS;
}
#define UART_STOP_SEC_CODE
/* @violates @ref Uart_c_REF_1 This violation is not fixed since the inclusion of Uart_MemMap.h is as per AUTOSAR requirement*/
#include "Uart_MemMap.h"

#endif /* (FLEXIO_UART_IP_IS_USING == STD_ON) */
#endif /* #ifdef FLEXIO_UART_IP_IS_USING */

#ifdef __cplusplus
}

/** @} */

#endif
