/*==================================================================================================
*   Project              : RTD AUTOSAR 4.4
*   Platform             : CORTEXM
*   Peripheral           : 
*   Dependencies         : none
*
*   Autosar Version      : 4.4.0
*   Autosar Revision     : ASR_REL_4_4_REV_0000
*   Autosar Conf.Variant :
*   SW Version           : 0.9.0
*   Build Version        : S32K3_RTD_0_9_0_P02_D2107_ASR_REL_4_4_REV_0000_20210716
*
*   (c) Copyright 2020 - 2021 NXP Semiconductors
*   All Rights Reserved.
*
*   NXP Confidential. This software is owned or controlled by NXP and may only be
*   used strictly in accordance with the applicable license terms. By expressly
*   accepting such terms or by downloading, installing, activating and/or otherwise
*   using the software, you are agreeing that you have read, and that you agree to
*   comply with and are bound by, such license terms. If you do not agree to be
*   bound by the applicable license terms, then you may not retain, install,
*   activate or otherwise use the software.
==================================================================================================*/
/**
*   @file       MC_ME.c
*   @version    0.9.0
*
*   @brief   POWER driver implementations.
*   @details POWER driver implementations.
*
*   @addtogroup POWER_DRIVER Power Ip Driver
*   @{
*/


#ifdef __cplusplus
extern "C"{
#endif


/**
* @page misra_violations MISRA-C:2012 violations
*
* @section MC_ME_c_REF_1
* Violates MISRA 2012 Advisory Rule 20.1, #include directives should only be preceded by preprocessor
* directives or comments. AUTOSAR imposes the specification of the sections in which certain parts
* of the driver must be placed.
*
* @section MC_ME_c_REF_2
* Violates MISRA 2012 Required Directive 4.10, Precautions shall be taken in order to prevent the contents
* of a header file being included more than once. This comes from the order of includes in the .c file
* and from include dependencies. As a safe approach, any file must include all its dependencies.
* Header files are already protected against double inclusions. The inclusion of Mcu_MemMap.h is as
* per AUTOSAR requirement [SWS_MemMap_00003].
*
*/


/*==================================================================================================
                                         INCLUDE FILES
 1) system and project includes
 2) needed interfaces from external units
 3) internal and external interfaces from this unit
==================================================================================================*/
#include "Power_Ip_Private.h"

#if (defined(POWER_IP_ENABLE_USER_MODE_SUPPORT) && (STD_ON == POWER_IP_ENABLE_USER_MODE_SUPPORT))
  #if (defined(MCAL_MC_ME_REG_PROT_AVAILABLE))
    #if (STD_ON == MCAL_MC_ME_REG_PROT_AVAILABLE)
      #define USER_MODE_REG_PROT_ENABLED      (STD_ON)
      #include "RegLockMacros.h"
    #endif /* (STD_ON == MCAL_MC_ME_REG_PROT_AVAILABLE) */
  #endif
#endif /* (STD_ON == POWER_IP_ENABLE_USER_MODE_SUPPORT) */


/*==================================================================================================
                               SOURCE FILE VERSION INFORMATION
==================================================================================================*/
#define MC_ME_VENDOR_ID_C                      43
#define MC_ME_AR_RELEASE_MAJOR_VERSION_C       4
#define MC_ME_AR_RELEASE_MINOR_VERSION_C       4
#define MC_ME_AR_RELEASE_REVISION_VERSION_C    0
#define MC_ME_SW_MAJOR_VERSION_C               0
#define MC_ME_SW_MINOR_VERSION_C               9
#define MC_ME_SW_PATCH_VERSION_C               0

/*==================================================================================================
*                                     FILE VERSION CHECKS
==================================================================================================*/
/* Check if MC_ME.c file and Power_Ip_Private.h file are of the same vendor */
#if (MC_ME_VENDOR_ID_C != POWER_IP_PRIVATE_VENDOR_ID)
    #error "MC_ME.c and Power_Ip_Private.h have different vendor ids"
#endif

/* Check if MC_ME.c file and Power_Ip_Private.h file are of the same Autosar version */
#if ((MC_ME_AR_RELEASE_MAJOR_VERSION_C != POWER_IP_PRIVATE_AR_RELEASE_MAJOR_VERSION) || \
     (MC_ME_AR_RELEASE_MINOR_VERSION_C != POWER_IP_PRIVATE_AR_RELEASE_MINOR_VERSION) || \
     (MC_ME_AR_RELEASE_REVISION_VERSION_C != POWER_IP_PRIVATE_AR_RELEASE_REVISION_VERSION) \
    )
    #error "AutoSar Version Numbers of MC_ME.c and Power_Ip_Private.h are different"
#endif

/* Check if MC_ME.c file and Power_Ip_Private.h file are of the same Software version */
#if ((MC_ME_SW_MAJOR_VERSION_C != POWER_IP_PRIVATE_SW_MAJOR_VERSION) || \
     (MC_ME_SW_MINOR_VERSION_C != POWER_IP_PRIVATE_SW_MINOR_VERSION) || \
     (MC_ME_SW_PATCH_VERSION_C != POWER_IP_PRIVATE_SW_PATCH_VERSION) \
    )
    #error "Software Version Numbers of MC_ME.c and Power_Ip_Private.h are different"
#endif

#if (defined(POWER_IP_ENABLE_USER_MODE_SUPPORT) && (STD_ON == POWER_IP_ENABLE_USER_MODE_SUPPORT))
  #if (defined(MCAL_MC_ME_REG_PROT_AVAILABLE))
    #if (STD_ON == MCAL_MC_ME_REG_PROT_AVAILABLE)
        #ifndef DISABLE_MCAL_INTERMODULE_ASR_CHECK
            /* Check if MC_ME.c file and RegLockMacros.h file are of the same Autosar version */
            #if ((MC_ME_AR_RELEASE_MAJOR_VERSION_C    != REGLOCKMACROS_AR_RELEASE_MAJOR_VERSION) || \
                (MC_ME_AR_RELEASE_MINOR_VERSION_C    != REGLOCKMACROS_AR_RELEASE_MINOR_VERSION))
                #error "AutoSar Version Numbers of MC_ME.c and RegLockMacros.h are different"
            #endif
        #endif
    #endif /* (STD_ON == MCAL_MC_ME_REG_PROT_AVAILABLE) */
  #endif
#endif /* (STD_ON == POWER_IP_ENABLE_USER_MODE_SUPPORT) */
/*==================================================================================================
                          LOCAL TYPEDEFS (STRUCTURES, UNIONS, ENUMS)
==================================================================================================*/


/*==================================================================================================
                                       LOCAL MACROS
==================================================================================================*/


/*==================================================================================================
                                       LOCAL CONSTANTS
==================================================================================================*/


/*==================================================================================================
                                       LOCAL VARIABLES
==================================================================================================*/
#define MCU_START_SEC_VAR_INIT_UNSPECIFIED
/**
* @violates @ref MC_ME_c_REF_1 #include directives should only be preceded by preprocessor
* directives or comments.
*
* @violates @ref MC_ME_c_REF_2 Precautions shall be taken in order to prevent the contents
* of a header file being included more than once.
*/
#include "Mcu_MemMap.h"

static Power_Ip_MC_ME_Type * pMC_ME = (Power_Ip_MC_ME_Type *)MC_ME_BASE;

#define MCU_STOP_SEC_VAR_INIT_UNSPECIFIED
/**
* @violates @ref MC_ME_c_REF_1 #include directives should only be preceded by preprocessor
* directives or comments.
*
* @violates @ref MC_ME_c_REF_2 Precautions shall be taken in order to prevent the contents
* of a header file being included more than once.
*/
#include "Mcu_MemMap.h"
/*==================================================================================================
                                       GLOBAL CONSTANTS
==================================================================================================*/


/*==================================================================================================
                                       GLOBAL VARIABLES
==================================================================================================*/


/*==================================================================================================
                                   LOCAL FUNCTION PROTOTYPES
==================================================================================================*/
#define MCU_START_SEC_CODE
/**
* @violates @ref MC_ME_c_REF_1 #include directives should only be preceded by preprocessor
* directives or comments.
*
* @violates @ref MC_ME_c_REF_2 Precautions shall be taken in order to prevent the contents
* of a header file being included more than once.
*/
#include "Mcu_MemMap.h"


static inline void Power_Ip_MC_ME_WriteControlKeys(void);
static inline void Power_Ip_MC_ME_TriggerModeUpdate(void);

static inline void Power_Ip_MC_ME_TriggerPartitionUpdate(uint32 u32TriggerMask, uint8 u8PartitionIndex);
static inline void Power_Ip_MC_ME_TriggerCofbUpdate(uint8 u8PartitionIndex);
static inline void Power_Ip_MC_ME_TriggerCoreUpdate(uint8 u8PartitionIndex, uint8 u8CoreIndex);

static void Power_Ip_MC_ME_ConfigurePartition(const Power_Ip_MC_ME_PartitionConfigType * MC_ME_pPartitionConfigPtr);
static void Power_Ip_MC_ME_ConfigureCOFB(const Power_Ip_MC_ME_CofbConfigType * MC_ME_pCofbConfigPtr, uint8 u8PartitionIndex);
static void Power_Ip_MC_ME_ConfigureCore(const Power_Ip_MC_ME_CoreConfigType * MC_ME_pCoreConfigPtr, uint8 u8PartitionIndex);


/*==================================================================================================
                                       LOCAL FUNCTIONS
==================================================================================================*/
/**
* @brief            This function triggers the hardware processes of MC_ME.
* @details          This function writes the key 0x5AF0 and the inverted key
*                   0xA50F to the Control Key Register (MC_ME_CTL_KEY).
*
* @param[in]        void
*
* @return           void
*
*/
static inline void Power_Ip_MC_ME_WriteControlKeys(void)
{
    /* Starting the hardware processes */

    /* Write key to MC_ME_CTL_KEY */
    pMC_ME->CTL_KEY = MC_ME_CTL_KEY_KEY(MC_ME_CTL_KEY_DIRECT_KEY_U32);

    /* Write inverted key to MC_ME_CTL_KEY */
    pMC_ME->CTL_KEY = MC_ME_CTL_KEY_KEY(MC_ME_CTL_KEY_INVERTED_KEY_U32);
}

/**
* @brief            This function triggers a mode update.
* @details          This function triggers a mode update by writing to the
*                   corresponding MC_ME_MODE_UPD register.
*
* @param[in]        void
*
* @return           void
*
*/
static inline void Power_Ip_MC_ME_TriggerModeUpdate(void)
{
    pMC_ME->MODE_UPD = MC_ME_MODE_UPD_MODE_UPD(MC_ME_MODE_UPD_MODE_UPD_MASK);

    Power_Ip_MC_ME_WriteControlKeys();
}

/**
* @brief            This function triggers a partition update.
* @details          This function triggers a partition update based on the value of u32TriggerMask
*                   by writing to the corresponding MC_ME_PRTNx_PUPD register.
*
* @param[in]        u32TriggerMask      Mask containing the process updates to be triggered.
* @param[in]        u8PartitionIndex    Index of the partition to be updated.
*
* @return           void
*
*/
static inline void Power_Ip_MC_ME_TriggerPartitionUpdate(uint32 u32TriggerMask, uint8 u8PartitionIndex)
{
    uint32 tempValue = 0U;
    tempValue = pMC_ME->Power_IP_MC_ME_Prtn_Array[u8PartitionIndex].PRTN_PUPD;
    tempValue &= ~u32TriggerMask;
    tempValue |= u32TriggerMask;
    pMC_ME->Power_IP_MC_ME_Prtn_Array[u8PartitionIndex].PRTN_PUPD = tempValue;

    Power_Ip_MC_ME_WriteControlKeys();
}

/**
* @brief            This function triggers a COFB update.
* @details          This function triggers a COFB update by writing to
*                   the corresponding MC_ME_PRTNx_PUPD register.
*
* @param[in]        u8PartitionIndex    Index of the partition (in which the COFB set resides) to be updated.
*
* @return           void
*
*/
static inline void Power_Ip_MC_ME_TriggerCofbUpdate(uint8 u8PartitionIndex)
{
    uint32 tempValue = 0U;
    tempValue = pMC_ME->Power_IP_MC_ME_Prtn_Array[u8PartitionIndex].PRTN_PUPD;
    tempValue &= (~MC_ME_PRTN0_PUPD_PCUD_MASK);
    tempValue |= (MC_ME_PRTNX_PUPD_PCUD_TRIG_U32 & MC_ME_PRTN0_PUPD_PCUD_MASK);
    pMC_ME->Power_IP_MC_ME_Prtn_Array[u8PartitionIndex].PRTN_PUPD = tempValue;

    Power_Ip_MC_ME_WriteControlKeys();
}

/**
* @brief            This function triggers a core update.
* @details          This function triggers a core update by writing to the corresponding MC_ME_PRTNx_COREx_PUPD register.
*
* @param[in]        u8PartitionIndex    Index of the partition to be updated.
* @param[in]        u8CoreIndex         Index of the core (within the partition) to be updated.
*
* @return           void
*
*/
static inline void Power_Ip_MC_ME_TriggerCoreUpdate(uint8 u8PartitionIndex, uint8 u8CoreIndex)
{
    uint32 tempValue = 0U;
    tempValue = pMC_ME->Power_IP_MC_ME_Prtn_Array[u8PartitionIndex].Power_Ip_MC_ME_Prtn_Core_Array[u8CoreIndex].PRTN_CORE_PUPD;
    tempValue &= ~MC_ME_PRTN0_CORE0_PUPD_CCUPD_MASK;
    tempValue |= (MC_ME_PRTNX_COREX_PUPD_CCUPD_TRIG_U32 & MC_ME_PRTN0_CORE0_PUPD_CCUPD_MASK);
    pMC_ME->Power_IP_MC_ME_Prtn_Array[u8PartitionIndex].Power_Ip_MC_ME_Prtn_Core_Array[u8CoreIndex].PRTN_CORE_PUPD = tempValue;

    Power_Ip_MC_ME_WriteControlKeys();
}

/**
* @brief            This function configures a given partition.
* @details          Configures the processes and waits until the updates have finished.
*
* @param[in]        MC_ME_pPartitionConfigPtr   Pointer to a Partition configuration structure
*                   (member of 'Power_Ip_MC_ME_ModeConfigType' struct).
*
* @return           void
*
*/
static void Power_Ip_MC_ME_ConfigurePartition(const Power_Ip_MC_ME_PartitionConfigType * MC_ME_pPartitionConfigPtr)
{
    uint32 tempValue = 0U;
    uint32 u32PartitionConfig      = MC_ME_pPartitionConfigPtr->u32PartitionPconfRegValue;
    uint32 u32PartitionTriggerMask = MC_ME_pPartitionConfigPtr->u32PartitionTriggerMask;
    uint32 u32PartitionStatus;
    uint32 StartTime;
    uint32 ElapsedTime;
    uint32 TimeoutTicks;
    boolean TimeoutOccurred = FALSE;
    uint8 u8PartitionIndex = MC_ME_pPartitionConfigPtr->u8PartitionIndex;

    if (MC_ME_PRTNX_PUPD_PCUD_TRIG_U32 == (u32PartitionTriggerMask & MC_ME_PRTN0_PUPD_PCUD_MASK))
    {
        /* Write the new configuration for PRTNx_PCONF[PCE]. */
        tempValue = pMC_ME->Power_IP_MC_ME_Prtn_Array[u8PartitionIndex].PRTN_PCONF;
        tempValue &= ~MC_ME_PRTN0_PCONF_PCE_MASK;
        tempValue |= (u32PartitionConfig & MC_ME_PRTN0_PCONF_PCE_MASK);
        pMC_ME->Power_IP_MC_ME_Prtn_Array[u8PartitionIndex].PRTN_PCONF = tempValue;

        /* Trigger the update in hardware. */
        Power_Ip_MC_ME_TriggerPartitionUpdate(MC_ME_PRTN0_PUPD_PCUD_MASK, u8PartitionIndex);

        /* Wait until the update has finished. */
        PowerStartTimeout(&StartTime, &ElapsedTime, &TimeoutTicks, POWER_TIMEOUT_VALUE_US);
        do
        {
            TimeoutOccurred = PowerTimeoutExpired(&StartTime, &ElapsedTime, TimeoutTicks);

            u32PartitionStatus = pMC_ME->Power_IP_MC_ME_Prtn_Array[u8PartitionIndex].PRTN_STAT;
        } while ( ((u32PartitionStatus & MC_ME_PRTN0_STAT_PCS_MASK) != (u32PartitionConfig & MC_ME_PRTN0_PCONF_PCE_MASK)) && (!TimeoutOccurred) );
        /* timeout notification */
        if(TimeoutOccurred)
        {
            ReportPowerErrors(POWER_IP_REPORT_TIMEOUT_ERROR, POWER_IP_ERR_CODE_RESERVED);
        }
    }

#if (defined(FEATURE_POWER_HAS_OUTPUT_SAFE_STATE))
  #if (FEATURE_POWER_HAS_OUTPUT_SAFE_STATE == STD_ON)

    if (!TimeoutOccurred)
    {
        if (MC_ME_PRTNX_PUPD_OSSUD_TRIG_U32 == (u32PartitionTriggerMask & MC_ME_PRTN1_PUPD_OSSUD_MASK))
        {
            /*  Write the new configuration for PRTNx_PCONF[OSSE]. */
            tempValue = pMC_ME->Power_IP_MC_ME_Prtn_Array[u8PartitionIndex].PRTN_PCONF;
            tempValue &= ~MC_ME_PRTN1_PCONF_OSSE_MASK;
            tempValue |= (u32PartitionConfig & MC_ME_PRTN1_PCONF_OSSE_MASK);
            pMC_ME->Power_IP_MC_ME_Prtn_Array[u8PartitionIndex].PRTN_PCONF = tempValue;

            /* Trigger the update in hardware. */
            Power_Ip_MC_ME_TriggerPartitionUpdate(MC_ME_PRTN1_PUPD_OSSUD_MASK, u8PartitionIndex);

            /* Wait until the update has finished. */
            PowerStartTimeout(&StartTime, &ElapsedTime, &TimeoutTicks, POWER_TIMEOUT_VALUE_US);
            do
            {
                TimeoutOccurred = PowerTimeoutExpired(&StartTime, &ElapsedTime, TimeoutTicks);

                u32PartitionStatus = pMC_ME->Power_IP_MC_ME_Prtn_Array[u8PartitionIndex].PRTN_STAT;
            } while ( ((u32PartitionStatus & MC_ME_PRTN1_STAT_OSSS_MASK) != (u32PartitionConfig & MC_ME_PRTN1_PCONF_OSSE_MASK)) && (!TimeoutOccurred) );
            /* timeout notification */
            if(TimeoutOccurred)
            {
                ReportPowerErrors(POWER_IP_REPORT_TIMEOUT_ERROR, POWER_IP_ERR_CODE_RESERVED);
            }
        }
    }

  #endif /* (FEATURE_POWER_HAS_OUTPUT_SAFE_STATE == STD_ON) */
#endif

}

/**
* @brief            This function configures a given COFB set.
* @details          Configures the blocks within the given COFB set and wait until the updates have finished.
*
* @param[in]        MC_ME_pCofbConfigPtr    Pointer to a COFB set configuration structure
*                   (member of 'Power_Ip_MC_ME_PartitionConfigType' struct).
* @param[in]        u8PartitionIndex        Index of the partition in which the COFB set resides.
*
* @return           void
*
*/
static void Power_Ip_MC_ME_ConfigureCOFB(const Power_Ip_MC_ME_CofbConfigType * MC_ME_pCofbConfigPtr, uint8 u8PartitionIndex)
{
    uint32 u32PeripheralsConfig   = MC_ME_pCofbConfigPtr->u32CofbClkenRegValue;
    uint32 u32PeripheralsToUpdate = MC_ME_pCofbConfigPtr->u32CofbBlocksToUpdateMask;
    uint32 u32PeripheralsStatus;
    uint32 StartTime;
    uint32 ElapsedTime;
    uint32 TimeoutTicks;
    boolean TimeoutOccurred = FALSE;
    uint8 u8CofbIndex = MC_ME_pCofbConfigPtr->u8CofbIndex;

    /* Write the new configuration for the COFB set. */
    pMC_ME->Power_IP_MC_ME_Prtn_Array[u8PartitionIndex].PRTN_COFB_CLKEN[u8CofbIndex] = u32PeripheralsConfig;

    /* Trigger the update in hardware. */
    Power_Ip_MC_ME_TriggerCofbUpdate(u8PartitionIndex);

    /* Wait until the update has finished. */
    PowerStartTimeout(&StartTime, &ElapsedTime, &TimeoutTicks, POWER_TIMEOUT_VALUE_US);
    do
    {
        TimeoutOccurred = PowerTimeoutExpired(&StartTime, &ElapsedTime, TimeoutTicks);

        u32PeripheralsStatus = pMC_ME->Power_IP_MC_ME_Prtn_Array[u8PartitionIndex].PRTN_COFB_STAT[u8CofbIndex];
    } while ( (u32PeripheralsConfig != (u32PeripheralsStatus & u32PeripheralsToUpdate)) && (!TimeoutOccurred) );
    /* timeout notification */
    if(TimeoutOccurred)
    {
        ReportPowerErrors(POWER_IP_REPORT_TIMEOUT_ERROR, POWER_IP_ERR_CODE_RESERVED);
    }
}

/**
* @brief            This function configures a given core.
* @details          Configures the core within the given partition and waits until the updates have finished.
*
* @param[in]        MC_ME_pCoreConfigPtr    Pointer to a core configuration structure
*                   (member of 'Power_Ip_MC_ME_PartitionConfigType' struct).
* @param[in]        u8PartitionIndex        Index of the partition in which the core resides.
*
* @return           void
*
*/
static void Power_Ip_MC_ME_ConfigureCore(const Power_Ip_MC_ME_CoreConfigType * MC_ME_pCoreConfigPtr, uint8 u8PartitionIndex)
{
    uint32 u32CoreConfig = MC_ME_pCoreConfigPtr->u32CorePconfRegValue;
    uint32 u32CoreStatus;
    uint32 StartTime;
    uint32 ElapsedTime;
    uint32 TimeoutTicks;
    boolean TimeoutOccurred = FALSE;
    uint8 u8CoreIndex = MC_ME_pCoreConfigPtr->u8CoreIndex;

    if (MC_ME_PRTNX_COREX_PCONF_CCE_DIS_U32 == (MC_ME_pCoreConfigPtr->u32CorePconfRegValue & MC_ME_PRTN0_CORE0_PCONF_CCE_MASK))
    {
        /* Wait until WFI is set */
        PowerStartTimeout(&StartTime, &ElapsedTime, &TimeoutTicks, POWER_TIMEOUT_VALUE_US);
        do
        {
            TimeoutOccurred = PowerTimeoutExpired(&StartTime, &ElapsedTime, TimeoutTicks);

            u32CoreStatus = pMC_ME->Power_IP_MC_ME_Prtn_Array[u8PartitionIndex].Power_Ip_MC_ME_Prtn_Core_Array[u8CoreIndex].PRTN_CORE_STAT;
        } while ( (MC_ME_PRTNX_COREX_STAT_WFI_EXECUTED_U32 != (u32CoreStatus & MC_ME_PRTN0_CORE0_STAT_WFI_MASK)) && (!TimeoutOccurred) );
        /* timeout notification */
        if(TimeoutOccurred)
        {
            ReportPowerErrors(POWER_IP_REPORT_TIMEOUT_ERROR, POWER_IP_ERR_CODE_RESERVED);
        }
    }
#if (defined(MCU_CONFIGURE_CADDRN))
  #if (MCU_CONFIGURE_CADDRN == STD_ON)
    else
    {
        /* Set the boot address for the core */
        pMC_ME->Power_IP_MC_ME_Prtn_Array[u8PartitionIndex].Power_Ip_MC_ME_Prtn_Core_Array[u8CoreIndex].PRTN_CORE_ADDR = (uint32) MC_ME_pCoreConfigPtr->u32CoreBootAddress;
    }
  #endif
#endif

    if (!TimeoutOccurred)
    {
        /* Write the new configuration for the core. */
        pMC_ME->Power_IP_MC_ME_Prtn_Array[u8PartitionIndex].Power_Ip_MC_ME_Prtn_Core_Array[u8CoreIndex].PRTN_CORE_PCONF = u32CoreConfig;

        /* Trigger the update in hardware */
        Power_Ip_MC_ME_TriggerCoreUpdate(u8PartitionIndex, u8CoreIndex);

        /* Wait until the update has finished. */
        PowerStartTimeout(&StartTime, &ElapsedTime, &TimeoutTicks, POWER_TIMEOUT_VALUE_US);
        do
        {
            TimeoutOccurred = PowerTimeoutExpired(&StartTime, &ElapsedTime, TimeoutTicks);

            u32CoreStatus = pMC_ME->Power_IP_MC_ME_Prtn_Array[u8PartitionIndex].Power_Ip_MC_ME_Prtn_Core_Array[u8CoreIndex].PRTN_CORE_STAT;
        } while ( ((u32CoreStatus & MC_ME_PRTN0_CORE0_STAT_CCS_MASK) != (u32CoreConfig & MC_ME_PRTN0_CORE0_PCONF_CCE_MASK)) && (!TimeoutOccurred) );
        /* timeout notification */
        if(TimeoutOccurred)
        {
            ReportPowerErrors(POWER_IP_REPORT_TIMEOUT_ERROR, POWER_IP_ERR_CODE_RESERVED);
        }
    }
}

/*==================================================================================================
                                       GLOBAL FUNCTIONS
==================================================================================================*/

#if (defined(POWER_IP_ENABLE_USER_MODE_SUPPORT) && (STD_ON == POWER_IP_ENABLE_USER_MODE_SUPPORT))
  #if (defined(MCAL_MC_ME_REG_PROT_AVAILABLE))
    #if (STD_ON == MCAL_MC_ME_REG_PROT_AVAILABLE)
/**
* @brief            This function will enable writing in User mode by configuring REG_PROT
*/
void Power_Ip_MC_ME_SetUserAccessAllowed(void)
{
#if (defined(MC_ME_BASE))
    SET_USER_ACCESS_ALLOWED(MC_ME_BASE, MC_ME_PROT_MEM_U32);
#endif
}
    #endif /* (STD_ON == MCAL_MC_ME_REG_PROT_AVAILABLE) */
  #endif
#endif /* (STD_ON == POWER_IP_ENABLE_USER_MODE_SUPPORT) */

/**
* @brief            This function initializes the mode structure.
* @details          This function initializes the mode structure by configuring the MC_ME module.
*                   The target mode is requested by accessing the ME_MCTL register with the required
*                   keys. This mode transition request by software must be a valid request
*                   satisfying a set of pre-defined rules to initiate the process. In the case of
*                   mode transitions occurring because of hardware events such as a reset, a SAFE
*                   mode request, or interrupt requests and wakeup events to exit from low-power
*                   modes, the TARGET_MODE bit field of the ME_MCTL register is automatically
*                   updated with the appropriate target mode.
*
* @param[in]        MC_ME_pModeConfigPtr   Pointer to MC_ME configuration structure
*                   (member of 'ModeConfigType' struct).
*
* @return           void
*
*/
void Power_Ip_MC_ME_ModeConfig(const Power_Ip_MC_ME_ModeConfigType * MC_ME_pModeConfigPtr)
{
    const Power_Ip_MC_ME_PartitionConfigType *pTempPartitionConfig = NULL_PTR;
    uint8 u8PartitionIndex;
    uint8 u8CofbIndex;
    uint8 u8CoreIndex;

    for (u8PartitionIndex = 0U; u8PartitionIndex < POWER_IP_MAX_NUMBER_OF_PARTITIONS; ++u8PartitionIndex)
    {
        pTempPartitionConfig = &(*MC_ME_pModeConfigPtr->apPartitionConfig)[u8PartitionIndex];

        if ( (boolean)TRUE == pTempPartitionConfig->bPartitionUnderMcuControl )
        {
            if ( (boolean)TRUE == pTempPartitionConfig->bPartitionPowerUnderMcuControl )
            {
                /* The partition must be enabled before configuring its COFBs and Cores */
                if (MC_ME_PRTNX_PCONF_PCE_EN_U32 == (pTempPartitionConfig->u32PartitionPconfRegValue & MC_ME_PRTN0_PCONF_PCE_MASK))
                {
                    Power_Ip_MC_ME_ConfigurePartition(pTempPartitionConfig);
                }
            }

            for (u8CofbIndex = 0U; u8CofbIndex < pTempPartitionConfig->u8NumberOfCofbs; ++u8CofbIndex)
            {
                if ( (boolean)TRUE == (*pTempPartitionConfig->apPartitionCofbConfig)[u8CofbIndex].bCofbUnderMcuControl )
                {
                    Power_Ip_MC_ME_ConfigureCOFB(&(*pTempPartitionConfig->apPartitionCofbConfig)[u8CofbIndex], pTempPartitionConfig->u8PartitionIndex);
                }
            }

            for (u8CoreIndex = 0U; u8CoreIndex < pTempPartitionConfig->u8NumberOfCores; ++u8CoreIndex)
            {
                if ( (boolean)TRUE == (*pTempPartitionConfig->apPartitionCoreConfig)[u8CoreIndex].bCoreUnderMcuControl )
                {
                    Power_Ip_MC_ME_ConfigureCore(&(*pTempPartitionConfig->apPartitionCoreConfig)[u8CoreIndex], pTempPartitionConfig->u8PartitionIndex);
                }
            }

            if ( (boolean)TRUE == pTempPartitionConfig->bPartitionPowerUnderMcuControl )
            {
                /* The partition must be disabled after configuring its COFBs and Cores */
                if (MC_ME_PRTNX_PCONF_PCE_DIS_U32 == (pTempPartitionConfig->u32PartitionPconfRegValue & MC_ME_PRTN0_PCONF_PCE_MASK))
                {
                    Power_Ip_MC_ME_ConfigurePartition(pTempPartitionConfig);
                }
            }
        }
    }
}

/**
* @brief            This function triggers a reset event (destructive or functional) for the SoC.
* @details          This function triggers a reset event (destructive or functional) for the SoC.
*
* @return           Status of the previous mode.
*
*/
void Power_Ip_MC_ME_SocTriggerResetEvent(Power_Ip_PowerModeType ePowerMode)
{
    if (POWER_IP_DEST_RESET_MODE == ePowerMode)
    {
        /* Makes a request to trigger a Destructive Reset Event */
        pMC_ME->MODE_CONF=MC_ME_MODE_CONF(MC_ME_MODE_CONF_DEST_RST_MASK);
    }
    else if (POWER_IP_FUNC_RESET_MODE == ePowerMode)
    {
        /* Makes a request to trigger a Functional Reset Event */
        pMC_ME->MODE_CONF=MC_ME_MODE_CONF(MC_ME_MODE_CONF_FUNC_RST_MASK);
    }
    else
    {
        /* Should never happen. Throw an error. */
    }

    /* Trigger the update in hardware */
    Power_Ip_MC_ME_TriggerModeUpdate();
}

#if (defined(MCU_ENTER_LOW_POWER_MODE))
  #if (MCU_ENTER_LOW_POWER_MODE == STD_ON)
/**
* @brief            This function will transition a core to Standby mode.
* @details          This function will transition a core to Standby mode.
*
* @param[in]        void
*
* @return           void
*
*/
void Power_Ip_MC_ME_CoreStandbyEntry(void)
{
    /* Disable all IRQs */
    OsIf_SuspendAllInterrupts();

#ifdef MCAL_PLATFORM_ARM
    /* Finish/Retire outstanding instructions */
    ASM_KEYWORD("dsb");
#endif

    /* Execute WFI */
    EXECUTE_WAIT();

#ifdef MCAL_PLATFORM_ARM
    /* Finish/Retire outstanding instructions */
    ASM_KEYWORD("dsb");
#endif

    /* Enable all IRQs */
    OsIf_ResumeAllInterrupts();
}
  #endif /* (MCU_ENTER_LOW_POWER_MODE == STD_ON) */
#endif

#if (defined(MCU_ENTER_LOW_POWER_MODE))
  #if (MCU_ENTER_LOW_POWER_MODE == STD_ON)
/**
* @brief            This function will transition the SoC to Standby mode.
* @details          This function will transition the SoC to Standby mode.
*
* @param[in]        MC_ME_pModeConfigPtr   Pointer to MC_ME configuration structure
*                   (member of 'Power_Ip_MC_ME_ModeConfigType' struct).
*
* @return           void
*
*/
void Power_Ip_MC_ME_SocStandbyEntry(const Power_Ip_MC_ME_ModeConfigType * MC_ME_pModeConfigPtr)
{
    /* Disable all IRQs */
    OsIf_SuspendAllInterrupts();

    /* Program MC_ME for valid main core id */
    pMC_ME->MAIN_COREID = MC_ME_pModeConfigPtr->u32MainCoreIdRegValue;

#ifdef MCAL_PLATFORM_ARM
    /* Finish/Retire outstanding instructions */
    ASM_KEYWORD("dsb");
#endif

    /* Makes a request to go to Standby mode */
    pMC_ME->MODE_CONF = MC_ME_MODE_CONF(MC_ME_MODE_CONF_STANDBY_MASK);

    /* Trigger the update in hardware */
    Power_Ip_MC_ME_TriggerModeUpdate();

#ifdef MCAL_PLATFORM_ARM
    /* Finish/Retire outstanding instructions */
    ASM_KEYWORD("dsb");
#endif

    /* Execute WFI */
    /** @violates @ref MC_ME_c_REF_11 The controlling expression of an if statement and the controlling expression of an
      * iteration-statement shall have essentially Boolean type. */
    EXECUTE_WAIT();
}
  #endif /* (MCU_ENTER_LOW_POWER_MODE == STD_ON) */
#endif

#if (defined(MCU_ENTER_LOW_POWER_MODE))
  #if (MCU_ENTER_LOW_POWER_MODE == STD_ON)
/**
* @brief            This function returns the previous mode.
* @details          This function returns the previous mode.
*
* @return           Status of the previous mode.
*
*/
Power_Ip_PowerModeType Power_Ip_MC_ME_GetPreviousMode(void)
{
    Power_Ip_PowerModeType ePrevMode;

    if (MC_ME_MODE_STAT_PREV_MODE_STANDBY_U32 == (pMC_ME->MODE_STAT & MC_ME_MODE_STAT_PREV_MODE_MASK))
    {
        ePrevMode = POWER_IP_SOC_STANDBY_MODE;
    }
    else
    {
        ePrevMode = POWER_IP_RESET_MODE;
    }

    return ePrevMode;
}
  #endif /* (MCU_ENTER_LOW_POWER_MODE == STD_ON) */
#endif


#define MCU_STOP_SEC_CODE
/**
* @violates @ref MC_ME_c_REF_1 #include directives should only be preceded by preprocessor
* directives or comments.
*
* @violates @ref MC_ME_c_REF_2 Precautions shall be taken in order to prevent the contents
* of a header file being included more than once.
*/
#include "Mcu_MemMap.h"


#ifdef __cplusplus
}
#endif

/** @} */
