/*==================================================================================================
*   Project              : RTD AUTOSAR 4.4
*   Platform             : CORTEXM
*   Peripheral           : 
*   Dependencies         : none
*
*   Autosar Version      : 4.4.0
*   Autosar Revision     : ASR_REL_4_4_REV_0000
*   Autosar Conf.Variant :
*   SW Version           : 0.9.0
*   Build Version        : S32K3_RTD_0_9_0_P02_D2107_ASR_REL_4_4_REV_0000_20210716
*
*   (c) Copyright 2020 - 2021 NXP Semiconductors
*   All Rights Reserved.
*
*   NXP Confidential. This software is owned or controlled by NXP and may only be
*   used strictly in accordance with the applicable license terms. By expressly
*   accepting such terms or by downloading, installing, activating and/or otherwise
*   using the software, you are agreeing that you have read, and that you agree to
*   comply with and are bound by, such license terms. If you do not agree to be
*   bound by the applicable license terms, then you may not retain, install,
*   activate or otherwise use the software.
==================================================================================================*/
/**
*   @file       PMC.c
*   @version    0.9.0
*
*   @brief
*   @brief   POWER driver implementations.
*   @details POWER driver implementations.
*
*   @addtogroup POWER_DRIVER Power Ip Driver
*   @{
*/


#ifdef __cplusplus
extern "C"{
#endif


/**
* @page misra_violations MISRA-C:2012 violations
*
* @section PMC_c_REF_1
* Violates MISRA 2012 Advisory Rule 20.1, #include directives should only be preceded by preprocessor
* directives or comments. AUTOSAR imposes the specification of the sections in which certain parts
* of the driver must be placed.
*
* @section PMC_c_REF_2
* Violates MISRA 2012 Required Directive 4.10, Precautions shall be taken in order to prevent the contents
* of a header file being included more than once. This comes from the order of includes in the .c file
* and from include dependencies. As a safe approach, any file must include all its dependencies.
* Header files are already protected against double inclusions. The inclusion of Mcu_MemMap.h is as
* per AUTOSAR requirement [SWS_MemMap_00003].
*
*/


/*==================================================================================================
                                         INCLUDE FILES
 1) system and project includes
 2) needed interfaces from external units
 3) internal and external interfaces from this unit
==================================================================================================*/
#include "Power_Ip_Private.h"

#if (defined(POWER_IP_ENABLE_USER_MODE_SUPPORT) && (STD_ON == POWER_IP_ENABLE_USER_MODE_SUPPORT))
  #if (defined(MCAL_PMC_REG_PROT_AVAILABLE))
    #if (STD_ON == MCAL_PMC_REG_PROT_AVAILABLE)
      #define USER_MODE_REG_PROT_ENABLED      (STD_ON)
      #include "RegLockMacros.h"
    #endif /* (STD_ON == MCAL_PMC_REG_PROT_AVAILABLE) */
  #endif
#endif /* (STD_ON == POWER_IP_ENABLE_USER_MODE_SUPPORT) */


/*==================================================================================================
                               SOURCE FILE VERSION INFORMATION
==================================================================================================*/
#define PMC_VENDOR_ID_C                      43
#define PMC_AR_RELEASE_MAJOR_VERSION_C       4
#define PMC_AR_RELEASE_MINOR_VERSION_C       4
#define PMC_AR_RELEASE_REVISION_VERSION_C    0
#define PMC_SW_MAJOR_VERSION_C               0
#define PMC_SW_MINOR_VERSION_C               9
#define PMC_SW_PATCH_VERSION_C               0

/*==================================================================================================
*                                     FILE VERSION CHECKS
==================================================================================================*/
/* Check if PMC.c file and Power_Ip_Private.h file are of the same vendor */
#if (PMC_VENDOR_ID_C != POWER_IP_PRIVATE_VENDOR_ID)
    #error "PMC.c and Power_Ip_Private.h have different vendor ids"
#endif

/* Check if PMC.c file and Power_Ip_Private.h file are of the same Autosar version */
#if ((PMC_AR_RELEASE_MAJOR_VERSION_C != POWER_IP_PRIVATE_AR_RELEASE_MAJOR_VERSION) || \
     (PMC_AR_RELEASE_MINOR_VERSION_C != POWER_IP_PRIVATE_AR_RELEASE_MINOR_VERSION) || \
     (PMC_AR_RELEASE_REVISION_VERSION_C != POWER_IP_PRIVATE_AR_RELEASE_REVISION_VERSION) \
    )
    #error "AutoSar Version Numbers of PMC.c and Power_Ip_Private.h are different"
#endif

/* Check if PMC.c file and Power_Ip_Private.h file are of the same Software version */
#if ((PMC_SW_MAJOR_VERSION_C != POWER_IP_PRIVATE_SW_MAJOR_VERSION) || \
     (PMC_SW_MINOR_VERSION_C != POWER_IP_PRIVATE_SW_MINOR_VERSION) || \
     (PMC_SW_PATCH_VERSION_C != POWER_IP_PRIVATE_SW_PATCH_VERSION) \
    )
    #error "Software Version Numbers of PMC.c and Power_Ip_Private.h are different"
#endif

#if (defined(POWER_IP_ENABLE_USER_MODE_SUPPORT) && (STD_ON == POWER_IP_ENABLE_USER_MODE_SUPPORT))
  #if (defined(MCAL_PMC_REG_PROT_AVAILABLE))
    #if (STD_ON == MCAL_PMC_REG_PROT_AVAILABLE)
        #ifndef DISABLE_MCAL_INTERMODULE_ASR_CHECK
        /* Check if PMC.c file and RegLockMacros.h file are of the same Autosar version */
            #if ((PMC_AR_RELEASE_MAJOR_VERSION_C    != REGLOCKMACROS_AR_RELEASE_MAJOR_VERSION) || \
                (PMC_AR_RELEASE_MINOR_VERSION_C    != REGLOCKMACROS_AR_RELEASE_MINOR_VERSION))
                #error "AutoSar Version Numbers of PMC.c and RegLockMacros.h are different"
            #endif
        #endif
    #endif /* (STD_ON == MCAL_PMC_REG_PROT_AVAILABLE) */
  #endif
#endif /* (STD_ON == POWER_IP_ENABLE_USER_MODE_SUPPORT) */
/*==================================================================================================
                          LOCAL TYPEDEFS (STRUCTURES, UNIONS, ENUMS)
==================================================================================================*/


/*==================================================================================================
                                       LOCAL MACROS
==================================================================================================*/


/*==================================================================================================
                                       LOCAL CONSTANTS
==================================================================================================*/


/*==================================================================================================
                                       LOCAL VARIABLES
==================================================================================================*/


/*==================================================================================================
                                       GLOBAL CONSTANTS
==================================================================================================*/


/*==================================================================================================
                                       GLOBAL VARIABLES
==================================================================================================*/
#if (defined(MCU_VOLTAGE_ERROR_ISR_USED))
  #if (MCU_VOLTAGE_ERROR_ISR_USED == STD_ON)

#define MCU_START_SEC_VAR_INIT_UNSPECIFIED
#include "Mcu_MemMap.h"

static volatile PMC_StatusType PMC_Status = PMC_UNINIT;

#define MCU_STOP_SEC_VAR_INIT_UNSPECIFIED
#include "Mcu_MemMap.h"

  #endif
#endif
/*==================================================================================================
                                   LOCAL FUNCTION PROTOTYPES
==================================================================================================*/


/*==================================================================================================
                                       LOCAL FUNCTIONS
==================================================================================================*/
#define MCU_START_SEC_CODE
/**
* @violates @ref PMC_c_REF_2 Precautions shall be taken in order to prevent the contents
* of a header file being included more than once.
*/
#include "Mcu_MemMap.h"


/*==================================================================================================
                                       GLOBAL FUNCTIONS
==================================================================================================*/
#if (defined(POWER_IP_ENABLE_USER_MODE_SUPPORT) && (STD_ON == POWER_IP_ENABLE_USER_MODE_SUPPORT))
  #if (defined(MCAL_PMC_REG_PROT_AVAILABLE))
    #if (STD_ON == MCAL_PMC_REG_PROT_AVAILABLE)
/**
* @brief            This function will enable writing in User mode by configuring REG_PROT
*/
void PMC_SetUserAccessAllowed(void)
{
#if (defined(PMC_BASE))
    SET_USER_ACCESS_ALLOWED(PMC_BASE, PMC_PROT_MEM_U32);
#endif
}
    #endif
  #endif /* MCAL_PMC_REG_PROT_AVAILABLE */
#endif /* POWER_IP_ENABLE_USER_MODE_SUPPORT */


/**
* @brief            This function configure the Power Management Controller
* @details          The operating voltages are monitored by a set of on-chip supervisory circuits
*                   to ensure that this device works within the correct voltage range.
*
* @param[in]        pConfigPtr   Pointer to PMC configuration structure.
*
* @return           void
*
*/
void PMC_PowerInit(const PMC_ConfigType * pConfigPtr)
{
    uint32 StartTime;
    uint32 ElapsedTime;
    uint32 TimeoutTicks;
    boolean TimeoutOccurred = FALSE;
    uint32 u32LvscValue;
    uint32 u32LastMileRegValue;
    uint32 u32ConfigValue;

    /* After the initial power ramp-up of the MCU, in PMC_LVSC register the POR flag
     * and the LVR flags are all set to 1. The GNG flags will have an arbitrary value.
     * Therefore, after the initial power ramp-up, all flags in PMC_LVSC must be
     * cleared by writing 0xFFFF_FFFF */

    if (PMC_LVSC_RAMP_UP_RESET_FLAGS_RWBITS_MASK == (PMC->LVSC & PMC_LVSC_RAMP_UP_RESET_FLAGS_RWBITS_MASK))
    {
        PMC->LVSC = PMC_LVSC_RWBITS_MASK;
    }

    /* Initialize the rest of the PMC module */
    if (PMC_CONFIG_LAST_MILE_REG_ENABLE == (pConfigPtr->u32ConfigRegister & PMC_CONFIG_LMEN_MASK))
    {
        PMC->CONFIG = (uint32)(pConfigPtr->u32ConfigRegister & (~(uint32)PMC_CONFIG_LMEN_MASK));

        PowerStartTimeout(&StartTime, &ElapsedTime, &TimeoutTicks, POWER_TIMEOUT_VALUE_US);
        do
        {
            TimeoutOccurred = PowerTimeoutExpired(&StartTime, &ElapsedTime, TimeoutTicks);

            u32LvscValue = PMC->LVSC;
        } while ( (PMC_LVSC_V15_ABOVE_LV != (u32LvscValue & PMC_LVSC_LVD15S_MASK)) && (!TimeoutOccurred) );

        if (!TimeoutOccurred)
        {
            u32ConfigValue = PMC->CONFIG;
            u32ConfigValue = (uint32)(u32ConfigValue & (~(uint32)PMC_CONFIG_LMEN_MASK));
            u32ConfigValue = (uint32)(u32ConfigValue | (uint32)(PMC_CONFIG_LAST_MILE_REG_ENABLE & PMC_CONFIG_LMEN_MASK));
            PMC->CONFIG = u32ConfigValue;
        }
        else
        {
            ReportPowerErrors(POWER_IP_REPORT_TIMEOUT_ERROR, POWER_IP_ERR_CODE_RESERVED);
        }
    }
    else
    {
        PMC->CONFIG = pConfigPtr->u32ConfigRegister;
        
        if (PMC_CONFIG_LAST_MILE_REG_AUTO_ENABLE == (pConfigPtr->u32ConfigRegister & PMC_CONFIG_LMAUTOEN_MASK))
        {
            PowerStartTimeout(&StartTime, &ElapsedTime, &TimeoutTicks, POWER_TIMEOUT_VALUE_US);
            do
            {
                TimeoutOccurred = PowerTimeoutExpired(&StartTime, &ElapsedTime, TimeoutTicks);
    
                u32LastMileRegValue = (PMC->CONFIG & PMC_CONFIG_LMSTAT_MASK);
            } while ((PMC_CONFIG_LAST_MILE_REG_ON != u32LastMileRegValue) && (!TimeoutOccurred));

            if(TimeoutOccurred)
            {
                ReportPowerErrors(POWER_IP_REPORT_TIMEOUT_ERROR, POWER_IP_ERR_CODE_RESERVED);
            }
            else
            {
                /* do nothing */
            }
        }
    }
#if (defined(MCU_VOLTAGE_ERROR_ISR_USED))
  #if (MCU_VOLTAGE_ERROR_ISR_USED == STD_ON)
    /* make Status of PMC to initialized to check in the interrupt function */
    PMC_Status = PMC_INIT;
  #endif
#endif
}


#if (defined(MCU_ENTER_LOW_POWER_MODE) && (MCU_ENTER_LOW_POWER_MODE == STD_ON))
/**
* @brief            This function prepares the PMC module for Standby/Low Power entry.
* @details          If the Lastmile Regulator is enabled, clear both PMC_LVSC[LMEN]
*                   and PMC_LSVC[LMBCTLEN] simultaneously and notify the application
*                   of the regulator disablement. The LVD/HVD Interrupts are also
*                   disabled.
*
* @param[in]        void
*
* @return           void
*
*/
void PMC_PrepareLowPowerEntry(void)
{
    uint32 u32TempValue = 0U;

    /* Disable LVD/HVD Interrupts */
    u32TempValue = PMC->CONFIG;
    u32TempValue = (uint32)(u32TempValue & (uint32)( ~(PMC_CONFIG_LVDIE_MASK | PMC_CONFIG_HVDIE_MASK) ) );
    u32TempValue = (uint32)(u32TempValue | (uint32)(PMC_CONFIG_LVD_INTERRUPTS_DISABLE | PMC_CONFIG_HVD_INTERRUPTS_DISABLE));
    PMC->CONFIG = u32TempValue;

    /* Disable LM Regulator, if enabled, and notify the application */
    if ((PMC_CONFIG_LAST_MILE_REG_ENABLE == ((uint32)(PMC->CONFIG & PMC_CONFIG_LMEN_MASK))) || \
        (PMC_CONFIG_LAST_MILE_REG_AUTO_ENABLE == ((uint32)(PMC->CONFIG & PMC_CONFIG_LMAUTOEN_MASK))))
    {
        u32TempValue = PMC->CONFIG;
        u32TempValue = (uint32)(u32TempValue & (uint32)( ~(PMC_CONFIG_LMBCTLEN_MASK | PMC_CONFIG_LMEN_MASK | PMC_CONFIG_LMAUTOEN_MASK) ) );
        u32TempValue = (uint32)(u32TempValue | (uint32)(PMC_CONFIG_LM_BASE_CONTROL_DISABLE | PMC_CONFIG_LAST_MILE_REG_DISABLE | PMC_CONFIG_LAST_MILE_REG_AUTO_DISABLE));
        PMC->CONFIG = u32TempValue;

        /* Notify the application of the regulator disablement to further signal the SBC, if needed */
        ReportPowerErrors(POWER_IP_PMC_NOTIFICATION, POWER_LAST_MILE_REGULATOR_DISABLED);
    }
}
#endif /* (MCU_ENTER_LOW_POWER_MODE == STD_ON) */

#if (defined(MCU_VOLTAGE_ERROR_ISR_USED))
  #if (MCU_VOLTAGE_ERROR_ISR_USED == STD_ON)
/**
* @brief            This function handles the voltage error detection.
*
* @return           void
*
* @isr
* @implements PMC_VoltageErrorIsr_Activity
*/
void PMC_VoltageErrorIsr(void)
{
    uint32 u32RegValue;
    uint32 u32VoltageIsrEnabled;
    uint32 u32VoltageIsrStatus;
    u32RegValue = PMC->LVSC;
    
    u32VoltageIsrEnabled = u32RegValue & PMC_LVSC_OV_UV_IRQ_FLAGS_MASK32;

    PMC->LVSC = u32VoltageIsrEnabled;

    if (PMC_UNINIT != PMC_Status)
    {
        /** @violates @ref Mcu_PMC_c_REF_4 Required Rule 11.1, Conversion from integer to pointer */
        /** @violates @ref Mcu_PMC_c_REF_6 The cast is used to access memory mapped registers.*/
        u32VoltageIsrStatus = u32RegValue & PMC_LVSC_OV_UV_STATUS_FLAGS_MASK32;

        u32RegValue = PMC->CONFIG;

        if (PMC_CONFIG_LVD_INTERRUPTS_DISABLE == (u32RegValue & PMC_CONFIG_LVDIE_MASK))
        {
            u32VoltageIsrEnabled = (u32VoltageIsrEnabled & (uint32)(~PMC_LVSC_UV_IRQ_FLAGS_MASK32));
        }

        if (PMC_CONFIG_HVD_INTERRUPTS_DISABLE == (u32RegValue & PMC_CONFIG_HVDIE_MASK))
        {
            u32VoltageIsrEnabled = (u32VoltageIsrEnabled & (uint32)(~PMC_LVSC_OV_IRQ_FLAGS_MASK32));
        }

        /* Align IRQ Flags with Status flags */
        u32VoltageIsrEnabled = (u32VoltageIsrEnabled << 8U);

        /* Check if there is any event to report */
        if ((uint32)0U != (u32VoltageIsrStatus & u32VoltageIsrEnabled))
        {
            ReportPowerErrors(POWER_IP_ERROR_ISR_NOTIFICATION, POWER_E_ISR_VOLTAGE_ERROR);
        }
    }
}
  #endif
#endif /* (MCU_VOLTAGE_ERROR_ISR_USED == STD_ON) */

#define MCU_STOP_SEC_CODE
/**
* @violates @ref PMC_c_REF_1 #include directives should only be preceded by preprocessor
* directives or comments.
*
* @violates @ref PMC_c_REF_2 Precautions shall be taken in order to prevent the contents
* of a header file being included more than once.
*/
#include "Mcu_MemMap.h"


#ifdef __cplusplus
}
#endif

/** @} */
