/*==================================================================================================
*   Project              : RTD AUTOSAR 4.4
*   Platform             : CORTEXM
*   Peripheral           : 
*   Dependencies         : none
*
*   Autosar Version      : 4.4.0
*   Autosar Revision     : ASR_REL_4_4_REV_0000
*   Autosar Conf.Variant :
*   SW Version           : 0.9.0
*   Build Version        : S32K3_RTD_0_9_0_P02_D2107_ASR_REL_4_4_REV_0000_20210716
*
*   (c) Copyright 2020 - 2021 NXP Semiconductors
*   All Rights Reserved.
*
*   NXP Confidential. This software is owned or controlled by NXP and may only be
*   used strictly in accordance with the applicable license terms. By expressly
*   accepting such terms or by downloading, installing, activating and/or otherwise
*   using the software, you are agreeing that you have read, and that you agree to
*   comply with and are bound by, such license terms. If you do not agree to be
*   bound by the applicable license terms, then you may not retain, install,
*   activate or otherwise use the software.
==================================================================================================*/
/**
*   @file       Power_Ip.c
*   @version    0.9.0
*
*   @brief      
*   @brief   POWER driver implementations.
*   @details POWER driver implementations.
*
*   @addtogroup POWER_DRIVER Power Ip Driver
*   @{
*/


#ifdef __cplusplus
extern "C"{
#endif


/**
* @page misra_violations MISRA-C:2012 violations
*
* @section Power_Ip_c_REF_1
* Violates MISRA 2012 Advisory Rule 20.1, #include directives should only be preceded by preprocessor
* directives or comments. AUTOSAR imposes the specification of the sections in which certain parts
* of the driver must be placed.
*
* @section Power_Ip_c_REF_2
* Violates MISRA 2012 Required Directive 4.10, Precautions shall be taken in order to prevent the contents
* of a header file being included more than once. This comes from the order of includes in the .c file
* and from include dependencies. As a safe approach, any file must include all its dependencies.
* Header files are already protected against double inclusions. The inclusion of Mcu_MemMap.h is as
* per AUTOSAR requirement [SWS_MemMap_00003].
*
*/

/*==================================================================================================
*                                        INCLUDE FILES
* 1) system and project includes
* 2) needed interfaces from external units
* 3) internal and external interfaces from this unit
==================================================================================================*/
#include "Power_Ip_Private.h"
#include "OsIf.h"


/*==================================================================================================
                               SOURCE FILE VERSION INFORMATION
==================================================================================================*/
#define POWER_IP_VENDOR_ID_C                      43
#define POWER_IP_AR_RELEASE_MAJOR_VERSION_C       4
#define POWER_IP_AR_RELEASE_MINOR_VERSION_C       4
#define POWER_IP_AR_RELEASE_REVISION_VERSION_C    0
#define POWER_IP_SW_MAJOR_VERSION_C               0
#define POWER_IP_SW_MINOR_VERSION_C               9
#define POWER_IP_SW_PATCH_VERSION_C               0

/*==================================================================================================
*                                     FILE VERSION CHECKS
==================================================================================================*/
/* Check if Power_Ip.c file and Power_Ip_Private.h file are of the same vendor */
#if (POWER_IP_VENDOR_ID_C != POWER_IP_PRIVATE_VENDOR_ID)
    #error "Power_Ip.c and Power_Ip_Private.h have different vendor ids"
#endif

/* Check if Power_Ip.c file and Power_Ip_Private.h file are of the same Autosar version */
#if ((POWER_IP_AR_RELEASE_MAJOR_VERSION_C != POWER_IP_PRIVATE_AR_RELEASE_MAJOR_VERSION) || \
     (POWER_IP_AR_RELEASE_MINOR_VERSION_C != POWER_IP_PRIVATE_AR_RELEASE_MINOR_VERSION) || \
     (POWER_IP_AR_RELEASE_REVISION_VERSION_C != POWER_IP_PRIVATE_AR_RELEASE_REVISION_VERSION) \
    )
    #error "AutoSar Version Numbers of Power_Ip.c and Power_Ip_Private.h are different"
#endif

/* Check if Power_Ip.c file and Power_Ip_Private.h file are of the same Software version */
#if ((POWER_IP_SW_MAJOR_VERSION_C != POWER_IP_PRIVATE_SW_MAJOR_VERSION) || \
     (POWER_IP_SW_MINOR_VERSION_C != POWER_IP_PRIVATE_SW_MINOR_VERSION) || \
     (POWER_IP_SW_PATCH_VERSION_C != POWER_IP_PRIVATE_SW_PATCH_VERSION) \
    )
    #error "Software Version Numbers of Power_Ip.c and Power_Ip_Private.h are different"
#endif

#ifndef DISABLE_MCAL_INTERMODULE_ASR_CHECK
/* Check if Power_Ip.c file and OsIf.h file are of the same Autosar version */
#if ((POWER_IP_AR_RELEASE_MAJOR_VERSION_C    != OSIF_AR_RELEASE_MAJOR_VERSION) || \
     (POWER_IP_AR_RELEASE_MINOR_VERSION_C    != OSIF_AR_RELEASE_MINOR_VERSION))
    #error "AutoSar Version Numbers of Power_Ip.c and OsIf.h are different"
#endif
#endif
/*==================================================================================================
                          LOCAL TYPEDEFS (STRUCTURES, UNIONS, ENUMS)
==================================================================================================*/


/*==================================================================================================
                                        LOCAL MACROS
==================================================================================================*/


/*==================================================================================================
                                       LOCAL CONSTANTS
==================================================================================================*/


/*==================================================================================================
                                       LOCAL VARIABLES
==================================================================================================*/
#define MCU_START_SEC_VAR_INIT_UNSPECIFIED
/**
* @violates @ref Power_Ip_c_REF_1 #include directives should only be preceded by preprocessor
* directives or comments.
*
* @violates @ref Power_Ip_c_REF_2 Precautions shall be taken in order to prevent the contents
* of a header file being included more than once.
*/
#include "Mcu_MemMap.h"

/* Power Report Error Callback */
static Power_Ip_ReportErrorsCallbackType powerReportErrorsCallback = ReportPowerErrorsEmptyCallback;

#define MCU_STOP_SEC_VAR_INIT_UNSPECIFIED
/**
* @violates @ref Power_Ip_c_REF_1 #include directives should only be preceded by preprocessor
* directives or comments.
*
* @violates @ref Power_Ip_c_REF_2 Precautions shall be taken in order to prevent the contents
* of a header file being included more than once.
*/
#include "Mcu_MemMap.h"

/*==================================================================================================
                                       GLOBAL CONSTANTS
==================================================================================================*/


/*==================================================================================================
                                       GLOBAL VARIABLES
==================================================================================================*/


/*==================================================================================================
                                   LOCAL FUNCTION PROTOTYPES
==================================================================================================*/


/*==================================================================================================
                                       LOCAL FUNCTIONS
==================================================================================================*/


/*==================================================================================================
                                       GLOBAL FUNCTIONS
==================================================================================================*/
#define MCU_START_SEC_CODE
/**
* @violates @ref Power_Ip_c_REF_1 #include directives should only be preceded by preprocessor
* directives or comments.
*
* @violates @ref Power_Ip_c_REF_2 Precautions shall be taken in order to prevent the contents
* of a header file being included more than once.
*/
#include "Mcu_MemMap.h"



#if (defined(POWER_IP_ENABLE_USER_MODE_SUPPORT) && (STD_ON == POWER_IP_ENABLE_USER_MODE_SUPPORT))
void Power_Ip_SetUserAccessAllowed(void)
{
#if (defined(MCAL_MC_ME_REG_PROT_AVAILABLE))
  #if (STD_ON == MCAL_MC_ME_REG_PROT_AVAILABLE)
    OsIf_Trusted_Call(Power_Ip_MC_ME_SetUserAccessAllowed);
  #endif
#endif
}
#endif /* (STD_ON == POWER_IP_ENABLE_USER_MODE_SUPPORT) */


/**
* @brief            This function initializes the mode structure.
* @details          This function initializes the mode structure by configuring the MC_ME module.
*
* @param[in]        Power_Ip_pModeConfigPtr   Pointer to mode configuration structure.
*
* @return           void
*
* @implements Power_Ip_SetMode_Activity
*
*/
void Power_Ip_SetMode(const Power_Ip_ModeConfigType * Power_Ip_pModeConfigPtr)
{
    Power_Ip_PowerModeType ePowerMode;
#if (defined(POWER_IP_WARM_RESET_SUPPORT) && (POWER_IP_WARM_RESET_SUPPORT == STD_ON))
    Power_Ip_MSCM_CpxType u32McuCpxType;
#endif /* (POWER_IP_WARM_RESET_SUPPORT == STD_ON) */

    POWER_DEV_ASSERT(NULL_PTR != Power_Ip_pModeConfigPtr);
    
    ePowerMode = Power_Ip_pModeConfigPtr->Power_Ip_ePowerMode;

#if (defined(MCU_RESET_DOMAINS_SUPPORTED) && (MCU_RESET_DOMAINS_SUPPORTED == STD_ON))
    /* Request new mode configuration from MC_RGM. */
    Call_Power_Ip_MC_RGM_ModeConfig(Power_Ip_pModeConfigPtr->MC_RGM_pModeConfig);
    /* Check new mode configuration from MC_RGM. */
    Call_Power_Ip_MC_RGM_CheckModeConfig(Power_Ip_pModeConfigPtr->MC_RGM_pModeConfig);
#endif /* (MCU_RESET_DOMAINS_SUPPORTED == STD_ON) */
    /* Request new mode configuration from MC_ME. */
    Power_Ip_MC_ME_ModeConfig(Power_Ip_pModeConfigPtr->MC_ME_pModeConfig);

    if ( (POWER_IP_DEST_RESET_MODE == ePowerMode) || (POWER_IP_FUNC_RESET_MODE == ePowerMode) )
    {
        Power_Ip_MC_ME_SocTriggerResetEvent(ePowerMode);
    }
#if (defined(MCU_ENTER_LOW_POWER_MODE) && (MCU_ENTER_LOW_POWER_MODE == STD_ON))
    else if (POWER_IP_CORE_STANDBY_MODE == ePowerMode)
    {
        /* External assumption: The application will make sure that the core is prepared for standby entry. */
        Power_Ip_MC_ME_CoreStandbyEntry();
    }
#ifdef POWER_IP_SOC_PREPARE_STANDBY_MODE_SUPPORT
  #if (POWER_IP_SOC_PREPARE_STANDBY_MODE_SUPPORT == STD_ON)
    else if (POWER_IP_SOC_PREPARE_STANDBY_MODE == ePowerMode)
    {
        /* External assumption: The application will disable the PLL (and optionally disable FIRC/SIRC/SXOSC/FXOSC
         * as per power-consumption requirements in order to reduce power consumption in STANDBY mode) and ensure
         * that no flash high-voltage operations are ongoing. */

        if ((boolean)FALSE == Power_Ip_FLASH_HighVoltageOperationOngoing())
        {
            Call_PMC_PrepareLowPowerEntry();
        }
        else
        {
            ReportPowerErrors(POWER_IP_ERROR_ISR_NOTIFICATION, POWER_E_FLASH_HV_OPERATION_ONGOING);
        }
    }
  #endif
#endif
    else if (POWER_IP_SOC_STANDBY_MODE == ePowerMode)
    {
        /* External assumption: The application will program the necessary wakeup IP(s). */
        Power_Ip_MC_ME_SocStandbyEntry(Power_Ip_pModeConfigPtr->MC_ME_pModeConfig);
    }
    else if (POWER_IP_STANDBY_MODE == ePowerMode)
    {
#ifdef POWER_IP_SOC_PREPARE_STANDBY_MODE_SUPPORT
  #if (POWER_IP_SOC_PREPARE_STANDBY_MODE_SUPPORT == STD_ON)
        /* Step 1: prepare soc standby */
        if ((boolean)FALSE == Power_Ip_FLASH_HighVoltageOperationOngoing())
        {
            Call_PMC_PrepareLowPowerEntry();
        }
        else
        {
            ReportPowerErrors(POWER_IP_ERROR_ISR_NOTIFICATION, POWER_E_FLASH_HV_OPERATION_ONGOING);
        }
  #endif
#endif
        /* External assumption: The application will program the necessary wakeup IP(s). */
        /* Step 2: set soc standby */
        Power_Ip_MC_ME_SocStandbyEntry(Power_Ip_pModeConfigPtr->MC_ME_pModeConfig);
    }
#endif /* (MCU_ENTER_LOW_POWER_MODE == STD_ON) */

#if (defined(POWER_IP_WARM_RESET_SUPPORT) && (POWER_IP_WARM_RESET_SUPPORT == STD_ON))
    else if (POWER_IP_CORE_WARM_RESET_MODE == ePowerMode)
    {
        u32McuCpxType = Mcu_MSCM_GetPersonality();

        switch (u32McuCpxType)
        {
#if (defined(MCAL_PLATFORM_ARM))
  #if (MCAL_PLATFORM_ARM == MCAL_ARM_AARCH64)
            case POWER_IP_CORE_A53:
            {
                /**  @violates @ref Mcu_IPW_c_REF_12 The comma operator shall not be used. */
                Call_Mcu_CortexA64_WarmReset();
                break;
            }
  #endif
#endif
            case POWER_IP_CORE_CM7:
            {
                /**  @violates @ref Mcu_IPW_c_REF_12 The comma operator shall not be used. */
                Call_Mcu_CortexM_WarmReset();
                break;
            }

            default:
            {
                /* Should never reach this branch. */
                break;
            }
        }
    }
#endif /* (POWER_IP_WARM_RESET_SUPPORT == STD_ON) */
    else
    {
        /* Nothing else to be done. Ignore the power mode. */
    }
}

#if (defined(MCU_ENTER_LOW_POWER_MODE) && (MCU_ENTER_LOW_POWER_MODE == STD_ON))
/**
* @brief            This function returns the previous mode.
* @details          This function returns the previous mode.
*
* @return           Status of the previous mode.
*
* @implements Power_Ip_GetPreviousMode_Activity
*
*/
Power_Ip_PowerModeType Power_Ip_GetPreviousMode(void)
{
    Power_Ip_PowerModeType Power_PrevMode;

    Power_PrevMode = Power_Ip_MC_ME_GetPreviousMode();

    return Power_PrevMode;
}
#endif /* (MCU_ENTER_LOW_POWER_MODE == STD_ON) */

#if (MCU_PERFORM_RESET_API == STD_ON)
/**
* @brief            This function performs a microcontroller reset.
* @details          This function performs a microcontroller reset by using the hardware feature of
*                   the microcontroller.
*
* @param[in]        Power_Ip_pHwIPsConfigPtr  Pointer to LLD configuration structure (member of
*                   'Mcu_ConfigType' struct).
*
* @return           void
*
* @implements Power_Ip_PerformReset_Activity
*
*/
void Power_Ip_PerformReset(const Power_Ip_HwIPsConfigType * Power_Ip_pHwIPsConfigPtr)
{
    
    POWER_DEV_ASSERT(NULL_PTR != Power_Ip_pHwIPsConfigPtr);
    /*  
     *  Performs a microcontroller reset.
     *  A RESET mode requested via the ME_MCTL register is passed to the MC_RGM, which generates a
     *  global system reset and initiates the reset sequence.
     */
    
    Call_MC_RGM_PerformReset(Power_Ip_pHwIPsConfigPtr->MC_RGM_pConfig);
}
#endif /* (MCU_PERFORM_RESET_API == STD_ON) */

/**
* @brief            This function returns the reset reason.
* @details          This routine returns the Reset reason that is read from the hardware.
*
* @param[in]        None
*
* @return           void
*
* @implements Power_Ip_GetResetReason_Activity
*
*/
Power_Ip_ResetType Power_Ip_GetResetReason(void)
{
    Power_Ip_ResetType eResetReason = MCU_RESET_UNDEFINED;
#if (defined(MCU_ENTER_LOW_POWER_MODE) && (MCU_ENTER_LOW_POWER_MODE == STD_ON))
    Power_Ip_PowerModeType ePreviousMode;
#endif /*(MCU_ENTER_LOW_POWER_MODE == STD_ON) */

#if (defined(FEATURE_RESET_DURING_STANDBY_SUPPORTED))
  #if (FEATURE_RESET_DURING_STANDBY_SUPPORTED == STD_ON)
    #if (defined(MCU_ENTER_LOW_POWER_MODE) && (MCU_ENTER_LOW_POWER_MODE == STD_ON))
    if ( (boolean)FALSE ==(boolean)Call_MC_RGM_ResetDuringStandby() )
    {
    #endif
  #endif
#endif

    eResetReason = (Power_Ip_ResetType)Call_MC_RGM_GetResetReason();

#if (defined(FEATURE_RESET_DURING_STANDBY_SUPPORTED))
  #if (FEATURE_RESET_DURING_STANDBY_SUPPORTED == STD_ON)
    #if (defined(MCU_ENTER_LOW_POWER_MODE) && (MCU_ENTER_LOW_POWER_MODE == STD_ON))
    }
    else
    {
        ePreviousMode = Power_Ip_MC_ME_GetPreviousMode();
        if ( POWER_IP_SOC_STANDBY_MODE == ePreviousMode )
        {
            eResetReason = MCU_WAKEUP_REASON;
        }
    }
    #endif
  #endif
#endif

    return eResetReason;
}

/**
* @brief            This function returns the reset reason.
* @details          This routine returns the Reset reason that is read from the hardware.
*                   Called by:
*                       - Mcu_GetResetReason() from HLD.
*
* @param[in]        None
*
* @return           void
*
* @implements Power_Ip_GetResetRawValue_Activity
*
*/
Power_Ip_RawResetType Power_Ip_GetResetRawValue(void)
{
    Power_Ip_RawResetType ResetReason;

    /**  @violates @ref Mcu_IPW_c_REF_12 The comma operator shall not be used. */
    ResetReason = Call_MC_RGM_GetResetRawValue();
    return ResetReason;
}

/**
* @brief            This function initializes the reset module.
* @details          
*
* @param[in]        Power_Ip_pHwIPsConfigPtr  Pointer to LLD configuration structure.
*
* @return           void
*
* @implements Power_Ip_InitReset_Activity
*
*/
void Power_Ip_InitReset (const Power_Ip_HwIPsConfigType * Power_Ip_pHwIPsConfigPtr)
{
    POWER_DEV_ASSERT(NULL_PTR != Power_Ip_pHwIPsConfigPtr);
#if (defined(POWER_IP_ENABLE_USER_MODE_SUPPORT) && (STD_ON == POWER_IP_ENABLE_USER_MODE_SUPPORT))
#if (defined(MCAL_MC_RGM_REG_PROT_AVAILABLE))
  #if (STD_ON == MCAL_MC_RGM_REG_PROT_AVAILABLE)
    OsIf_Trusted_Call(MC_RGM_SetUserAccessAllowed);
  #endif
#endif
#if (defined(MCAL_RDC_REG_PROT_AVAILABLE))
  #if (STD_ON == MCAL_RDC_REG_PROT_AVAILABLE)
    OsIf_Trusted_Call(RDC_SetUserAccessAllowed);
  #endif
#endif
#endif
    Call_MC_RGM_ResetInit(Power_Ip_pHwIPsConfigPtr->MC_RGM_pConfig);
}

/**
* @brief            This function initializes the power module.
* @details          
*
* @param[in]        Power_Ip_pHwIPsConfigPtr  Pointer to LLD configuration structure.
*
* @return           void
*
* @implements Power_Ip_InitPower_Activity
*
*/
void Power_Ip_InitPower (const Power_Ip_HwIPsConfigType * Power_Ip_pHwIPsConfigPtr)
{
    POWER_DEV_ASSERT(NULL_PTR != Power_Ip_pHwIPsConfigPtr);
#if (defined(POWER_IP_ENABLE_USER_MODE_SUPPORT) && (STD_ON == POWER_IP_ENABLE_USER_MODE_SUPPORT))
#if (defined(MCAL_PMC_REG_PROT_AVAILABLE))
  #if (STD_ON == MCAL_PMC_REG_PROT_AVAILABLE)
    OsIf_Trusted_Call(PMC_SetUserAccessAllowed);
  #endif
#endif

#if (defined(MCU_ENTER_LOW_POWER_MODE) && (MCU_ENTER_LOW_POWER_MODE == STD_ON))
  #if (defined(MCAL_C40ASF_REG_PROT_AVAILABLE))
    #if(STD_ON == MCAL_C40ASF_REG_PROT_AVAILABLE)
    OsIf_Trusted_Call(Power_Ip_FLASH_C40ASF_SetUserAccessAllowed);
    #endif
  #endif
#endif
#endif
    PMC_PowerInit(Power_Ip_pHwIPsConfigPtr->PMC_pConfig);
}

/*FUNCTION**********************************************************************
 *
 * Function Name : PowerStartTimeout
 * Description   : Checks for timeout condition
 *
 *END**************************************************************************/
void PowerStartTimeout(uint32 *startTimeOut,
                       uint32 *elapsedTimeOut,
                       uint32 *timeoutTicksOut,
                       uint32 timeoutUs)
{
    *startTimeOut    = OsIf_GetCounter(POWER_TIMEOUT_TYPE);
    *elapsedTimeOut  = 0U;
    *timeoutTicksOut = OsIf_MicrosToTicks(timeoutUs, POWER_TIMEOUT_TYPE);
}

/*FUNCTION**********************************************************************
 *
 * Function Name : PowerTimeoutExpired
 * Description   : Checks for timeout expiration condition
 *
 *END**************************************************************************/
boolean PowerTimeoutExpired(uint32 *startTimeInOut,
                            uint32 *elapsedTimeInOut,
                            uint32 TimeoutTicks)
{
    *elapsedTimeInOut += OsIf_GetElapsed(startTimeInOut, POWER_TIMEOUT_TYPE);

    return ((*elapsedTimeInOut >= TimeoutTicks)? TRUE : FALSE);
}

/**
* @brief            This function installs a callback for reporting errors from power driver.
* @details          
*
* @param[in]        reportErrorsCallback    Callback to be installed.
*
* @return           void
*
* @implements Power_Ip_InstallNotificationsCallback_Activity
*/
void Power_Ip_InstallNotificationsCallback(Power_Ip_ReportErrorsCallbackType reportErrorsCallback)
{
    POWER_DEV_ASSERT(NULL_PTR != reportErrorsCallback);

    powerReportErrorsCallback = reportErrorsCallback;
}

void ReportPowerErrorsEmptyCallback(Power_Ip_PowerReportErrorType error, uint8 u8ErrorCode)
{
    /* No implementation */
    (void)error;
    (void)u8ErrorCode;
}

void ReportPowerErrors(Power_Ip_PowerReportErrorType error, uint8 u8ErrorCode)
{
    powerReportErrorsCallback(error, u8ErrorCode);
}

#define MCU_STOP_SEC_CODE
/**
* @violates @ref Power_Ip_c_REF_1 #include directives should only be preceded by preprocessor
* directives or comments.
*
* @violates @ref Power_Ip_c_REF_2 Precautions shall be taken in order to prevent the contents
* of a header file being included more than once.
*/
#include "Mcu_MemMap.h"


#ifdef __cplusplus
}
#endif

/** @} */

