/*==================================================================================================
*   Project              : RTD AUTOSAR 4.4
*   Platform             : CORTEXM
*   Peripheral           : FLEXIO
*   Dependencies         : 
*
*   Autosar Version      : 4.4.0
*   Autosar Revision     : ASR_REL_4_4_REV_0000
*   Autosar Conf.Variant :
*   SW Version           : 0.9.0
*   Build Version        : S32K3_RTD_0_9_0_P02_D2107_ASR_REL_4_4_REV_0000_20210716
*
*   (c) Copyright 2020 - 2021 NXP Semiconductors
*   All Rights Reserved.
*
*   NXP Confidential. This software is owned or controlled by NXP and may only be
*   used strictly in accordance with the applicable license terms. By expressly
*   accepting such terms or by downloading, installing, activating and/or otherwise
*   using the software, you are agreeing that you have read, and that you agree to
*   comply with and are bound by, such license terms. If you do not agree to be
*   bound by the applicable license terms, then you may not retain, install,
*   activate or otherwise use the software.
==================================================================================================*/

/**
*   @file Uart.c
*   @implements Uart.c_Artifact
*   @addtogroup UART
*   @{
*/


#ifdef __cplusplus
extern "C"
{
#endif

/*==================================================================================================
*                                        INCLUDE FILES
* 1) system and project includes
* 2) needed interfaces from external units
* 3) internal and external interfaces from this unit
==================================================================================================*/
#include "Uart.h"
#if ((UART_DEV_ERROR_DETECT == STD_ON) || (UART_RUNTIME_ERROR_DETECT ==STD_ON))
#include "Det.h"
#endif

/*==================================================================================================
*                              SOURCE FILE VERSION INFORMATION
==================================================================================================*/

#define UART_VENDOR_ID_C                      43
#define UART_AR_RELEASE_MAJOR_VERSION_C       4
#define UART_AR_RELEASE_MINOR_VERSION_C       4
#define UART_AR_RELEASE_REVISION_VERSION_C    0
#define UART_SW_MAJOR_VERSION_C               0
#define UART_SW_MINOR_VERSION_C               9
#define UART_SW_PATCH_VERSION_C               0

/*=================================================================================================*/

/*==================================================================================================
                                      FILE VERSION CHECKS
==================================================================================================*/
/* Check if current file and Uart header file are of the same Autosar version */
#if (UART_VENDOR_ID_C != UART_VENDOR_ID)
    #error "Uart.c and Uart.h have different vendor ids"
#endif
#if ((UART_AR_RELEASE_MAJOR_VERSION_C    != UART_AR_RELEASE_MAJOR_VERSION) || \
     (UART_AR_RELEASE_MINOR_VERSION_C    != UART_AR_RELEASE_MINOR_VERSION) || \
     (UART_AR_RELEASE_REVISION_VERSION_C != UART_AR_RELEASE_REVISION_VERSION))
    #error "AutoSar Version Numbers of Uart.c and Uart.h are different"
#endif
/* Check if current file and Uart header file are of the same Software version */
#if ((UART_SW_MAJOR_VERSION_C != UART_SW_MAJOR_VERSION) || \
     (UART_SW_MINOR_VERSION_C != UART_SW_MINOR_VERSION) || \
     (UART_SW_PATCH_VERSION_C != UART_SW_PATCH_VERSION) )
    #error "Software Version Numbers of Uart.c and Uart.h are different"
#endif
#ifndef DISABLE_MCAL_INTERMODULE_ASR_CHECK

#if (UART_DEV_ERROR_DETECT == STD_ON)
        /* Check if the source file and Det header file are of the same Autosar version */
        #if ((UART_AR_RELEASE_MAJOR_VERSION_C != DET_AR_RELEASE_MAJOR_VERSION) || \
             (UART_AR_RELEASE_MINOR_VERSION_C != DET_AR_RELEASE_MINOR_VERSION))
            #error "AutoSar Version Numbers of Uart.c and Det.h are different"
        #endif
    #endif
 #endif /* DISABLE_MCAL_INTERMODULE_ASR_CHECK */

/*==================================================================================================
                          LOCAL TYPEDEFS (STRUCTURES, UNIONS, ENUMS)
==================================================================================================*/

/*==================================================================================================
*                                       LOCAL MACROS
==================================================================================================*/

/*==================================================================================================
                                       LOCAL CONSTANTS
==================================================================================================*/

/*==================================================================================================
                                       LOCAL VARIABLES
==================================================================================================*/
#define UART_START_SEC_VAR_NO_INIT_UNSPECIFIED
/* @violates @ref Uart_c_REF_1 This violation is not fixed since the inclusion of Uart_MemMap.h is as per AUTOSAR requirement*/
#include "Uart_MemMap.h"
/**
* @brief          UART driver status variable.
* @details        UART driver initialization status variable.
*/
static Uart_DrvStatusType Uart_au8DriverStatus[UART_MAX_PARTITIONS];

#define UART_STOP_SEC_VAR_NO_INIT_UNSPECIFIED
/* @violates @ref Uart_c_REF_1 This violation is not fixed since the inclusion of Uart_MemMap.h is as per AUTOSAR requirement*/
#include "Uart_MemMap.h"

#define UART_START_SEC_VAR_NO_INIT_32
/* @violates @ref Uart_c_REF_1 This violation is not fixed since the inclusion of Uart_MemMap.h is as per AUTOSAR requirement*/
#include "Uart_MemMap.h"

/**
* @brief          Uart clock frequencies.
* @details        This variable retains the values of clock frequencies set at driver initialization for each channel.
*/
static uint32 Uart_u32ClockFrequency[UART_MAX_PARTITIONS][UART_CH_MAX_CONFIG];

#define UART_STOP_SEC_VAR_NO_INIT_32
/* @violates @ref Uart_c_REF_1 This violation is not fixed since the inclusion of Uart_MemMap.h is as per AUTOSAR requirement*/
#include "Uart_MemMap.h"
/*==================================================================================================
                                       GLOBAL CONSTANTS
==================================================================================================*/

/*==================================================================================================
                                       GLOBAL VARIABLES
==================================================================================================*/
#define UART_START_SEC_CONFIG_DATA_UNSPECIFIED
/* @violates @ref Uart_c_REF_1 This violation is not fixed since the inclusion of Uart_MemMap.h is as per AUTOSAR requirement*/
#include "Uart_MemMap.h"

#if (UART_PRECOMPILE_SUPPORT == STD_ON)
/**
* @brief          Global precompiled configuration pointer.
* @details        Pointer to the precompiled configuration structure.
*/
extern const Uart_ConfigType* const Uart_PBcfgVariantPredefined[UART_MAX_PARTITIONS];

#endif /* (UART_PRECOMPILE_SUPPORT == STD_OFF) */

#define UART_STOP_SEC_CONFIG_DATA_UNSPECIFIED
/* @violates @ref Uart_c_REF_1 This violation is not fixed since the inclusion of Uart_MemMap.h is as per AUTOSAR requirement*/
#include "Uart_MemMap.h"

#define UART_START_SEC_VAR_NO_INIT_UNSPECIFIED
/* @violates @ref Uart_c_REF_1 This violation is not fixed since the inclusion of Uart_MemMap.h is as per AUTOSAR requirement*/

#include "Uart_MemMap.h"
/**
* @brief          Global configuration pointer.
* @details        Pointer to the configuration structure.
*/
static const Uart_ConfigType * Uart_apConfig[UART_MAX_PARTITIONS];

#define UART_STOP_SEC_VAR_NO_INIT_UNSPECIFIED
/* @violates @ref Uart_c_REF_1 This violation is not fixed since the inclusion of Uart_MemMap.h is as per AUTOSAR requirement*/
#include "Uart_MemMap.h"

/*==================================================================================================
                                   LOCAL FUNCTION PROTOTYPES
==================================================================================================*/
#define UART_START_SEC_CODE
/* @violates @ref Uart_c_REF_1 This violation is not fixed since the inclusion of Uart_MemMap.h is as per AUTOSAR requirement*/
#include "Uart_MemMap.h"

static Std_ReturnType Uart_StartSyncReceive(uint8 u8CoreId, uint8 u8Channel, uint8* pBuffer, uint32 u32BufferSize);
static Std_ReturnType Uart_StartSyncSend(uint8 u8CoreId, uint8 u8Channel, const uint8* pBuffer, uint32 u32BufferSize);
static Std_ReturnType Uart_StartAsyncReceive(uint8 u8CoreId, uint8 u8Channel, uint8* pBuffer, uint32 u32BufferSize);
static Std_ReturnType Uart_StartAsyncSend(uint8 u8CoreId, uint8 u8Channel, const uint8* pBuffer, uint32 u32BufferSize);
static Std_ReturnType Uart_StartSetBuffer(uint8 u8CoreId, uint8 u8Channel, uint8* pBuffer, uint32 u32BufferSize, Uart_DataDirectionType eDir);

/*==================================================================================================
                                       LOCAL FUNCTIONS
==================================================================================================*/
/**
* @internal
* @brief  Start Receive data with polling method.
* @details This function will start receive data when Uart_Ipw_GetReceiveStatus do not in ONGOING status.
* @param[in] u8CoreId       Core ID
* @param[in] u8Channel      Uart channel used for transmission.
* @param[in] pBuffer        Buffer to receive.
* @param[in] u32BufferSize  The number of bytes to receive.
*
*
* @return   E_OK:               Reception ended successfully
*           E_NOT_OK:           Reception didn't end successfully
*           E_UART_TIMEOUT:     Reception has timeout
* This is not a public API as it is called from other driver functions.
*/
static Std_ReturnType Uart_StartSyncReceive(uint8 u8CoreId, uint8 u8Channel, uint8* pBuffer, uint32 u32BufferSize)
{
    Std_ReturnType tempReturn = E_NOT_OK;
    #if (UART_MULTICORE_SUPPORT == STD_ON)
    if (u8CoreId != Uart_apConfig[u8CoreId]->u32PartitionCoreId)
    {
        #if (UART_DEV_ERROR_DETECT == STD_ON)
        /* Invalid CoreId */
        (void)Det_ReportError((uint16)UART_MODULE_ID,
                        (uint8)0,
                        (uint8)UART_SYNCRECEIVE_ID,
                        (uint8)UART_E_PARAM_CONFIG);
        #endif /* (UART_DEV_ERROR_DETECT == STD_ON) */
    }
    else
    {
        if (Uart_apConfig[u8CoreId]->Configs[u8Channel] == NULL_PTR)
        {
            #if (UART_DEV_ERROR_DETECT == STD_ON)
            (void)Det_ReportError((uint16)UART_MODULE_ID,
                                    (uint8)0,
                                    (uint8)UART_SYNCRECEIVE_ID,
                                    (uint8)UART_E_INVALID_CHANNEL);
            #endif /* (UART_DEV_ERROR_DETECT     == STD_ON) */
        }
        else
        {
    #endif /* (UART_MULTICORE_SUPPORT == STD_ON) */
    if (UART_OPERATION_ONGOING == Uart_Ipw_GetReceiveStatus(u8Channel, NULL_PTR))
    {
        #if (UART_DEV_ERROR_DETECT == STD_ON)
        (void)Det_ReportError((uint16)UART_MODULE_ID,
                                (uint8)0,
                                (uint8)UART_SYNCRECEIVE_ID,
                                (uint8)UART_E_CHANNEL_BUSY);
        #endif /* (UART_DEV_ERROR_DETECT == STD_ON) */
    }
    else
    {

        tempReturn = Uart_Ipw_SyncReceive(u8Channel,
                                        pBuffer,
                                        u32BufferSize);
        (void)u8CoreId;

        if (E_UART_TIMEOUT == tempReturn)
        {
        #if (UART_RUNTIME_ERROR_DETECT == STD_ON)
            /* Timeout occur */
            (void)Det_ReportRuntimeError((uint16)UART_MODULE_ID,
                                    (uint8)0,
                                    (uint8)UART_SYNCRECEIVE_ID,
                                    (uint8)UART_E_TIMEOUT);
        #endif /* (UART_RUNTIME_ERROR_DETECT == STD_ON) */
            tempReturn = E_NOT_OK;
        }
    }

    #if (UART_MULTICORE_SUPPORT == STD_ON)
        }
    }
    #endif /* (UART_MULTICORE_SUPPORT == STD_ON) */
    return tempReturn;
}

/**
* @internal
* @brief  Start Transfer data with polling method.
* @details This function will start transfer data when Uart_Ipw_GetReceiveStatus do not in ONGOING status.
* @param[in] u8CoreId       Core ID
* @param[in] u8Channel      Uart channel used for transmission.
* @param[in] pBuffer        Buffer to send.
* @param[in] u32BufferSize  The number of bytes to send.
*
*
* @return   E_OK:               Reception ended successfully
*           E_NOT_OK:           Reception didn't end successfully
*           E_UART_TIMEOUT:     Reception has timeout
* This is not a public API as it is called from other driver functions.
*/
static Std_ReturnType Uart_StartSyncSend(uint8 u8CoreId, uint8 u8Channel, const uint8* pBuffer, uint32 u32BufferSize)
{
    Std_ReturnType tempReturn = E_NOT_OK;
    #if (UART_MULTICORE_SUPPORT == STD_ON)
    if (u8CoreId != Uart_apConfig[u8CoreId]->u32PartitionCoreId)
    {
        #if (UART_DEV_ERROR_DETECT == STD_ON)
        /* Invalid CoreId */
        (void)Det_ReportError((uint16)UART_MODULE_ID,
                        (uint8)0,
                        (uint8)UART_SYNCSEND_ID,
                        (uint8)UART_E_PARAM_CONFIG);
        #endif /* (UART_DEV_ERROR_DETECT == STD_ON) */
    }
    else
    {
        if (Uart_apConfig[u8CoreId]->Configs[u8Channel] == NULL_PTR)
        {
            #if (UART_DEV_ERROR_DETECT == STD_ON)
            (void)Det_ReportError((uint16)UART_MODULE_ID,
                                    (uint8)0,
                                    (uint8)UART_SYNCSEND_ID,
                                    (uint8)UART_E_INVALID_CHANNEL);
            #endif /* (UART_DEV_ERROR_DETECT     == STD_ON) */
        }
        else
        {
    #endif /* (UART_MULTICORE_SUPPORT == STD_ON) */
    if (UART_OPERATION_ONGOING == Uart_Ipw_GetTransmitStatus(u8Channel, NULL_PTR))
    {
        #if (UART_DEV_ERROR_DETECT == STD_ON)
        (void)Det_ReportError((uint16)UART_MODULE_ID,
                                (uint8)0,
                                (uint8)UART_SYNCSEND_ID,
                                (uint8)UART_E_CHANNEL_BUSY);
        #endif /* (UART_DEV_ERROR_DETECT == STD_ON) */
    }
    else
    {
        tempReturn = Uart_Ipw_SyncSend(u8Channel, pBuffer, u32BufferSize);
        (void)u8CoreId;

        if (E_UART_TIMEOUT == tempReturn)
        {
        #if (UART_RUNTIME_ERROR_DETECT == STD_ON)
                /* Timeout occur */
                (void)Det_ReportRuntimeError((uint16)UART_MODULE_ID,
                                        (uint8)0,
                                        (uint8)UART_SYNCSEND_ID,
                                        (uint8)UART_E_TIMEOUT);
        #endif /* (UART_RUNTIME_ERROR_DETECT == STD_ON) */
            tempReturn = E_NOT_OK;
        }
    }

    #if (UART_MULTICORE_SUPPORT == STD_ON)
        }
    }
    #endif /* (UART_MULTICORE_SUPPORT == STD_ON) */
    return tempReturn;
}

/**
* @internal
* @brief  Start Receive data with asynchronously method.
* @details This function will start receive data when Uart_Ipw_GetReceiveStatus do not in ONGOING status.
* @param[in] u8CoreId       Core ID
* @param[in] u8Channel      Uart channel used for transmission.
* @param[in] pBuffer        Buffer to receive.
* @param[in] u32BufferSize  The number of bytes to receive.
*
*
* @return   E_OK:               Reception ended successfully
*           E_NOT_OK:           Reception didn't end successfully
*           E_UART_TIMEOUT:     Reception has timeout
* This is not a public API as it is called from other driver functions.
*/
static Std_ReturnType Uart_StartAsyncReceive(uint8 u8CoreId, uint8 u8Channel, uint8* pBuffer, uint32 u32BufferSize)
{
    Std_ReturnType tempReturn = E_NOT_OK;
    #if (UART_MULTICORE_SUPPORT == STD_ON)
    if (u8CoreId != Uart_apConfig[u8CoreId]->u32PartitionCoreId)
    {
        #if (UART_DEV_ERROR_DETECT == STD_ON)
        /* Invalid CoreId */
        (void)Det_ReportError((uint16)UART_MODULE_ID,
                        (uint8)0,
                        (uint8)UART_ASYNCRECEIVE_ID,
                        (uint8)UART_E_PARAM_CONFIG);
        #endif /* (UART_DEV_ERROR_DETECT == STD_ON) */
    }
    else
    {
        if (Uart_apConfig[u8CoreId]->Configs[u8Channel] == NULL_PTR)
        {
            #if (UART_DEV_ERROR_DETECT == STD_ON)
            (void)Det_ReportError((uint16)UART_MODULE_ID,
                                    (uint8)0,
                                    (uint8)UART_ASYNCRECEIVE_ID,
                                    (uint8)UART_E_INVALID_CHANNEL);
            #endif /* (UART_DEV_ERROR_DETECT     == STD_ON) */
        }
        else
        {
            #endif /* (UART_MULTICORE_SUPPORT == STD_ON) */
            if (UART_OPERATION_ONGOING == Uart_Ipw_GetReceiveStatus(u8Channel, NULL_PTR))
            {
                #if (UART_DEV_ERROR_DETECT == STD_ON)
                (void)Det_ReportError((uint16)UART_MODULE_ID,
                                        (uint8)0,
                                        (uint8)UART_ASYNCRECEIVE_ID,
                                        (uint8)UART_E_CHANNEL_BUSY);
                #endif /* (UART_DEV_ERROR_DETECT == STD_ON) */
            }
            else
            {
                tempReturn = Uart_Ipw_AsyncReceive(u8Channel,
                                                   pBuffer,
                                                   u32BufferSize);
                (void)u8CoreId;
            }

            #if (UART_MULTICORE_SUPPORT == STD_ON)
        }
    }
            #endif /* (UART_MULTICORE_SUPPORT == STD_ON) */
    return tempReturn;
}

/**
* @internal
* @brief  Start Transfer data with asynchronously method.
* @details This function will start transfer data when Uart_Ipw_GetReceiveStatus do not in ONGOING status.
* @param[in] u8CoreId       Core ID
* @param[in] u8Channel      Uart channel used for transmission.
* @param[in] pBuffer        Buffer to send.
* @param[in] u32BufferSize  The number of bytes to send.
*
*
* @return   E_OK:               Reception ended successfully
*           E_NOT_OK:           Reception didn't end successfully
*           E_UART_TIMEOUT:     Reception has timeout
* This is not a public API as it is called from other driver functions.
*/
static Std_ReturnType Uart_StartAsyncSend(uint8 u8CoreId, uint8 u8Channel, const uint8* pBuffer, uint32 u32BufferSize)
{
    Std_ReturnType tempReturn = E_NOT_OK;

    #if (UART_MULTICORE_SUPPORT == STD_ON)
    if (u8CoreId != Uart_apConfig[u8CoreId]->u32PartitionCoreId)
    {
        #if (UART_DEV_ERROR_DETECT == STD_ON)
        /* Invalid CoreId */
        (void)Det_ReportError((uint16)UART_MODULE_ID,
                        (uint8)0,
                        (uint8)UART_ASYNCSEND_ID,
                        (uint8)UART_E_PARAM_CONFIG);
        #endif /* (UART_DEV_ERROR_DETECT == STD_ON) */
    }
    else
    {
        if (Uart_apConfig[u8CoreId]->Configs[u8Channel] == NULL_PTR)
        {
            #if (UART_DEV_ERROR_DETECT == STD_ON)
            (void)Det_ReportError((uint16)UART_MODULE_ID,
                                    (uint8)0,
                                    (uint8)UART_ASYNCSEND_ID,
                                    (uint8)UART_E_INVALID_CHANNEL);
            #endif /* (UART_DEV_ERROR_DETECT     == STD_ON) */
        }
        else
        {
            #endif /* (UART_MULTICORE_SUPPORT == STD_ON) */
            if (UART_OPERATION_ONGOING == Uart_Ipw_GetTransmitStatus(u8Channel, NULL_PTR))
            {
                #if (UART_DEV_ERROR_DETECT == STD_ON)
                (void)Det_ReportError((uint16)UART_MODULE_ID,
                                        (uint8)0,
                                        (uint8)UART_ASYNCSEND_ID,
                                        (uint8)UART_E_CHANNEL_BUSY);
                #endif /* (UART_DEV_ERROR_DETECT == STD_ON) */
            }
            else
            {
                tempReturn = Uart_Ipw_AsyncSend(u8Channel,
                                                pBuffer,
                                                u32BufferSize);
                (void)u8CoreId;
            }
            #if (UART_MULTICORE_SUPPORT == STD_ON)
        }
    }
            #endif /* (UART_MULTICORE_SUPPORT == STD_ON) */
    return tempReturn;
}

/**
* @internal
* @brief  Sets a tx or rx buffer.
* @details This function will start set up a buffer to receive or transmit.
* @param[in] u8CoreId       Core ID
* @param[in] u8Channel      Uart channel used for transmission.
* @param[in] pBuffer        Buffer to send.
* @param[in] u32BufferSize  The number of bytes to send.
* @param[in] eDir           Transmission type.
*
* @return   E_OK:               Reception ended successfully
*           E_NOT_OK:           Reception didn't end successfully
* This is not a public API as it is called from other driver functions.
*/
static Std_ReturnType Uart_StartSetBuffer(uint8 u8CoreId, uint8 u8Channel, uint8* pBuffer, uint32 u32BufferSize, Uart_DataDirectionType eDir)
{
    Std_ReturnType tempReturn = E_NOT_OK;

    #if (UART_MULTICORE_SUPPORT == STD_ON)
    if (u8CoreId != Uart_apConfig[u8CoreId]->u32PartitionCoreId)
    {
        #if (UART_DEV_ERROR_DETECT == STD_ON)
        /* Invalid CoreId */
        (void)Det_ReportError((uint16)UART_MODULE_ID,
                        (uint8)0,
                        (uint8)UART_SETBUFFER_ID,
                        (uint8)UART_E_PARAM_CONFIG);
        #endif /* (UART_DEV_ERROR_DETECT == STD_ON) */
    }
    else
    {
        if (Uart_apConfig[u8CoreId]->Configs[u8Channel] == NULL_PTR)
        {
            #if (UART_DEV_ERROR_DETECT == STD_ON)
            (void)Det_ReportError((uint16)UART_MODULE_ID,
                                    (uint8)0,
                                    (uint8)UART_SETBUFFER_ID,
                                    (uint8)UART_E_INVALID_CHANNEL);
            #endif /* (UART_DEV_ERROR_DETECT     == STD_ON) */
        }
        else
        {
    #endif /* (UART_MULTICORE_SUPPORT == STD_ON) */
            if (UART_SEND == eDir)
            {
                Uart_Ipw_SetTxBuffer(u8Channel,(const uint8 *)pBuffer, u32BufferSize);
            }
            else
            {
                Uart_Ipw_SetRxBuffer(u8Channel, pBuffer, u32BufferSize);
            }
            tempReturn = E_OK;
            (void)u8CoreId;
                    #if (UART_MULTICORE_SUPPORT == STD_ON)
            }
        }
        #endif /* (UART_MULTICORE_SUPPORT == STD_ON) */
    return tempReturn;
}
/*==================================================================================================
                                       GLOBAL FUNCTIONS
==================================================================================================*/
/* implements     Uart_Init_Activity*/
void Uart_Init(const Uart_ConfigType * pConfig)
{
    uint8 u8ChLoop;
    uint8 u8CoreId;
    uint8 u8Channel;

    /* Get core ID of current processor */
    u8CoreId = (uint8) Uart_GetCoreID();

    /* Check whether the Uart driver is in UART_UNINIT state */
    if (UART_UNINIT != Uart_au8DriverStatus[u8CoreId])
    {
        #if (UART_DEV_ERROR_DETECT == STD_ON)
        /* Uart driver has been already initialized */
        (void)Det_ReportError((uint16)UART_MODULE_ID,
                                (uint8)0,
                                (uint8)UART_INIT_ID,
                                (uint8)UART_E_ALREADY_INITIALIZED);
        #endif /* (UART_DEV_ERROR_DETECT == STD_ON) */
    }
    else
    {
        #if (UART_PRECOMPILE_SUPPORT == STD_OFF)
        /* Check Config for not being a null pointer */
        if(NULL_PTR == pConfig)
        {
            #if (UART_DEV_ERROR_DETECT == STD_ON)

            /* Invalid pointer */
            (void)Det_ReportError((uint16)UART_MODULE_ID, \
                                    (uint8)0, \
                                    (uint8)UART_INIT_ID, \
                                    (uint8)UART_E_INVALID_POINTER \
                                    );
            #endif /* (UART_DEV_ERROR_DETECT == STD_ON) */
        }
        #if (UART_MULTICORE_SUPPORT == STD_ON)
        else if (u8CoreId != pConfig->u32PartitionCoreId)
        {
            #if (UART_DEV_ERROR_DETECT == STD_ON)
            /* Invalid CoreId */
            (void)Det_ReportError((uint16)UART_MODULE_ID,
                                (uint8)0,
                                (uint8)UART_INIT_ID,
                                (uint8)UART_E_PARAM_CONFIG);
            #endif /* (UART_DEV_ERROR_DETECT == STD_ON) */
        }
        #endif /* (UART_MULTICORE_SUPPORT == STD_ON) */
        else
        {
            Uart_apConfig[u8CoreId] = pConfig;

        #else /* UART_PRECOMPILE_SUPPORT == STD_ON */
        if (NULL_PTR != pConfig)
        {
        #if (UART_DEV_ERROR_DETECT == STD_ON)
            /* Invalid pointer */
            (void)Det_ReportError((uint16)UART_MODULE_ID,
                                (uint8)0,
                                (uint8)UART_INIT_ID,
                                (uint8)UART_E_INVALID_POINTER);
        #endif /* (UART_DEV_ERROR_DETECT == STD_ON) */
        }
        #if (UART_MULTICORE_SUPPORT == STD_ON)
        else if ((NULL_PTR == Uart_PBcfgVariantPredefined[u8CoreId]) ||
                (u8CoreId != Uart_PBcfgVariantPredefined[u8CoreId]->u32PartitionCoreId))
        {
            #if (UART_DEV_ERROR_DETECT == STD_ON)
            /* Invalid CoreId */
            (void)Det_ReportError((uint16)UART_MODULE_ID,
                            (uint8)0,
                            (uint8)UART_INIT_ID,
                            (uint8)UART_E_PARAM_CONFIG);
            #endif /* UART_DEV_ERROR_DETECT == STD_ON */
        }
        #endif /* UART_MULTICORE_SUPPORT == STD_ON */
        else
        {
            Uart_apConfig[u8CoreId] = Uart_PBcfgVariantPredefined[u8CoreId];
            (void)pConfig;

    #endif /* UART_PRECOMPILE_SUPPORT == STD_ON */

            for (u8ChLoop = 0U; u8ChLoop < UART_CH_MAX_CONFIG; u8ChLoop++)
            {
                #if (UART_MULTICORE_SUPPORT == STD_ON)
                if(Uart_apConfig[u8CoreId]->Configs[u8ChLoop] != NULL_PTR)
                {
                    if (u8CoreId == Uart_apConfig[u8CoreId]->Configs[u8ChLoop]->u32ChannelCoreId)
                    {
                #endif /* UART_MULTICORE_SUPPORT == STD_ON */
                        u8Channel = Uart_apConfig[u8CoreId]->Configs[u8ChLoop]->u8UartChannelId;
                #if (STD_ON == UART_ENABLE_USER_MODE_SUPPORT)

                        Uart_Ipw_SetUserAccessAllowed(u8Channel);
                #endif /* STD_ON == UART_ENABLE_USER_MODE_SUPPORT */
                        Uart_Ipw_Init(u8Channel, Uart_apConfig[u8CoreId]->Configs[u8ChLoop]->pUartChannelConfig);
                        Uart_u32ClockFrequency[u8CoreId][u8Channel] = Uart_apConfig[u8CoreId]->Configs[u8ChLoop]->u32ChannelClockFrequency;
                        Uart_au8DriverStatus[u8CoreId] = UART_INIT;
                #if (UART_MULTICORE_SUPPORT == STD_ON)
                    }
                }
                #endif /* UART_MULTICORE_SUPPORT == STD_ON */
            }
            /* Update UART driver status to UART_INIT */
        }
    }
}
/* implements     Uart_Deinit_Activity*/
void Uart_Deinit(void)
{
    uint8 u8ChLoop = 0U;
    uint8 u8CoreId = 0U;

    /* Get core ID of current processor */
    u8CoreId = (uint8) Uart_GetCoreID();

    /* Check whether the UART driver is in UART_UNINIT state */
    if (UART_INIT != Uart_au8DriverStatus[u8CoreId])
    {
        #if (UART_DEV_ERROR_DETECT == STD_ON)
        /* Uart driver has been already initialized */
        (void)Det_ReportError((uint16)UART_MODULE_ID,
                                (uint8)0,
                                (uint8)UART_DEINIT_ID,
                                (uint8)UART_E_UNINIT);
        #endif /* (UART_DEV_ERROR_DETECT == STD_ON) */
    }
    else
    {
        for(u8ChLoop = 0U; u8ChLoop < UART_CH_MAX_CONFIG; u8ChLoop++)
        {
            #if (UART_MULTICORE_SUPPORT == STD_ON)
            if(Uart_apConfig[u8CoreId]->Configs[u8ChLoop] != NULL_PTR)
            {
                if (u8CoreId == Uart_apConfig[u8CoreId]->Configs[u8ChLoop]->u32ChannelCoreId)
                {
            #endif /* UART_MULTICORE_SUPPORT == STD_ON */

                    /* Deinit channel */
                    Uart_Ipw_Deinit(Uart_apConfig[u8CoreId]->Configs[u8ChLoop]->u8UartChannelId);
                    Uart_au8DriverStatus[u8CoreId] = UART_UNINIT;
            #if (UART_MULTICORE_SUPPORT == STD_ON)
                }
            }
            #endif /* UART_MULTICORE_SUPPORT == STD_ON */
        }
        Uart_apConfig[u8CoreId] = NULL_PTR;
    }
}


/* implements     Uart_SetBaudrate_Activity*/
Std_ReturnType Uart_SetBaudrate(uint8 u8Channel, Uart_BaudrateType baudrate)
{
    Std_ReturnType tempReturn = E_NOT_OK;
    uint8 u8CoreId;

    u8CoreId = (uint8) Uart_GetCoreID();

    if (UART_UNINIT == Uart_au8DriverStatus[u8CoreId])
    {
        #if (UART_DEV_ERROR_DETECT == STD_ON)
        (void)Det_ReportError((uint16)UART_MODULE_ID,
                                (uint8)0,
                                (uint8)UART_SETBAUDRATE_ID,
                                (uint8)UART_E_UNINIT);
        #endif /* (UART_DEV_ERROR_DETECT     == STD_ON) */

    }
    else
    {
        if (u8Channel >= UART_CH_MAX_CONFIG)
        {
            #if (UART_DEV_ERROR_DETECT == STD_ON)
            (void)Det_ReportError((uint16)UART_MODULE_ID,
                                    (uint8)0,
                                    (uint8)UART_SETBAUDRATE_ID,
                                    (uint8)UART_E_INVALID_CHANNEL);
            #endif /* (UART_DEV_ERROR_DETECT     == STD_ON) */
        }
        else
        {
            #if (UART_MULTICORE_SUPPORT == STD_ON)
            if (u8CoreId != Uart_apConfig[u8CoreId]->u32PartitionCoreId)
            {
                #if (UART_DEV_ERROR_DETECT == STD_ON)
                /* Invalid CoreId */
                (void)Det_ReportError((uint16)UART_MODULE_ID,
                                (uint8)0,
                                (uint8)UART_SETBAUDRATE_ID,
                                (uint8)UART_E_PARAM_CONFIG);
                #endif /* (UART_DEV_ERROR_DETECT == STD_ON) */
            }
            else
            {
                if (Uart_apConfig[u8CoreId]->Configs[u8Channel] == NULL_PTR)
                {
                    #if (UART_DEV_ERROR_DETECT == STD_ON)
                    (void)Det_ReportError((uint16)UART_MODULE_ID,
                                            (uint8)0,
                                            (uint8)UART_SETBAUDRATE_ID,
                                            (uint8)UART_E_INVALID_CHANNEL);
                    #endif /* (UART_DEV_ERROR_DETECT     == STD_ON) */
                }
                else
                {
            #endif /* (UART_MULTICORE_SUPPORT == STD_ON) */
                    /* Check whether channel is busy sending or receiving */
                    if ((UART_OPERATION_ONGOING == Uart_Ipw_GetReceiveStatus(u8Channel, NULL_PTR)) ||
                        (UART_OPERATION_ONGOING == Uart_Ipw_GetTransmitStatus(u8Channel, NULL_PTR)))
                    {
                        #if (UART_DEV_ERROR_DETECT == STD_ON)
                        (void)Det_ReportError((uint16)UART_MODULE_ID,
                                                (uint8)0,
                                                (uint8)UART_SETBAUDRATE_ID,
                                                (uint8)UART_E_CHANNEL_BUSY);
                        #endif /* (UART_DEV_ERROR_DETECT     == STD_ON) */
                    }
                    else
                    {
                        tempReturn = Uart_Ipw_SetBaudrate(u8Channel, baudrate, Uart_u32ClockFrequency[u8CoreId][u8Channel]);
                    }

            #if (UART_MULTICORE_SUPPORT == STD_ON)
                }
            }
            #endif /* (UART_MULTICORE_SUPPORT == STD_ON) */
        }
    }
    return tempReturn;
}


/* implements     Uart_GetBaudrate_Activity*/
Std_ReturnType Uart_GetBaudrate(uint8 u8Channel, uint32 *baudrate)
{
    Std_ReturnType tempReturn = E_NOT_OK;
    uint8 u8CoreId;

    /* Get core ID of current processor */
    u8CoreId = (uint8) Uart_GetCoreID();

    if (UART_UNINIT == Uart_au8DriverStatus[u8CoreId])
    {
        #if (UART_DEV_ERROR_DETECT == STD_ON)
        (void)Det_ReportError((uint16)UART_MODULE_ID,
                                (uint8)0,
                                (uint8)UART_GETBAUDRATE_ID,
                                (uint8)UART_E_UNINIT);
        #endif /* (UART_DEV_ERROR_DETECT == STD_ON) */
    }
    else
    {
        if (u8Channel >= UART_CH_MAX_CONFIG)
        {
            #if (UART_DEV_ERROR_DETECT == STD_ON)
            (void)Det_ReportError((uint16)UART_MODULE_ID,
                                    (uint8)0,
                                    (uint8)UART_GETBAUDRATE_ID,
                                    (uint8)UART_E_INVALID_CHANNEL);
            #endif /* (UART_DEV_ERROR_DETECT == STD_ON) */
        }
        else
        {
            if (NULL_PTR == baudrate)
            {
                #if (UART_DEV_ERROR_DETECT == STD_ON)
                (void)Det_ReportError((uint16)UART_MODULE_ID,
                                        (uint8)0,
                                        (uint8)UART_GETBAUDRATE_ID,
                                        (uint8)UART_E_INVALID_POINTER);
                #endif /* (UART_DEV_ERROR_DETECT == STD_ON) */
            }
            else
            {
                #if (UART_MULTICORE_SUPPORT == STD_ON)
                if (u8CoreId != Uart_apConfig[u8CoreId]->u32PartitionCoreId)
                {
                    #if (UART_DEV_ERROR_DETECT == STD_ON)
                    /* Invalid CoreId */
                    (void)Det_ReportError((uint16)UART_MODULE_ID,
                                    (uint8)0,
                                    (uint8)UART_GETBAUDRATE_ID,
                                    (uint8)UART_E_PARAM_CONFIG);
                    #endif /* (UART_DEV_ERROR_DETECT == STD_ON) */
                }
                else
                {
                    if (Uart_apConfig[u8CoreId]->Configs[u8Channel] == NULL_PTR)
                    {
                        #if (UART_DEV_ERROR_DETECT == STD_ON)
                        (void)Det_ReportError((uint16)UART_MODULE_ID,
                                                (uint8)0,
                                                (uint8)UART_GETBAUDRATE_ID,
                                                (uint8)UART_E_INVALID_CHANNEL);
                        #endif /* (UART_DEV_ERROR_DETECT     == STD_ON) */
                    }
                    else
                    {
                #endif /* (UART_MULTICORE_SUPPORT == STD_ON) */

                        Uart_Ipw_GetBaudrate(u8Channel, baudrate);
                        tempReturn = E_OK;
                        (void)u8CoreId;

                #if (UART_MULTICORE_SUPPORT == STD_ON)
                    }
                }
                #endif /* (UART_DEV_ERROR_DETECT == STD_ON) */
            }
        }
    }
    return tempReturn;
}


/* implements     Uart_Abort_Activity*/
Std_ReturnType Uart_Abort(uint8 u8Channel, Uart_DataDirectionType eTransmissionType)
{
    Std_ReturnType tempReturn = E_NOT_OK;
    uint8 u8CoreId;

   /* Get core ID of current processor */
    u8CoreId = (uint8) Uart_GetCoreID();

    if (UART_UNINIT == Uart_au8DriverStatus[u8CoreId])
    {
        #if (UART_DEV_ERROR_DETECT == STD_ON)
        (void)Det_ReportError((uint16)UART_MODULE_ID,
                                (uint8)0,
                                (uint8)UART_ABORT_ID,
                                (uint8)UART_E_UNINIT);
        #endif /* (UART_DEV_ERROR_DETECT == STD_ON) */
    }
    else
    {
        if (u8Channel >= UART_CH_MAX_CONFIG)
        {
            #if (UART_DEV_ERROR_DETECT == STD_ON)
            (void)Det_ReportError((uint16)UART_MODULE_ID,
                                (uint8)0,
                                (uint8)UART_ABORT_ID,
                                (uint8)UART_E_INVALID_CHANNEL);
            #endif /* (UART_DEV_ERROR_DETECT == STD_ON) */
        }
        else
        {
            #if (UART_MULTICORE_SUPPORT == STD_ON)
            if (u8CoreId != Uart_apConfig[u8CoreId]->u32PartitionCoreId)
            {
                #if (UART_DEV_ERROR_DETECT == STD_ON)
                /* Invalid CoreId */
                (void)Det_ReportError((uint16)UART_MODULE_ID,
                                (uint8)0,
                                (uint8)UART_ABORT_ID,
                                (uint8)UART_E_PARAM_CONFIG);
                #endif /* (UART_DEV_ERROR_DETECT == STD_ON) */
            }
            else
            {
                if (Uart_apConfig[u8CoreId]->Configs[u8Channel] == NULL_PTR)
                {
                    #if (UART_DEV_ERROR_DETECT == STD_ON)
                    (void)Det_ReportError((uint16)UART_MODULE_ID,
                                            (uint8)0,
                                            (uint8)UART_ABORT_ID,
                                            (uint8)UART_E_INVALID_CHANNEL);
                    #endif /* (UART_DEV_ERROR_DETECT     == STD_ON) */
                }
                else
                {
            #endif /* (UART_MULTICORE_SUPPORT == STD_ON) */

                    if (UART_SEND == eTransmissionType)
                    {
                        tempReturn = Uart_Ipw_AbortSendingData(u8Channel);

                    }
                    else
                    {
                        tempReturn = Uart_Ipw_AbortReceivingData(u8Channel);

                    }
                    (void)u8CoreId;

            #if (UART_MULTICORE_SUPPORT == STD_ON)
                }
            }
            #endif /* (UART_MULTICORE_SUPPORT == STD_ON) */
        }
    }
    return tempReturn;
}

/* implements     Uart_SetBuffer_Activity*/
Std_ReturnType Uart_SetBuffer(uint8 u8Channel,  uint8* pBuffer, uint32 u32BufferSize, Uart_DataDirectionType eDir)
{
    Std_ReturnType tempReturn = E_NOT_OK;
    uint8 u8CoreId;

   /* Get core ID of current processor */
    u8CoreId = (uint8) Uart_GetCoreID();

    if (UART_UNINIT == Uart_au8DriverStatus[u8CoreId])
    {
        #if (UART_DEV_ERROR_DETECT == STD_ON)
        (void)Det_ReportError((uint16)UART_MODULE_ID,
                            (uint8)0,
                            (uint8)UART_SETBUFFER_ID,
                            (uint8)UART_E_UNINIT);
        #endif /* (UART_DEV_ERROR_DETECT == STD_ON) */
    }
    else
    {
        if (u8Channel >= UART_CH_MAX_CONFIG)
        {
            #if (UART_DEV_ERROR_DETECT == STD_ON)
            (void)Det_ReportError((uint16)UART_MODULE_ID,
                                (uint8)0,
                                (uint8)UART_SETBUFFER_ID,
                                (uint8)UART_E_INVALID_CHANNEL);
            #endif /* (UART_DEV_ERROR_DETECT == STD_ON) */
        }
        else
        {
            if (NULL_PTR == pBuffer)
            {
                #if (UART_DEV_ERROR_DETECT == STD_ON)
                (void)Det_ReportError((uint16)UART_MODULE_ID,
                                    (uint8)0,
                                    (uint8)UART_SETBUFFER_ID,
                                    (uint8)UART_E_INVALID_POINTER);
                #endif /* (UART_DEV_ERROR_DETECT == STD_ON) */
            }
            else
            {
                if (0U == u32BufferSize)
                {
                    #if (UART_DEV_ERROR_DETECT == STD_ON)
                    (void)Det_ReportError((uint16)UART_MODULE_ID,
                                        (uint8)0,
                                        (uint8)UART_SETBUFFER_ID,
                                        (uint8)UART_E_INVALID_PARAMETER);
                    #endif /* (UART_DEV_ERROR_DETECT == STD_ON) */
                }
                else
                {
                    tempReturn = Uart_StartSetBuffer(u8CoreId, u8Channel, pBuffer, u32BufferSize, eDir);
                }
            }
        }
    }
    return tempReturn;
}



/* implements     Uart_SyncSend_Activity*/
Std_ReturnType Uart_SyncSend(uint8 u8Channel, const uint8* pBuffer, uint32 u32BufferSize)
{
    Std_ReturnType tempReturn = E_NOT_OK;
    uint8 u8CoreId;

   /* Get core ID of current processor */
    u8CoreId = (uint8) Uart_GetCoreID();

    if (UART_UNINIT == Uart_au8DriverStatus[u8CoreId])
    {
        #if (UART_DEV_ERROR_DETECT == STD_ON)
        (void)Det_ReportError((uint16)UART_MODULE_ID,
                                (uint8)0,
                                (uint8)UART_SYNCSEND_ID,
                                (uint8)UART_E_UNINIT);
        #endif /* (UART_DEV_ERROR_DETECT == STD_ON) */
    }
    else
    {
        if (u8Channel >= UART_CH_MAX_CONFIG)
        {
            #if (UART_DEV_ERROR_DETECT == STD_ON)
            (void)Det_ReportError((uint16)UART_MODULE_ID,
                                    (uint8)0,
                                    (uint8)UART_SYNCSEND_ID,
                                    (uint8)UART_E_INVALID_CHANNEL);
            #endif /* (UART_DEV_ERROR_DETECT == STD_ON) */
        }
        else
        {
            if (NULL_PTR == pBuffer)
            {
                #if (UART_DEV_ERROR_DETECT == STD_ON)
                (void)Det_ReportError((uint16)UART_MODULE_ID,
                                        (uint8)0,
                                        (uint8)UART_SYNCSEND_ID,
                                        (uint8)UART_E_INVALID_POINTER);
                #endif /* (UART_DEV_ERROR_DETECT == STD_ON) */
            }
            else
            {
                if (0U == u32BufferSize)
                {
                    #if (UART_DEV_ERROR_DETECT == STD_ON)
                    (void)Det_ReportError((uint16)UART_MODULE_ID,
                                            (uint8)0,
                                            (uint8)UART_SYNCSEND_ID,
                                            (uint8)UART_E_INVALID_PARAMETER);
                    #endif /* (UART_DEV_ERROR_DETECT == STD_ON) */
                }
                else
                {
                    tempReturn = Uart_StartSyncSend(u8CoreId, u8Channel, pBuffer, u32BufferSize);
                }
            }
        }
    }
    return tempReturn;
}




/* implements     Uart_SyncReceive_Activity*/
Std_ReturnType Uart_SyncReceive(uint8 u8Channel, uint8* pBuffer, uint32 u32BufferSize)
{
    Std_ReturnType tempReturn = E_NOT_OK;
    uint8 u8CoreId;

   /* Get core ID of current processor */
    u8CoreId = (uint8) Uart_GetCoreID();

    if (UART_UNINIT == Uart_au8DriverStatus[u8CoreId])
    {
        #if (UART_DEV_ERROR_DETECT == STD_ON)
        (void)Det_ReportError((uint16)UART_MODULE_ID,
                                (uint8)0,
                                (uint8)UART_SYNCRECEIVE_ID,
                                (uint8)UART_E_UNINIT);
        #endif /* (UART_DEV_ERROR_DETECT == STD_ON) */
    }
    else
    {
        if (u8Channel >= UART_CH_MAX_CONFIG)
        {
            #if (UART_DEV_ERROR_DETECT == STD_ON)
            (void)Det_ReportError((uint16)UART_MODULE_ID,
                                    (uint8)0,
                                    (uint8)UART_SYNCRECEIVE_ID,
                                    (uint8)UART_E_INVALID_CHANNEL);
            #endif /* (UART_DEV_ERROR_DETECT == STD_ON) */
        }
        else
        {
            if (NULL_PTR == pBuffer)
            {
                #if (UART_DEV_ERROR_DETECT == STD_ON)
                (void)Det_ReportError((uint16)UART_MODULE_ID,
                                        (uint8)0,
                                        (uint8)UART_SYNCRECEIVE_ID,
                                        (uint8)UART_E_INVALID_POINTER);
                #endif /* (UART_DEV_ERROR_DETECT == STD_ON) */
            }
            else
            {
                if (0U == u32BufferSize)
                {
                    #if (UART_DEV_ERROR_DETECT == STD_ON)
                    (void)Det_ReportError((uint16)UART_MODULE_ID,
                                            (uint8)0,
                                            (uint8)UART_SYNCRECEIVE_ID,
                                            (uint8)UART_E_INVALID_PARAMETER);
                    #endif /* (UART_DEV_ERROR_DETECT == STD_ON) */
                }
                else
                {
                    tempReturn = Uart_StartSyncReceive(u8CoreId, u8Channel, pBuffer, u32BufferSize);
                }
            }
        }
    }
    return tempReturn;
}


/* implements     Uart_AsyncReceive_Activity*/
Std_ReturnType Uart_AsyncReceive(uint8 u8Channel, uint8* pBuffer, uint32 u32BufferSize)
{
    Std_ReturnType tempReturn = E_NOT_OK;
    uint8 u8CoreId;

   /* Get core ID of current processor */
    u8CoreId = (uint8) Uart_GetCoreID();

    if (UART_UNINIT == Uart_au8DriverStatus[u8CoreId])
    {
        #if (UART_DEV_ERROR_DETECT == STD_ON)
        (void)Det_ReportError((uint16)UART_MODULE_ID,
                                (uint8)0,
                                (uint8)UART_ASYNCRECEIVE_ID,
                                (uint8)UART_E_UNINIT);
        #endif /* (UART_DEV_ERROR_DETECT == STD_ON) */
    }
    else
    {
        if (u8Channel >= UART_CH_MAX_CONFIG)
        {
            #if (UART_DEV_ERROR_DETECT == STD_ON)
            (void)Det_ReportError((uint16)UART_MODULE_ID,
                                    (uint8)0,
                                    (uint8)UART_ASYNCRECEIVE_ID,
                                    (uint8)UART_E_INVALID_CHANNEL);
            #endif /* (UART_DEV_ERROR_DETECT == STD_ON) */
        }
        else
        {
            if (NULL_PTR == pBuffer)
            {
                #if (UART_DEV_ERROR_DETECT == STD_ON)
                (void)Det_ReportError((uint16)UART_MODULE_ID,
                                        (uint8)0,
                                        (uint8)UART_ASYNCRECEIVE_ID,
                                        (uint8)UART_E_INVALID_POINTER);
                #endif /* (UART_DEV_ERROR_DETECT == STD_ON) */
            }
            else
            {
                if (0U == u32BufferSize)
                {
                    #if (UART_DEV_ERROR_DETECT == STD_ON)
                    (void)Det_ReportError((uint16)UART_MODULE_ID,
                                            (uint8)0,
                                            (uint8)UART_ASYNCRECEIVE_ID,
                                            (uint8)UART_E_INVALID_PARAMETER);
                    #endif /* (UART_DEV_ERROR_DETECT == STD_ON) */
                }
                else
                {
                    tempReturn = Uart_StartAsyncReceive(u8CoreId, u8Channel, pBuffer, u32BufferSize);
                }
            }
        }
    }
    return tempReturn;
}


/* implements     Uart_AsyncSend_Activity*/
Std_ReturnType Uart_AsyncSend(uint8 u8Channel, const uint8* pBuffer, uint32 u32BufferSize)
{
    Std_ReturnType tempReturn = E_NOT_OK;
    uint8 u8CoreId;

   /* Get core ID of current processor */
    u8CoreId = (uint8) Uart_GetCoreID();

    if (UART_UNINIT == Uart_au8DriverStatus[u8CoreId])
    {
        #if (UART_DEV_ERROR_DETECT == STD_ON)
        (void)Det_ReportError((uint16)UART_MODULE_ID,
                                (uint8)0,
                                (uint8)UART_ASYNCSEND_ID,
                                (uint8)UART_E_UNINIT);
        #endif /* (UART_DEV_ERROR_DETECT == STD_ON) */
    }
    else
    {
        if (u8Channel >= UART_CH_MAX_CONFIG)
        {
            #if (UART_DEV_ERROR_DETECT == STD_ON)
            (void)Det_ReportError((uint16)UART_MODULE_ID,
                                    (uint8)0,
                                    (uint8)UART_ASYNCSEND_ID,
                                    (uint8)UART_E_INVALID_CHANNEL);
            #endif /* (UART_DEV_ERROR_DETECT == STD_ON) */
        }
        else
        {
            if (NULL_PTR == pBuffer)
            {
                #if (UART_DEV_ERROR_DETECT == STD_ON)
                (void)Det_ReportError((uint16)UART_MODULE_ID,
                                        (uint8)0,
                                        (uint8)UART_ASYNCSEND_ID,
                                        (uint8)UART_E_INVALID_POINTER);
                #endif /* (UART_DEV_ERROR_DETECT == STD_ON) */
            }
            else
            {
                if (0U == u32BufferSize)
                {
                    #if (UART_DEV_ERROR_DETECT == STD_ON)
                    (void)Det_ReportError((uint16)UART_MODULE_ID,
                                            (uint8)0,
                                            (uint8)UART_ASYNCSEND_ID,
                                            (uint8)UART_E_INVALID_PARAMETER);
                    #endif /* (UART_DEV_ERROR_DETECT == STD_ON) */
                }
                else
                {
                    tempReturn = Uart_StartAsyncSend(u8CoreId, u8Channel, pBuffer, u32BufferSize);
                }
            }
        }
    }
    return tempReturn;
}

/* implements     Uart_GetStatus_Activity*/
Uart_StatusType Uart_GetStatus( uint8 u8Channel, uint32 *bytesTransfered, Uart_DataDirectionType transferType)
{
    Uart_StatusType tempReturn;
    uint8 u8CoreId;

    tempReturn = E_UART_RX_OVERRUN_ERROR;
   /* Get core ID of current processor */
    u8CoreId = (uint8) Uart_GetCoreID();

    if (UART_UNINIT == Uart_au8DriverStatus[u8CoreId])
    {
        #if (UART_DEV_ERROR_DETECT == STD_ON)
        (void)Det_ReportError((uint16)UART_MODULE_ID,
                                (uint8)0,
                                (uint8)UART_GETSTATUS_ID,
                                (uint8)UART_E_UNINIT);
        #endif /* (UART_DEV_ERROR_DETECT == STD_ON) */
    }
    else
    {
        if (u8Channel >= UART_CH_MAX_CONFIG)
        {
            #if (UART_DEV_ERROR_DETECT == STD_ON)
            (void)Det_ReportError((uint16)UART_MODULE_ID,
                                    (uint8)0,
                                    (uint8)UART_GETSTATUS_ID,
                                    (uint8)UART_E_INVALID_CHANNEL);
            #endif /* (UART_DEV_ERROR_DETECT == STD_ON) */
        }
        else
        {
            if (NULL_PTR == bytesTransfered)
            {
                #if (UART_DEV_ERROR_DETECT == STD_ON)
                (void)Det_ReportError((uint16)UART_MODULE_ID,
                                        (uint8)0,
                                        (uint8)UART_GETSTATUS_ID,
                                        (uint8)UART_E_INVALID_POINTER);
                #endif /* (UART_DEV_ERROR_DETECT == STD_ON) */
            }
            else
            {
                #if (UART_MULTICORE_SUPPORT == STD_ON)
                if (u8CoreId != Uart_apConfig[u8CoreId]->u32PartitionCoreId)
                {
                    #if (UART_DEV_ERROR_DETECT == STD_ON)
                    /* Invalid CoreId */
                    (void)Det_ReportError((uint16)UART_MODULE_ID,
                                    (uint8)0,
                                    (uint8)UART_GETSTATUS_ID,
                                    (uint8)UART_E_PARAM_CONFIG);
                    #endif /* (UART_DEV_ERROR_DETECT == STD_ON) */
                }
                else
                {
                    if (Uart_apConfig[u8CoreId]->Configs[u8Channel] == NULL_PTR)
                    {
                        #if (UART_DEV_ERROR_DETECT == STD_ON)
                        (void)Det_ReportError((uint16)UART_MODULE_ID,
                                                (uint8)0,
                                                (uint8)UART_GETSTATUS_ID,
                                                (uint8)UART_E_INVALID_CHANNEL);
                        #endif /* (UART_DEV_ERROR_DETECT     == STD_ON) */
                    }
                    else
                    {
                #endif /* (UART_MULTICORE_SUPPORT == STD_ON) */

                        if (UART_SEND == transferType)
                        {
                            tempReturn = Uart_Ipw_GetTransmitStatus(u8Channel, bytesTransfered);

                        }
                        else
                        {
                            tempReturn = Uart_Ipw_GetReceiveStatus(u8Channel, bytesTransfered);

                        }
                        (void)u8CoreId;

                #if (UART_MULTICORE_SUPPORT == STD_ON)
                    }
                }
                #endif /* (UART_MULTICORE_SUPPORT == STD_ON) */
            }
        }
    }
    return tempReturn;
}

#if (UART_VERSION_INFO_API == STD_ON)
/* implements     Uart_GetVersionInfo_Activity*/
void Uart_GetVersionInfo (Std_VersionInfoType * versioninfo)
{
    if( NULL_PTR == versioninfo )
    {
        #if( UART_DEV_ERROR_DETECT == STD_ON )
        (void)Det_ReportError((uint16)UART_MODULE_ID, \
                              (uint8)0, \
                              (uint8)UART_VERSIONINFO_ID, \
                              (uint8) UART_E_INVALID_POINTER \
                             );
        #endif /* UART_DEV_ERROR_DETECT == STD_ON */
    }
    else
    {

        versioninfo->vendorID = (uint16) UART_VENDOR_ID;
        versioninfo->moduleID = (uint8) UART_MODULE_ID;
        versioninfo->sw_major_version = (uint8) UART_SW_MAJOR_VERSION;
        versioninfo->sw_minor_version = (uint8) UART_SW_MINOR_VERSION;
        versioninfo->sw_patch_version = (uint8) UART_SW_PATCH_VERSION;
    }
}

#endif /* (UART_VERSION_INFO_API == STD_ON) */

#define UART_STOP_SEC_CODE
/* @violates @ref Uart_c_REF_1 This violation is not fixed since the inclusion of Uart_MemMap.h is as per AUTOSAR requirement*/
#include "Uart_MemMap.h"


#ifdef __cplusplus
}
/** @} */

#endif
