/*==================================================================================================
*   Project              : RTD AUTOSAR 4.4
*   Platform             : CORTEXM
*   Peripheral           : FLEXIO
*   Dependencies         : 
*
*   Autosar Version      : 4.4.0
*   Autosar Revision     : ASR_REL_4_4_REV_0000
*   Autosar Conf.Variant :
*   SW Version           : 0.9.0
*   Build Version        : S32K3_RTD_0_9_0_P02_D2107_ASR_REL_4_4_REV_0000_20210716
*
*   (c) Copyright 2020 - 2021 NXP Semiconductors
*   All Rights Reserved.
*
*   NXP Confidential. This software is owned or controlled by NXP and may only be
*   used strictly in accordance with the applicable license terms. By expressly
*   accepting such terms or by downloading, installing, activating and/or otherwise
*   using the software, you are agreeing that you have read, and that you agree to
*   comply with and are bound by, such license terms. If you do not agree to be
*   bound by the applicable license terms, then you may not retain, install,
*   activate or otherwise use the software.
==================================================================================================*/

/**
*   @file
*   @internal
*   @addtogroup UART
*   @{
*/

#ifdef __cplusplus
extern "C"
{
#endif

/*==================================================================================================
*                                        INCLUDE FILES
* 1) system and project includes
* 2) needed interfaces from external units
* 3) internal and external interfaces from this unit
==================================================================================================*/

#include "Uart_Ipw.h"

/*==================================================================================================
*                              SOURCE FILE VERSION INFORMATION
==================================================================================================*/
#define UART_IPW_VENDOR_ID_C                    43
#define UART_IPW_AR_RELEASE_MAJOR_VERSION_C     4
#define UART_IPW_AR_RELEASE_MINOR_VERSION_C     4
#define UART_IPW_AR_RELEASE_REVISION_VERSION_C  0
#define UART_IPW_SW_MAJOR_VERSION_C             0
#define UART_IPW_SW_MINOR_VERSION_C             9
#define UART_IPW_SW_PATCH_VERSION_C             0

/*==================================================================================================
*                                      FILE VERSION CHECKS
==================================================================================================*/
/*Checks against Uart_Ipw.h */
#if (UART_IPW_VENDOR_ID_C!= UART_IPW_VENDOR_ID)
    #error "Uart_Ipw.c and Uart_Ipw.h have different vendor ids"
#endif
#if ((UART_IPW_AR_RELEASE_MAJOR_VERSION_C   != UART_IPW_AR_RELEASE_MAJOR_VERSION) || \
     (UART_IPW_AR_RELEASE_MINOR_VERSION_C   != UART_IPW_AR_RELEASE_MINOR_VERSION) || \
     (UART_IPW_AR_RELEASE_REVISION_VERSION_C!= UART_IPW_AR_RELEASE_REVISION_VERSION))
    #error "AUTOSAR Version Numbers of Uart_Ipw.c and Uart_Ipw.h are different"
#endif
#if ((UART_IPW_SW_MAJOR_VERSION_C!= UART_IPW_SW_MAJOR_VERSION) || \
     (UART_IPW_SW_MINOR_VERSION_C!= UART_IPW_SW_MINOR_VERSION) || \
     (UART_IPW_SW_PATCH_VERSION_C!= UART_IPW_SW_PATCH_VERSION) \
    )
    #error "Software Version Numbers of Uart_Ipw.c and Uart_Ipw.h are different"
#endif
/*==================================================================================================
*                          LOCAL TYPEDEFS (STRUCTURES, UNIONS, ENUMS)
==================================================================================================*/

/*==================================================================================================
*                                       LOCAL MACROS
==================================================================================================*/

/*==================================================================================================
*                                       LOCAL CONSTANTS
==================================================================================================*/

/*==================================================================================================
*                                       LOCAL VARIABLES
==================================================================================================*/

/*==================================================================================================
*                                       GLOBAL CONSTANTS
==================================================================================================*/
#if (UART_CALLBACK_DECLARED == STD_ON)
#define UART_START_SEC_VAR_INIT_8
/* @violates @ref Uart_c_REF_1 This violation is not fixed since the inclusion of Uart_MemMap.h is as per AUTOSAR requirement*/
#include "Uart_MemMap.h"

static sint8 u8UartHwMapping[UART_NUMBER_OF_INSTANCES] = UART_HW_CHANNELS_MAPPING_FOR_INIT;

#define UART_STOP_SEC_VAR_INIT_8
/* @violates @ref Uart_c_REF_1 This violation is not fixed since the inclusion of Uart_MemMap.h is as per AUTOSAR requirement*/
#include "Uart_MemMap.h"
#endif /* UART_CALLBACK_DECLARED == STD_ON */
#define UART_START_SEC_VAR_NO_INIT_UNSPECIFIED
/* @violates @ref Uart_c_REF_1 This violation is not fixed since the inclusion of Uart_MemMap.h is as per AUTOSAR requirement*/
#include "Uart_MemMap.h"

const Uart_Ipw_HwConfigType *Uart_chnConfig[UART_CH_MAX_CONFIG];

#define UART_STOP_SEC_VAR_NO_INIT_UNSPECIFIED
/* @violates @ref Uart_c_REF_1 This violation is not fixed since the inclusion of Uart_MemMap.h is as per AUTOSAR requirement*/
#include "Uart_MemMap.h"

/*==================================================================================================
*                                       GLOBAL VARIABLES
==================================================================================================*/

/*==================================================================================================
*                                   LOCAL FUNCTION PROTOTYPES
==================================================================================================*/
#define UART_START_SEC_CODE
/* @violates @ref Uart_c_REF_1 This violation is not fixed since the inclusion of Uart_MemMap.h is as per AUTOSAR requirement*/
#include "Uart_MemMap.h"

/* Convert Lpuart Status type to Standard status type */
static Uart_StatusType Uart_Ipw_LpuartConvertStatus(Lpuart_Uart_Ip_StatusType eStatus);
#ifdef FLEXIO_UART_IP_IS_USING
    #if (FLEXIO_UART_IP_IS_USING == STD_ON)
        static Uart_StatusType Uart_Ipw_FlexioConvertStatus(Flexio_Uart_Ip_StatusType eStatus);
    #endif
#endif /* #ifdef FLEXIO_UART_IP_IS_USING */

/*==================================================================================================
*                                       LOCAL FUNCTIONS
==================================================================================================*/

/*==================================================================================================
*                                       GLOBAL FUNCTIONS
==================================================================================================*/
/**
* @internal
* @brief   Initialize an Uart channel.
* @details This function calls a LLD function initializes all hardware registers needed to
*          start the Uart functionality on the selected channel.
*
* @param   [in]    u8Channel  Uart channel to be initialized.
*          [in]    pConfig Configuration pointer containing hardware specific settings.
*
* @return            void.
*
*/
void Uart_Ipw_Init(uint8 u8Channel, const Uart_Ipw_HwConfigType* pConfig)
{
    uint32 u32HwUnit = pConfig->u32UartHwChannel;
    /* Save the mapping between hardware instance and logical channel */
    /* Save the channel settings */
    Uart_chnConfig[u8Channel] = pConfig;
    /* Call the LLD Initialization function */
    if (LPUART_IP == Uart_chnConfig[u8Channel]->eUartIpType)
    {
#if (UART_CALLBACK_DECLARED == STD_ON)
        u8UartHwMapping[u32HwUnit] = (sint8)u8Channel;
#endif /* UART_CALLBACK_DECLARED == STD_ON */
        Lpuart_Uart_Ip_Init(u32HwUnit, pConfig->pUserConfig->pLpuartUserConfig);
    }
#ifdef FLEXIO_UART_IP_IS_USING
#if (FLEXIO_UART_IP_IS_USING == STD_ON)
    else
    {
#if (UART_CALLBACK_DECLARED == STD_ON)
        u8UartHwMapping[UART_NUMBER_OF_INSTANCES - FLEXIO_UART_IP_NUMBER_OF_SHIFTER_AND_TIMER + u32HwUnit] = (sint8)u8Channel;
#endif /* UART_CALLBACK_DECLARED == STD_ON */
        Flexio_Uart_Ip_Init(u32HwUnit, pConfig->pUserConfig->pFlexioUartUserConfig);
    }
#endif /* (FLEXIO_UART_IP_IS_USING == STD_ON) */
#endif /* #ifdef FLEXIO_UART_IP_IS_USING */
}
/**
* @internal
* @brief   De-initialize an Uart channel.
* @details This function calls a LLD function de-initializes an Uart Hw instance and resets the current
            information regarding the current channel
*
* @param[in] u8Channel  Uart channel to be de-initialized.
*
* @return            void.
*
*/
void Uart_Ipw_Deinit(uint8 u8Channel)
{
    uint32 u32HwUnit = Uart_chnConfig[u8Channel]->u32UartHwChannel;


    if (LPUART_IP == Uart_chnConfig[u8Channel]->eUartIpType)
    {
#if (UART_CALLBACK_DECLARED == STD_ON)
        /* Remove the current hw to logic mapping */
        u8UartHwMapping[u32HwUnit] = -1;
#endif /* UART_CALLBACK_DECLARED == STD_ON */
        Lpuart_Uart_Ip_Deinit(u32HwUnit);
    }
#ifdef FLEXIO_UART_IP_IS_USING
#if (FLEXIO_UART_IP_IS_USING == STD_ON)
    else
    {
#if (UART_CALLBACK_DECLARED == STD_ON)
        /* Remove the current hw to logic mapping */
        u8UartHwMapping[UART_NUMBER_OF_INSTANCES - FLEXIO_UART_IP_NUMBER_OF_SHIFTER_AND_TIMER + u32HwUnit] = -1;
#endif /* UART_CALLBACK_DECLARED == STD_ON */
        Flexio_Uart_Ip_Deinit(u32HwUnit);
    }
#endif /* (FLEXIO_UART_IP_IS_USING == STD_ON) */
#endif /* #ifdef FLEXIO_UART_IP_IS_USING */
    /* Remove refference to the current channel configuration */
    Uart_chnConfig[u8Channel] = NULL_PTR;
}
/**
* @internal
* @brief  Set a baudrate value for the current channel.
* @details This function calls a LLD function which sets a baudrate value for a given channel,
*
* @param[in] u8Channel          Uart channel where the baudrate shall be configured
* @param[in] eBaudRateValue     The baudrate value to be configured.
* @param[in] u32ClockFrequency  The clock frequency set for the Uart module.
*
* @return   E_OK:     Baud rate successfully configured
*           E_NOT_OK: Baud rate wasn't successfully configured
*
*/
Std_ReturnType Uart_Ipw_SetBaudrate(uint8 u8Channel, Uart_BaudrateType eBaudRateValue, uint32 u32ClockFrequency)
{
    Std_ReturnType u8TempStatus = (uint8)E_NOT_OK;
    uint32 u32HwUnit = Uart_chnConfig[u8Channel]->u32UartHwChannel;

    if (LPUART_IP == Uart_chnConfig[u8Channel]->eUartIpType)
    {
        if (LPUART_UART_IP_STATUS_SUCCESS == Lpuart_Uart_Ip_SetBaudRate(u32HwUnit, (Lpuart_Uart_Ip_BaudrateType)eBaudRateValue, u32ClockFrequency))
        {
            u8TempStatus = (uint8)E_OK;
        }
    }
#ifdef FLEXIO_UART_IP_IS_USING
#if (FLEXIO_UART_IP_IS_USING == STD_ON)
    else
    {
        if (FLEXIO_UART_IP_STATUS_SUCCESS == Flexio_Uart_Ip_SetBaudRate(u32HwUnit, (Flexio_Uart_Ip_BaudrateType)eBaudRateValue, u32ClockFrequency))
        {
            u8TempStatus = (uint8)E_OK;
        }

    }
#endif /* (FLEXIO_UART_IP_IS_USING == STD_ON) */
#endif /* #ifdef FLEXIO_UART_IP_IS_USING */

    return u8TempStatus;
}
/**
* @internal
* @brief  Retrieves the baudrate value configured for the current channel.
* @details This function calls a LLD function returns the baudrate value configured for the given channel
* via a pointer parameter provided by user.
*
* @param[in] u8Channel           Uart channel where the baudrate shall be configured
* @param[out] pBaudrateValue     The pointer which will contain the baudrate value.
*
* @return   void
*
*/
void Uart_Ipw_GetBaudrate(uint8 u8Channel, uint32 *pBaudrateValue)
{
    uint32 u32HwUnit = Uart_chnConfig[u8Channel]->u32UartHwChannel;

    if (LPUART_IP == Uart_chnConfig[u8Channel]->eUartIpType)
    {
        Lpuart_Uart_Ip_GetBaudRate(u32HwUnit, pBaudrateValue);
    }
#ifdef FLEXIO_UART_IP_IS_USING
#if (FLEXIO_UART_IP_IS_USING == STD_ON)
    else
    {
        Flexio_Uart_Ip_GetBaudRate(u32HwUnit, pBaudrateValue);
    }
#endif /* (FLEXIO_UART_IP_IS_USING == STD_ON) */
#endif /* #ifdef FLEXIO_UART_IP_IS_USING */
}
/**
* @internal
* @brief  Aborts an on-going transmission.
* @details This function calls a LLD function which ends a transfer earlier.
*
* @param[in] u8Channel          Uart channel where the transmission shall be ended.
*
* @return   E_OK:     The transfer ended successfully
*           E_NOT_OK: The transfer didn't end successfully
*
*/
Std_ReturnType Uart_Ipw_AbortSendingData(uint8 u8Channel)
{
    Std_ReturnType u8TempStatus = (uint8)E_NOT_OK;
    uint32 u32HwUnit = Uart_chnConfig[u8Channel]->u32UartHwChannel;

    if (LPUART_IP == Uart_chnConfig[u8Channel]->eUartIpType)
    {
        if (LPUART_UART_IP_STATUS_SUCCESS == Lpuart_Uart_Ip_AbortSendingData(u32HwUnit))
        {
            u8TempStatus = (uint8)E_OK;
        }
    }
#ifdef FLEXIO_UART_IP_IS_USING
#if (FLEXIO_UART_IP_IS_USING == STD_ON)
    else
    {
        if (FLEXIO_UART_IP_STATUS_SUCCESS == Flexio_Uart_Ip_AbortTransferData(u32HwUnit))
        {
            u8TempStatus = (uint8)E_OK;
        }
    }
#endif /* (FLEXIO_UART_IP_IS_USING == STD_ON) */
#endif /* #ifdef FLEXIO_UART_IP_IS_USING */

    return u8TempStatus;
}
/**
* @internal
* @brief  Aborts an on-going reception.
* @details This function calls a LLD function which ends a reception earlier.
*
* @param[in] u8Channel          Uart channel where the reception shall be ended.
*
* @return   E_OK:     The reception ended successfully
*           E_NOT_OK: The reception didn't end successfully
*
*/
Std_ReturnType Uart_Ipw_AbortReceivingData(uint8 u8Channel)
{
    Std_ReturnType u8TempStatus = (uint8)E_NOT_OK;
    uint32 u32HwUnit = Uart_chnConfig[u8Channel]->u32UartHwChannel;

    if (LPUART_IP == Uart_chnConfig[u8Channel]->eUartIpType)
    {
        if (LPUART_UART_IP_STATUS_SUCCESS == Lpuart_Uart_Ip_AbortReceivingData(u32HwUnit))
        {
            u8TempStatus = (uint8)E_OK;
        }
    }
#ifdef FLEXIO_UART_IP_IS_USING
#if (FLEXIO_UART_IP_IS_USING == STD_ON)
    else
    {
        if (FLEXIO_UART_IP_STATUS_SUCCESS == Flexio_Uart_Ip_AbortTransferData(u32HwUnit))
        {
            u8TempStatus = (uint8)E_OK;
        }
    }
#endif /* (FLEXIO_UART_IP_IS_USING == STD_ON) */
#endif /* #ifdef FLEXIO_UART_IP_IS_USING */

    return u8TempStatus;
}
/**
* @internal
* @brief  Sets a tx buffer.
* @details This function calls a LLD function which sets a buffer for continuos transmission.
*
* @param[in] u8Channel  Uart channel where to set the buffer.
*
* @return  void
*/
void Uart_Ipw_SetTxBuffer(uint8 u8Channel, const uint8 *pBuffer, uint32 u32BufferSize)
{
    uint32 u32HwUnit = Uart_chnConfig[u8Channel]->u32UartHwChannel;

    if (LPUART_IP == Uart_chnConfig[u8Channel]->eUartIpType)
    {
        (void)Lpuart_Uart_Ip_SetTxBuffer(u32HwUnit, pBuffer, u32BufferSize);
    }
#ifdef FLEXIO_UART_IP_IS_USING
#if (FLEXIO_UART_IP_IS_USING == STD_ON)
    else
    {
        (void)Flexio_Uart_Ip_SetTxBuffer(u32HwUnit, pBuffer, u32BufferSize);
    }
#endif /* (FLEXIO_UART_IP_IS_USING == STD_ON) */
#endif /* #ifdef FLEXIO_UART_IP_IS_USING */
}
/**
* @brief  Sets a rx buffer.
* @details This function calls a LLD function which sets a buffer for continuos reception.
*
* @param[in] u8Channel  Uart channel where to set the buffer.
*
* @return  void
*/
void Uart_Ipw_SetRxBuffer(uint8 u8Channel, uint8 *pBuffer, uint32 u32BufferSize)
{
    uint32 u32HwUnit = Uart_chnConfig[u8Channel]->u32UartHwChannel;
    if (LPUART_IP == Uart_chnConfig[u8Channel]->eUartIpType)
    {
        (void)Lpuart_Uart_Ip_SetRxBuffer(u32HwUnit, pBuffer, u32BufferSize);
    }
#ifdef FLEXIO_UART_IP_IS_USING
#if (FLEXIO_UART_IP_IS_USING == STD_ON)
    else
    {
        (void)Flexio_Uart_Ip_SetRxBuffer(u32HwUnit, pBuffer, u32BufferSize);
    }
#endif /* (FLEXIO_UART_IP_IS_USING == STD_ON) */
#endif /* #ifdef FLEXIO_UART_IP_IS_USING */
}
/**
* @internal
* @brief  Sends a buffer synchronously.
* @details This function calls a LLD function which sends u32BufferSize bytes using polling method.
*
* @param[in] u8Channel      Uart channel used for transmission.
* @param[in] pBuffer        Buffer to send.
* @param[in] u32BufferSize  The number of bytes to send.
*
*
* @return   E_OK:                   Tranmission ended successfully
*           E_UART_TIMEOUT:         Tranmission has timeout
*           E_NOT_OK                Tranmission didn't end successfully
*/

Std_ReturnType Uart_Ipw_SyncSend(uint8 u8Channel, const uint8 *pBuffer, uint32 u32BufferSize)
{
    Std_ReturnType u8TempStatus = (uint8)E_NOT_OK;
    uint32 u32HwUnit = Uart_chnConfig[u8Channel]->u32UartHwChannel;

    /* LPUART is used */
    if (LPUART_IP == Uart_chnConfig[u8Channel]->eUartIpType)
    {
        Lpuart_Uart_Ip_StatusType u32LpuartIpStatus;
        u32LpuartIpStatus = Lpuart_Uart_Ip_SyncSend(u32HwUnit,
                                                    pBuffer,
                                                    u32BufferSize);
        if (LPUART_UART_IP_STATUS_SUCCESS == u32LpuartIpStatus)
        {
            u8TempStatus = (uint8)E_OK;
        }
        else if (LPUART_UART_IP_STATUS_TIMEOUT == u32LpuartIpStatus)
        {
            u8TempStatus = (uint8)E_UART_TIMEOUT;
        }
        else
        {
            u8TempStatus = (uint8)E_NOT_OK;
        }
    }
#ifdef FLEXIO_UART_IP_IS_USING
#if (FLEXIO_UART_IP_IS_USING == STD_ON)
    else /* FLEXIO_UART is used */
    {
        Flexio_Uart_Ip_StatusType u32FlexioIpStatus;
        u32FlexioIpStatus = Flexio_Uart_Ip_SyncSend(u32HwUnit,
                                                    pBuffer,
                                                    u32BufferSize);
        if (FLEXIO_UART_IP_STATUS_SUCCESS == u32FlexioIpStatus)
        {
            u8TempStatus = (uint8)E_OK;
        }
        else if (FLEXIO_UART_IP_STATUS_TIMEOUT == u32FlexioIpStatus)
        {
            u8TempStatus = (uint8)E_UART_TIMEOUT;
        }
        else
        {
            u8TempStatus = (uint8)E_NOT_OK;
        }
    }
#endif /* (FLEXIO_UART_IP_IS_USING == STD_ON) */
#endif /* #ifdef FLEXIO_UART_IP_IS_USING */

    return u8TempStatus;
}
/**
* @internal
* @brief  Receives a buffer synchronously.
* @details This function calls a LLD function which receives u32BufferSize bytes using polling method.
*
* @param[in] u8Channel      Uart channel used for reception.
* @param[in] pBuffer        Buffer to receive.
* @param[in] u32BufferSize  The number of bytes to receive.
*
*
* @return   E_OK:               Reception ended successfully
*           E_NOT_OK:           Reception didn't end successfully
*           E_UART_TIMEOUT:     Reception has timeout
*/
Std_ReturnType Uart_Ipw_SyncReceive(uint8 u8Channel, uint8 *pBuffer, uint32 u32BufferSize)
{
    Std_ReturnType u8TempStatus = (uint8)E_NOT_OK;
    uint32 u32HwUnit = Uart_chnConfig[u8Channel]->u32UartHwChannel;
    /* LPUART is used */
    if (LPUART_IP == Uart_chnConfig[u8Channel]->eUartIpType)
    {
        Lpuart_Uart_Ip_StatusType u32LpuartIpStatus;
        u32LpuartIpStatus = Lpuart_Uart_Ip_SyncReceive(u32HwUnit,
                                                       pBuffer,
                                                       u32BufferSize);
        if (LPUART_UART_IP_STATUS_SUCCESS == u32LpuartIpStatus)
        {
            u8TempStatus = (uint8)E_OK;
        }
        else if (LPUART_UART_IP_STATUS_TIMEOUT == u32LpuartIpStatus)
        {
            u8TempStatus = (uint8)E_UART_TIMEOUT;
        }
        else
        {
            u8TempStatus = (uint8)E_NOT_OK;
        }
    }
#ifdef FLEXIO_UART_IP_IS_USING
#if (FLEXIO_UART_IP_IS_USING == STD_ON)
    else /* FLEXIO_UART is used */
    {
        Flexio_Uart_Ip_StatusType u32FlexioIpStatus;
        u32FlexioIpStatus = Flexio_Uart_Ip_SyncReceive(u32HwUnit,
                                                       pBuffer,
                                                       u32BufferSize);
        if (FLEXIO_UART_IP_STATUS_SUCCESS == u32FlexioIpStatus)
        {
            u8TempStatus = (uint8)E_OK;
        }
        else if (FLEXIO_UART_IP_STATUS_TIMEOUT == u32FlexioIpStatus)
        {
            u8TempStatus = (uint8)E_UART_TIMEOUT;
        }
        else
        {
            u8TempStatus = (uint8)E_NOT_OK;
        }
    }
#endif /* (FLEXIO_UART_IP_IS_USING == STD_ON) */
#endif /* #ifdef FLEXIO_UART_IP_IS_USING */

    return u8TempStatus;
}
/**
* @internal
* @brief  Receives a buffer asynchronously.
* @details This function calls a LLD function which receives u32BufferSize bytes using an asynchornous method.
*
* @param[in] u8Channel      Uart channel used for reception.
* @param[in] pBuffer        Buffer to receive.
* @param[in] u32BufferSize  The number of bytes to receive.
*
*
* @return   E_OK:               Reception started successfully
            E_NOT_OK:           Reception didn't start successfully
*/
Std_ReturnType Uart_Ipw_AsyncReceive(uint8 u8Channel, uint8 *pBuffer, uint32 u32BufferSize)
{
    Std_ReturnType u8TempStatus = (uint8)E_NOT_OK;
    uint32 u32HwUnit = Uart_chnConfig[u8Channel]->u32UartHwChannel;
    /* LPUART is used */
    if (LPUART_IP == Uart_chnConfig[u8Channel]->eUartIpType)
    {
        if (LPUART_UART_IP_STATUS_SUCCESS == Lpuart_Uart_Ip_AsyncReceive(u32HwUnit,
                                                                         pBuffer,
                                                                         u32BufferSize))
        {
            u8TempStatus = (uint8)E_OK;
        }
    }
#ifdef FLEXIO_UART_IP_IS_USING
#if (FLEXIO_UART_IP_IS_USING == STD_ON)
    else /* FLEXIO_UART is used */
    {
        if (FLEXIO_UART_IP_STATUS_SUCCESS == Flexio_Uart_Ip_AsyncReceive(u32HwUnit,
                                                                         pBuffer,
                                                                         u32BufferSize))
        {
            u8TempStatus = (uint8)E_OK;
        }
    }
#endif /* (FLEXIO_UART_IP_IS_USING == STD_ON) */
#endif /* #ifdef FLEXIO_UART_IP_IS_USING */

    return u8TempStatus;
}
/**
* @internal
* @brief  Sends a buffer asynchronously.
* @details This function calls a LLD function which sends u32BufferSize bytes using an asynchornous method.
*
* @param[in] u8Channel      Uart channel used for transmission.
* @param[in] pBuffer        Buffer to send.
* @param[in] u32BufferSize  The number of bytes to send.
*
*
* @return   E_OK:               Transmission started successfully
            E_NOT_OK:           Transmission didn't start successfully
*/
Std_ReturnType Uart_Ipw_AsyncSend(uint8 u8Channel, const uint8 *pBuffer, uint32 u32BufferSize)
{
    Std_ReturnType u8TempStatus = (uint8)E_NOT_OK;
    uint32 u32HwUnit = Uart_chnConfig[u8Channel]->u32UartHwChannel;
    /* LPUART is used */
    if (LPUART_IP == Uart_chnConfig[u8Channel]->eUartIpType)
    {
        if (LPUART_UART_IP_STATUS_SUCCESS == Lpuart_Uart_Ip_AsyncSend(u32HwUnit,
                                                                      pBuffer,
                                                                      u32BufferSize))
        {
            u8TempStatus = (uint8)E_OK;
        }
    }
#ifdef FLEXIO_UART_IP_IS_USING
#if (FLEXIO_UART_IP_IS_USING == STD_ON)
    else /* FLEXIO_UART is used */
    {
        if (FLEXIO_UART_IP_STATUS_SUCCESS == Flexio_Uart_Ip_AsyncSend(u32HwUnit,
                                                                      pBuffer,
                                                                      u32BufferSize))
        {
            u8TempStatus = (uint8)E_OK;
        }
    }
#endif /* (FLEXIO_UART_IP_IS_USING == STD_ON) */
#endif /* #ifdef FLEXIO_UART_IP_IS_USING */

    return u8TempStatus;
}


/**
* @internal
* @brief  Retrieves the status of the previous transfer.
* @details This function calls a LLD function which retrieves the status of the previous transfer
*          and the number of the remaining bytes to send.
*
* @param[in] u8Channel   Uart channel.
* @param[out] pBytesRemaining   The pointer where to write the number of remaining bytes
*
*
* @return   UART_NO_ERR0R:                  The previous transfer ended successfully
            UART_OPERATION_ONGOING      The transfer is still on going
            UART_ABORTED            The previous transfer was aborted
            E_UART_TIMEOUT          The previous transfer was timeout

*/
Uart_StatusType Uart_Ipw_GetTransmitStatus(uint8 u8Channel, uint32 *pBytesRemaining)
{
    Uart_StatusType u8TempStatus = E_UART_TIMEOUT;
    uint32 u32HwUnit = Uart_chnConfig[u8Channel]->u32UartHwChannel;

    if (LPUART_IP == Uart_chnConfig[u8Channel]->eUartIpType)
    {
        Lpuart_Uart_Ip_StatusType u32LpuartIpStatus;
        u32LpuartIpStatus = Lpuart_Uart_Ip_GetTransmitStatus(u32HwUnit, pBytesRemaining);
        u8TempStatus = Uart_Ipw_LpuartConvertStatus(u32LpuartIpStatus);
    }
#ifdef FLEXIO_UART_IP_IS_USING
#if (FLEXIO_UART_IP_IS_USING == STD_ON)
    else /*FLEXIO is used*/
    {
        Flexio_Uart_Ip_StatusType u32FlexioIpStatus;
        u32FlexioIpStatus = Flexio_Uart_Ip_GetStatus(u32HwUnit, pBytesRemaining);
        u8TempStatus = Uart_Ipw_FlexioConvertStatus(u32FlexioIpStatus);
    }
#endif /* (FLEXIO_UART_IP_IS_USING == STD_ON) */
#endif /* #ifdef FLEXIO_UART_IP_IS_USING */

    return u8TempStatus;
}

/**
* @internal
* @brief  Retrieves the status of the previous reception.
* @details This function calls a LLD function which retrieves the status of the previous reception
*          and the number of the remaining bytes to receive.
*
* @param[in] u8Channel   Uart channel.
* @param[out] pBytesRemaining   The pointer where to write the number of remaining bytes
*
*
* @return   UART_NO_ERR0R:                  The previous transfer ended successfully
            UART_OPERATION_ONGOING      The reception is still on going
            UART_ABORTED            The previous reception was aborted
            E_UART_FRAMING_ERROR     Uart framing error
            E_UART_RX_OVERRUN_ERROR  Uart overrun error
            E_UART_PARITY_ERROR       Uart overrun error
            E_UART_TIMEOUT          Uart operation has timeout

*/
Uart_StatusType Uart_Ipw_GetReceiveStatus(uint8 u8Channel, uint32 *pBytesRemaining)
{
    Uart_StatusType u8TempStatus = E_UART_TIMEOUT;
    uint32 u32HwUnit = Uart_chnConfig[u8Channel]->u32UartHwChannel;
    if (LPUART_IP == Uart_chnConfig[u8Channel]->eUartIpType)
    {
        Lpuart_Uart_Ip_StatusType u32LpuartIpStatus;
        u32LpuartIpStatus = Lpuart_Uart_Ip_GetReceiveStatus(u32HwUnit, pBytesRemaining);
        u8TempStatus = Uart_Ipw_LpuartConvertStatus(u32LpuartIpStatus);
    }
#ifdef FLEXIO_UART_IP_IS_USING
#if (FLEXIO_UART_IP_IS_USING == STD_ON)
    else /*FLEXIO is used*/
    {
        Flexio_Uart_Ip_StatusType u32FlexioIpStatus;
        u32FlexioIpStatus = Flexio_Uart_Ip_GetStatus(u32HwUnit, pBytesRemaining);
        u8TempStatus = Uart_Ipw_FlexioConvertStatus(u32FlexioIpStatus);
    }
#endif /* (FLEXIO_UART_IP_IS_USING == STD_ON) */
#endif /* #ifdef FLEXIO_UART_IP_IS_USING */

    return u8TempStatus;

}

/**
* @internal
* @brief  Convert Uart IP layer return status to standard return status.
* @details This function receives Uart IP layer return status and convert to standard return status.
* @param[in] eStatus        Uart IP status
*
*
* @return   Standard status
* This is not a public API as it is called from other driver functions.
*/
static Uart_StatusType Uart_Ipw_LpuartConvertStatus(Lpuart_Uart_Ip_StatusType eStatus)
{
    Uart_StatusType u8TempStatus = E_UART_TIMEOUT;

    switch (eStatus)
    {
        case LPUART_UART_IP_STATUS_SUCCESS:
            u8TempStatus = UART_NO_ERROR;
            break;

        case LPUART_UART_IP_STATUS_BUSY:
            u8TempStatus = UART_OPERATION_ONGOING;
            break;

        case LPUART_UART_IP_STATUS_ABORTED:
            u8TempStatus =  UART_ABORTED;
            break;

        case LPUART_UART_IP_STATUS_TIMEOUT:
            u8TempStatus =  E_UART_TIMEOUT;
            break;

        case LPUART_UART_IP_STATUS_FRAMING_ERROR:
            u8TempStatus =  E_UART_FRAMING_ERROR;
            break;

        case LPUART_UART_IP_STATUS_RX_OVERRUN:
            u8TempStatus =  E_UART_RX_OVERRUN_ERROR;
            break;

        case LPUART_UART_IP_STATUS_NOISE_ERROR:
            u8TempStatus =  E_UART_NOISE_ERROR;
            break;

        case LPUART_UART_IP_STATUS_PARITY_ERROR:
            u8TempStatus =  E_UART_PARITY_ERROR;
            break;

        default:
            u8TempStatus =  E_UART_DMA_ERROR;
            break;
    }

    return u8TempStatus;
}
#ifdef FLEXIO_UART_IP_IS_USING
#if (FLEXIO_UART_IP_IS_USING == STD_ON)
/**
* @internal
* @brief  Convert Uart IP layer return status to standard return status.
* @details This function receives Uart IP layer return status and convert to standard return status.
* @param[in] eStatus        Uart IP status
*
*
* @return   Standard status
* This is not a public API as it is called from other driver functions.
*/
static Uart_StatusType Uart_Ipw_FlexioConvertStatus(Flexio_Uart_Ip_StatusType eStatus)
{
    Uart_StatusType u8TempStatus = E_UART_TIMEOUT;

    switch (eStatus)
    {
        case FLEXIO_UART_IP_STATUS_SUCCESS:
            u8TempStatus = UART_NO_ERROR;
            break;

        case FLEXIO_UART_IP_STATUS_BUSY:
            u8TempStatus = UART_OPERATION_ONGOING;
            break;

        case FLEXIO_UART_IP_STATUS_ABORTED:
            u8TempStatus =  UART_ABORTED;
            break;

        case FLEXIO_UART_IP_STATUS_TIMEOUT:
            u8TempStatus =  E_UART_TIMEOUT;
            break;

        default:
            u8TempStatus =  E_UART_DMA_ERROR;
            break;
    }

    return u8TempStatus;
}
#endif /* (FLEXIO_UART_IP_IS_USING == STD_ON) */
#endif /* #ifdef FLEXIO_UART_IP_IS_USING */

#if (UART_CALLBACK_DECLARED == STD_ON)
/**
* @internal
* @internal
* @brief   Low level driver Rx callback for Lpuart
* @details This function is called by the low level driver in the ISR.
*
*
* This is not a public API as it is called from other driver functions.
*/
void Uart_Ipw_LpuartRxCallback(uint32 hwInstance, void *driverState, Lpuart_Uart_Ip_EventType event, void *userData)
{
    (void)driverState;
    (void)userData;
    uint8 u8Channel = u8UartHwMapping[hwInstance];

    if (Uart_chnConfig[u8Channel]->pfRxCallback != NULL_PTR)
    {
        Uart_chnConfig[u8Channel]->pfRxCallback(u8Channel, (Uart_EventType)event);
    }
}

/**
* @internal
* @brief   Low level driver Tx callback for Lpuart
* @details This function is called by the low level driver in the ISR.
*
*
* This is not a public API as it is called from other driver functions.
*/
void Uart_Ipw_LpuartTxCallback(uint32 hwInstance, void *driverState, Lpuart_Uart_Ip_EventType event, void *userData)
{
    (void)driverState;
    (void)userData;
    uint8 u8Channel = u8UartHwMapping[hwInstance];

    if (Uart_chnConfig[u8Channel]->pfTxCallback != NULL_PTR)
    {
        Uart_chnConfig[u8Channel]->pfTxCallback(u8Channel, (Uart_EventType)event);
    }
}

/**
* @internal
* @brief   Low level driver Errpr callback for Lpuart
* @details This function is called by the low level driver in the ISR.
*
*
* This is not a public API as it is called from other driver functions.
*/
void Uart_Ipw_LpuartErrorCallback(uint32 hwInstance, void *driverState, Lpuart_Uart_Ip_EventType event, void *userData)
{
    (void)driverState;
    (void)userData;
    uint8 u8Channel = u8UartHwMapping[hwInstance];

    if (Uart_chnConfig[u8Channel]->pfErrorCallback != NULL_PTR)
    {
        Uart_chnConfig[u8Channel]->pfErrorCallback(u8Channel, (Uart_EventType)event);
    }
}
#ifdef FLEXIO_UART_IP_IS_USING
#if (FLEXIO_UART_IP_IS_USING == STD_ON)
/**
* @internal
* @brief   Low level driver Errpr callback for Flexio
* @details This function is called by the low level driver in the ISR.
*
*
* This is not a public API as it is called from other driver functions.
*/
void Uart_Ipw_FlexioErrorCallback(uint32 hwChannel, void *driverState, Flexio_Uart_Ip_EventType event, void *userData)
{
    (void)driverState;
    (void)userData;
    uint8 u8Channel = u8UartHwMapping[UART_NUMBER_OF_INSTANCES - FLEXIO_UART_IP_NUMBER_OF_SHIFTER_AND_TIMER + hwChannel];

    if (Uart_chnConfig[u8Channel]->pfErrorCallback != NULL_PTR)
    {
        Uart_chnConfig[u8Channel]->pfErrorCallback(u8Channel, (Uart_EventType)event);
    }
}

/**
* @internal
* @brief   Low level driver Tx or Rx callback for Flexio
* @details This function is called by the low level driver in the ISR.
*
*
* This is not a public API as it is called from other driver functions.
*/
void Uart_Ipw_FlexioTransferCallback(uint32 hwChannel, void *driverState, Flexio_Uart_Ip_EventType event, void *userData)
{
    (void)driverState;
    (void)userData;
    uint8 u8Channel = u8UartHwMapping[UART_NUMBER_OF_INSTANCES - FLEXIO_UART_IP_NUMBER_OF_SHIFTER_AND_TIMER + hwChannel];

    if (Uart_chnConfig[u8Channel]->pfTxCallback != NULL_PTR)
    {
        Uart_chnConfig[u8Channel]->pfTxCallback(u8Channel, (Uart_EventType)event);
    }

    if (Uart_chnConfig[u8Channel]->pfRxCallback != NULL_PTR)
    {
        Uart_chnConfig[u8Channel]->pfRxCallback(u8Channel, (Uart_EventType)event);
    }
}
#endif /* (FLEXIO_UART_IP_IS_USING == STD_ON) */
#endif /* #ifdef FLEXIO_UART_IP_IS_USING */
#endif /* UART_CALLBACK_DECLARED == STD_ON */
#define UART_STOP_SEC_CODE
/* @violates @ref Uart_c_REF_1 This violation is not fixed since the inclusion of Uart_MemMap.h is as per AUTOSAR requirement*/
#include "Uart_MemMap.h"


#ifdef __cplusplus
}

/** @} */

#endif
