/*==================================================================================================
* Project : RTD AUTOSAR 4.4
* Platform : CORTEXM
* Peripheral : Stm_Pit_Rtc_Emios
* Dependencies : none
*
* Autosar Version : 4.4.0
* Autosar Revision : ASR_REL_4_4_REV_0000
* Autosar Conf.Variant :
* SW Version : 0.9.0
* Build Version : S32K3_RTD_0_9_0_P02_D2107_ASR_REL_4_4_REV_0000_20210716
*
* (c) Copyright 2020 - 2021 NXP Semiconductors
* All Rights Reserved.
*
* NXP Confidential. This software is owned or controlled by NXP and may only be
* used strictly in accordance with the applicable license terms. By expressly
* accepting such terms or by downloading, installing, activating and/or otherwise
* using the software, you are agreeing that you have read, and that you agree to
* comply with and are bound by, such license terms. If you do not agree to be
* bound by the applicable license terms, then you may not retain, install,
* activate or otherwise use the software.
==================================================================================================*/
/**
 *   @file          Pit_Ip_PBcfg.c
 *
 *   @addtogroup    pit_ip Pit IPL
 *
 *   @{
 */



/*==================================================================================================*/
#ifdef __cplusplus
extern "C"{
#endif

 /*==================================================================================================
 *                                         INCLUDE FILES
 * 1) system and project includes
 * 2) needed interfaces from external units
 * 3) internal and external interfaces from this unit
 *================================================================================================*/
#include "Pit_Ip_VS_0_PBcfg.h"
#include "Gpt_Irq.h"

/*==================================================================================================
 *                              SOURCE FILE VERSION INFORMATION
 *================================================================================================*/
#define PIT_IP_VENDOR_ID_VS_0_PBCFG_C                    43
#define PIT_IP_AR_RELEASE_MAJOR_VERSION_VS_0_PBCFG_C     4
#define PIT_IP_AR_RELEASE_MINOR_VERSION_VS_0_PBCFG_C     4
#define PIT_IP_AR_RELEASE_REVISION_VERSION_VS_0_PBCFG_C  0
#define PIT_IP_SW_MAJOR_VERSION_VS_0_PBCFG_C             0
#define PIT_IP_SW_MINOR_VERSION_VS_0_PBCFG_C             9
#define PIT_IP_SW_PATCH_VERSION_VS_0_PBCFG_C             0
/*==================================================================================================
 *                                      FILE VERSION CHECKS
 *================================================================================================*/
#if (PIT_IP_VENDOR_ID_VS_0_PBCFG_C != PIT_IP_VENDOR_ID_VS_0_PBCFG_H)
    #error "Pit_Ip_PBcfg.c and Pit_Ip_Types.h have different vendor ids"
#endif
/* Check if this header file and header file are of the same Autosar version */
#if ((PIT_IP_AR_RELEASE_MAJOR_VERSION_VS_0_PBCFG_C != PIT_IP_AR_RELEASE_MAJOR_VERSION_VS_0_PBCFG_H) || \
     (PIT_IP_AR_RELEASE_MINOR_VERSION_VS_0_PBCFG_C != PIT_IP_AR_RELEASE_MINOR_VERSION_VS_0_PBCFG_H) || \
     (PIT_IP_AR_RELEASE_REVISION_VERSION_VS_0_PBCFG_C != PIT_IP_AR_RELEASE_REVISION_VERSION_VS_0_PBCFG_H) \
    )
    #error "AutoSar Version Numbers of Pit_Ip_PBcfg.c and Pit_Ip_Types.h are different"
#endif
/* Check if this header file and header file are of the same Software version */
#if ((PIT_IP_SW_MAJOR_VERSION_VS_0_PBCFG_C != PIT_IP_SW_MAJOR_VERSION_VS_0_PBCFG_H) || \
     (PIT_IP_SW_MINOR_VERSION_VS_0_PBCFG_C != PIT_IP_SW_MINOR_VERSION_VS_0_PBCFG_H) || \
     (PIT_IP_SW_PATCH_VERSION_VS_0_PBCFG_C != PIT_IP_SW_PATCH_VERSION_VS_0_PBCFG_H) \
    )
    #error "Software Version Numbers of Pit_Ip_PBcfg.c and Pit_Ip_Types.h are different"
#endif
#if (PIT_IP_VENDOR_ID_VS_0_PBCFG_C != GPT_IRQ_VENDOR_ID)
    #error "Pit_Ip_PBcfg.c and Gpt_Irq.h have different vendor ids"
#endif
/* Check if this header file and header file are of the same Autosar version */
#if ((PIT_IP_AR_RELEASE_MAJOR_VERSION_VS_0_PBCFG_C != GPT_IRQ_AR_RELEASE_MAJOR_VERSION) || \
     (PIT_IP_AR_RELEASE_MINOR_VERSION_VS_0_PBCFG_C != GPT_IRQ_AR_RELEASE_MINOR_VERSION) || \
     (PIT_IP_AR_RELEASE_REVISION_VERSION_VS_0_PBCFG_C != GPT_IRQ_AR_RELEASE_REVISION_VERSION) \
    )
    #error "AutoSar Version Numbers of Pit_Ip_PBcfg.c and Gpt_Irq.h are different"
#endif
/* Check if this header file and header file are of the same Software version */
#if ((PIT_IP_SW_MAJOR_VERSION_VS_0_PBCFG_C != GPT_IRQ_SW_MAJOR_VERSION) || \
     (PIT_IP_SW_MINOR_VERSION_VS_0_PBCFG_C != GPT_IRQ_SW_MINOR_VERSION) || \
     (PIT_IP_SW_PATCH_VERSION_VS_0_PBCFG_C != GPT_IRQ_SW_PATCH_VERSION) \
    )
    #error "Software Version Numbers of Pit_Ip_PBcfg.c and Gpt_Irq.h are different"
#endif
/*================================================================================================*/

/*==================================================================================================
 *                                       GLOBAL VARIABLES
 *================================================================================================*/
#define GPT_START_SEC_CODE
#include "Gpt_MemMap.h"
extern void Gpt_ErrorReport(void);
#define GPT_STOP_SEC_CODE
#include "Gpt_MemMap.h"

/*==================================================================================================
 *                                       GLOBAL CONSTANTS
 *================================================================================================*/

/*==================================================================================================
 *                                       GLOBAL FUNCTIONS
 *================================================================================================*/
#define GPT_START_SEC_CONFIG_DATA_UNSPECIFIED
#include "Gpt_MemMap.h"

Pit_Ip_InstanceConfigType PIT_0_InitConfig_PB =
{
    /** @brief PIT Standard Timer */
    (boolean)(TRUE), /* Enable standard timer */
#if(defined (PIT_IP_RTI_USED) && (PIT_IP_RTI_USED == STD_ON))
    (boolean)(FALSE), /* Disable real time interrupt timer */
#endif
    /** @brief PIT Debug Mode */
    (boolean)(FALSE) /* Disable Freeze Bit */
}; 

Pit_Ip_ChannelConfigType PIT_0_ChannelConfig_PB[1U] =
{
    /**@brief PitChannel_0 */ 
    {
        /** @brief PIT Channel Id */
        0U,
        #if (defined (PIT_IP_RTI_USED) && (PIT_IP_RTI_USED == STD_ON))
        /**< @brief errorReportCallBack */
        NULL_PTR,
        #endif
        /** @brief PIT Enable Interrupt */
        (boolean)(TRUE),
        /** @brief PIT callback name */
        &Gpt_ProcessCommonInterrupt,
        /** @brief PIT callbackparam  */
        (uint8)0U
    }
};
#define GPT_STOP_SEC_CONFIG_DATA_UNSPECIFIED
#include "Gpt_MemMap.h"

/*==================================================================================================
 *                                       LOCAL VARIABLES
 *================================================================================================*/

/*==================================================================================================
 *                          LOCAL TYPEDEFS (STRUCTURES, UNIONS, ENUMS)
 *================================================================================================*/

/*==================================================================================================
 *                                       LOCAL MACROS
 *================================================================================================*/

/*==================================================================================================
 *                                      LOCAL CONSTANTS
 *================================================================================================*/

/*==================================================================================================
 *                                       LOCAL FUNCTIONS
 *================================================================================================*/

/*==================================================================================================
 *                                   LOCAL FUNCTION PROTOTYPES
 *================================================================================================*/

#ifdef __cplusplus
}
#endif /* PIT_IP_PBCFG_C*/

/** @} */

