/*
*   (c) Copyright 2020 NXP
*
*   NXP Confidential. This software is owned or controlled by NXP and may only be used strictly
*   in accordance with the applicable license terms.  By expressly accepting
*   such terms or by downloading, installing, activating and/or otherwise using
*   the software, you are agreeing that you have read, and that you agree to
*   comply with and are bound by, such license terms.  If you do not agree to
*   be bound by the applicable license terms, then you may not retain,
*   install, activate or otherwise use the software.
*
*   This file contains sample code only. It is not part of the production code deliverables.
*/

#ifdef __cplusplus
extern "C" {
#endif


/*==================================================================================================
*                                        INCLUDE FILES
* 1) system and project includes
* 2) needed interfaces from external units
* 3) internal and external interfaces from this unit
==================================================================================================*/
#include "Mcu.h"
#include "Gpt.h"
#include "Dio.h"
#include "Platform.h"
#include "Siul2_Port_Ip.h"
#include "Siul2_Dio_Ip.h"
#include "stdio.h"
#include "retarget.h"
//#include "Uart.h"
//#include "string.h"
#include "check_example.h"
/*==================================================================================================
*                          LOCAL TYPEDEFS (STRUCTURES, UNIONS, ENUMS)
==================================================================================================*/
//#define WELCOME_MSG "This example is an simple echo using LPUART\r\n"
//#define UART_CHANNEL  0U

/*==================================================================================================
*                                       LOCAL MACROS
==================================================================================================*/
/*clock config mcu */
#define clockConfig &Mcu_aClockConfigPB[0]

/*==================================================================================================
*                                      LOCAL CONSTANTS
==================================================================================================*/


/*==================================================================================================
*                                      LOCAL VARIABLES
==================================================================================================*/


/*==================================================================================================
*                                      GLOBAL CONSTANTS
==================================================================================================*/


/*==================================================================================================
*                                      GLOBAL VARIABLES
==================================================================================================*/
volatile uint8 toggleLed = 0U;
volatile uint8 pinValue = STD_LOW;

/*==================================================================================================
*                                   LOCAL FUNCTION PROTOTYPES
==================================================================================================*/


/*==================================================================================================
*                                       LOCAL FUNCTIONS
==================================================================================================*/


/*==================================================================================================
*                                       GLOBAL FUNCTIONS
==================================================================================================*/

/**
* @brief        Gpt notification called by the configured channel periodically
* @details      Used to blink a led
*/
void Gpt_PitNotification(void)
{
    toggleLed = 1U;
    pinValue = (pinValue == STD_LOW ? STD_HIGH : STD_LOW);
}


/**
* @brief        Main function of the example
* @details      Initializez the used drivers and uses the Gpt
*               and Dio drivers to toggle a LED periodically
*/
int main(void) 
{
	/*Initial Pin */
	Siul2_Port_Ip_Init(NUM_OF_CONFIGURED_PINS0, g_pin_mux_InitConfigArr0);

	/* Initial Clock */
	Clock_Ip_Init(clockConfig);
    
    /* Initialize Platform driver */
    Platform_Init(NULL_PTR);
    
    /* Initialize the Gpt driver */
    Gpt_Init(NULL_PTR);
    
    /* Start the Gpt timer */
    Gpt_StartTimer(GptConf_GptChannelConfiguration_GptChannelConfiguration_0, 20000000U);
    
    /* Enable the Gpt notification to get the event for toggling the LED periodically */
    Gpt_EnableNotification(GptConf_GptChannelConfiguration_GptChannelConfiguration_0);

    /* Initializes an UART driver*/
	Console_SerialPort_Init();
	printf("This is a printf demo for S32K3.\r\n");
    while (1)
    {
        /* Toggle the LED when the Gpt notification is called */
        if (toggleLed == 1U)
        {
        	/*Led toggle - blink a led*/
        	//Siul2_Dio_Ip_TogglePins(LED_PORT, (1<<LED_PIN));
            Dio_WriteChannel(DioConf_DioChannel_Digital_Output_LED_D78,pinValue);
        	printf("This is a printf demo for S32K3.\r\n");
//            Uart_SyncSend(UART_CHANNEL, (const uint8 *)WELCOME_MSG, strlen(WELCOME_MSG));
            toggleLed = 0U;
        }
    }
    
    Gpt_StopTimer(GptConf_GptChannelConfiguration_GptChannelConfiguration_0);
    Exit_Example(TRUE);
    return (0U);
}


#ifdef __cplusplus
}
#endif

/** @} */
